@@ -1,3 +1,5 @@
+#undef USE_PANGO
+#undef USE_XFT
 #undef USE_MMX
 #undef OLD_PERL
 #undef ENABLE_NLS
@@ -5,16 +7,12 @@
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
-#undef USE_PYTHON
 #undef USE_DEBUG
 #undef USE_GNOME
 #undef USE_ZVT
-#undef USE_PERL
 #undef USE_PLUGIN
-#undef USE_GDK_PIXBUF
 #undef USE_XLIB
 #undef USE_PANEL
-#undef USE_MYGLIB
 #undef USE_TRANS
 #undef USE_HEBREW
 #undef USE_OPENSSL
