@@ -16,6 +16,7 @@ void list_free (GSList ** list);
 void list_loadconf (char *file, GSList ** list, char *defaultconf);
 int list_delentry (GSList ** list, char *name);
 void list_addentry (GSList ** list, char *cmd, char *name);
+int cmd_set (session *sess, char *tbuf, char *word[], char *word_eol[]);
 
 #define STRUCT_OFFSET_STR(type,field) \
 ( (unsigned int) (((char *) (&(((type *) NULL)->field)))- ((char *) NULL)) )
@@ -23,8 +24,8 @@ void list_addentry (GSList ** list, char *cmd, char *name);
 #define STRUCT_OFFSET_INT(type,field) \
 ( (unsigned int) (((int *) (&(((type *) NULL)->field)))- ((int *) NULL)) )
 
-#define PREFS_OFFSET(field) STRUCT_OFFSET_STR(struct xchatprefs, field)
-#define PREFS_OFFINT(field) STRUCT_OFFSET_INT(struct xchatprefs, field)
+#define P_OFFSET(field) STRUCT_OFFSET_STR(struct xchatprefs, field)
+#define P_OFFINT(field) STRUCT_OFFSET_INT(struct xchatprefs, field)
 
 struct prefs
 {
@@ -33,6 +34,6 @@ struct prefs
 	unsigned short type;
 };
 
-#define TYPE_STR 1
-#define TYPE_INT 2
-#define TYPE_BOOL 3
+#define TYPE_STR 0
+#define TYPE_INT 1
+#define TYPE_BOOL 2
