@@ -24,12 +24,13 @@
 #include "xchat.h"
 #include "cfgfiles.h"
 #include "util.h"
-#include "plugin.h"
 #include "modes.h"
 #include "outbound.h"
 #include "inbound.h"
 #include "dcc.h"
+#include "text.h"
 #include "ctcp.h"
+#include "server.h"
 #include "xchatc.h"
 
 
@@ -37,20 +38,12 @@ static void
 ctcp_reply (session *sess, char *tbuf, char *nick, char *word[],
 				char *word_eol[], char *conf)
 {
-	int oldc = prefs.perc_color;
-
 	conf = strdup (conf);
-	/* process %C %B etc, on the config only (not the inbound text) */
+	/* process %C %B etc */
 	check_special_chars (conf, TRUE);
 	auto_insert (tbuf, conf, word, word_eol, "", "", word_eol[5], "", "", nick);
 	free (conf);
-
-	/* must be disabled so the remote's text isn't interpreted with %xxx */
-	prefs.perc_color = 0;
-
-	handle_command (tbuf, sess, 0, 0);
-
-	prefs.perc_color = oldc;
+	handle_command (sess, tbuf, FALSE);
 }
 
 static int
@@ -84,16 +77,15 @@ ctcp_check (session *sess, char *tbuf, char *nick, char *word[],
 }
 
 void
-handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
+ctcp_handle (session *sess, char *outbuf, char *to, char *nick,
 				 char *msg, char *word[], char *word_eol[])
 {
-	char *po;
 	session *chansess;
 
 	if (!strncasecmp (msg, "VERSION", 7) && !prefs.hidever)
 	{
 		sprintf (outbuf, "NOTICE %s :\001VERSION xchat "VERSION" %s\001\r\n",
-					 nick, get_cpu_str ());
+					nick, get_cpu_str ());
 		tcp_send (sess->server, outbuf);
 	}
 
@@ -101,10 +93,7 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 	{
 		if (!strncasecmp (msg, "ACTION", 6))
 		{
-			po = strchr (msg + 7, '\001');
-			if (po)
-				po[0] = 0;
-			channel_action (sess, outbuf, to, nick, msg + 7, FALSE);
+			inbound_action (sess, outbuf, to, nick, msg + 7, FALSE);
 			return;
 		}
 		if (!strncasecmp (msg, "DCC", 3))
@@ -114,9 +103,6 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 		}
 		if (!strncasecmp (msg, "SOUND", 5))
 		{
-			po = strchr (word[5], '\001');
-			if (po)
-				po[0] = 0;
 			EMIT_SIGNAL (XP_TE_CTCPSND, sess->server->front_session, word[5],
 							 nick, NULL, NULL, 0);
 			sprintf (outbuf, "%s/%s", prefs.sounddir, word[5]);
@@ -130,22 +116,15 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 		}
 	}
 
-	po = strchr (msg, '\001');
-	if (po)
-		po[0] = 0;
 	if (!is_channel (sess->server, to))
 	{
 		EMIT_SIGNAL (XP_TE_CTCPGEN, sess->server->front_session, msg, nick,
 						 NULL, NULL, 0);
 	} else
 	{
-		chansess = find_channel (to, sess->server);
-		if (chansess)
-		{
-			EMIT_SIGNAL (XP_TE_CTCPGENC, chansess, msg, nick, to, NULL, 0);
-			return;
-		}
-		EMIT_SIGNAL (XP_TE_CTCPGENC, sess->server->front_session, msg, nick, to,
-						 NULL, 0);
+		chansess = find_session_from_channel (to, sess->server);
+		if (!chansess)
+			chansess = sess;
+		EMIT_SIGNAL (XP_TE_CTCPGENC, sess, msg, nick, to, NULL, 0);
 	}
 }
