@@ -36,18 +36,18 @@
 #include "inet.h"
 
 #include "xchat.h"
-#include "perlc.h"
 #include "util.h"
-#include "plugin.h"
 #include "fe.h"
 #include "outbound.h"
 #include "inbound.h"
 #include "network.h"
+#include "plugin.h"
+#include "server.h"
 #include "text.h"
 #include "xchatc.h"
 
 
-char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
+static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
 
 struct dccstat_info dccstat[] = {
 	{N_("Waiting"), 1 /*black */ },
@@ -59,6 +59,7 @@ struct dccstat_info dccstat[] = {
 };
 
 static struct DCC *new_dcc (void);
+static void dcc_close (struct DCC *dcc, int dccstat, int destroy);
 
 
 static void
@@ -182,7 +183,7 @@ update_dcc_window (int type)
 	}
 }
 
-void
+static void
 dcc_close (struct DCC *dcc, int dccstat, int destroy)
 {
 	char type = dcc->type;
@@ -244,6 +245,39 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 	}
 }
 
+void
+dcc_abort (session *sess, struct DCC *dcc)
+{
+	if (dcc)
+	{
+		switch (dcc->dccstat)
+		{
+		case STAT_QUEUED:
+		case STAT_CONNECTING:
+		case STAT_ACTIVE:
+			dcc_close (dcc, STAT_ABORTED, FALSE);
+			switch (dcc->type)
+			{
+			case TYPE_CHATSEND:
+			case TYPE_CHATRECV:
+				EMIT_SIGNAL (XP_TE_DCCCHATABORT, sess, dcc->nick, NULL, NULL,
+								 NULL, 0);
+				break;
+			case TYPE_SEND:
+				EMIT_SIGNAL (XP_TE_DCCSENDABORT, sess, file_part (dcc->file),
+								 dcc->nick, NULL, NULL, 0);
+				break;
+			case TYPE_RECV:
+				EMIT_SIGNAL (XP_TE_DCCRECVABORT, sess, dcc->file,
+								 dcc->nick, NULL, NULL, 0);
+			}
+			break;
+		default:
+			dcc_close (dcc, 0, TRUE);
+		}
+	}
+}
+
 void
 dcc_notify_kill (struct server *serv)
 {
@@ -290,7 +324,7 @@ dcc_write_chat (char *nick, char *text)
 			{
 				return 0;
 			}
-			strcpy (tbuf, text);
+			memcpy (tbuf, text, len + 1);	/* copy text with the NULL */
 			user2serv (tbuf);
 			send (dcc->sok, tbuf, len, 0);
 			if (tbuf != sbuf)
@@ -317,55 +351,50 @@ dcc_write_chat (char *nick, char *text)
 static int
 dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 {
-#ifdef USE_PERL
-	int skip;
-	char *host_n_nick_n_message;
-	GSList *list = dcc_list;
 	session *sess;
+	char *word[PDIWORDS];
+	char *po;
+	int ret, i;
 
-	host_n_nick_n_message = malloc (strlen (dcc->nick) + strlen (line) + 26);
-
-	sprintf (host_n_nick_n_message, "%s %d %s: %s",
-				net_ip (dcc->addr), dcc->port, dcc->nick, line);
-
-	sess = find_dialog (dcc->serv, dcc->nick);
+	sess = find_session_from_channel (dcc->nick, dcc->serv);
 	if (!sess)
 		sess = dcc->serv->front_session;
-	skip = perl_dcc_chat (sess, host_n_nick_n_message);
-	free (host_n_nick_n_message);
 
-	/* Perl code may have closed the chat. */
-	while (list) 
-	{
-		if (((struct DCC *)list->data) == dcc) break;
-		list = list->next;
-	}
-	if (list == 0)
+	sprintf (tbuf, "%d", dcc->port);
+
+	word[0] = "DCC Chat Text";
+	word[1] = net_ip (dcc->addr);
+	word[2] = tbuf;
+	word[3] = dcc->nick;
+	word[4] = line;
+	for (i = 5; i < PDIWORDS; i++)
+		word[i] = "\000";
+
+	ret = plugin_emit_server (sess, word[0], word, word);
+
+	/* did the plugin close it? */
+	if (!g_slist_find (dcc_list, dcc))
 		return 1;
 
-	if (skip)
+	/* did the plugin eat the event? */
+	if (ret)
 		return 0;
-#endif
+
 	fe_checkurl (line);
 
 	if (line[0] == 1 && !strncasecmp (line + 1, "ACTION", 6))
 	{
-		session *sess;
-		char *po = strchr (line + 8, '\001');
+		po = strchr (line + 8, '\001');
 		if (po)
 			po[0] = 0;
-		sess = find_dialog (dcc->serv, dcc->nick);
-		if (!sess)
-			sess = dcc->serv->front_session;
-		channel_action (sess, tbuf, dcc->serv->nick, dcc->nick,
-							 line + 8, FALSE);
+		inbound_action (sess, tbuf, dcc->serv->nick, dcc->nick, line + 8, FALSE);
 	} else
 	{
 #ifdef USE_TRANS
 		if (prefs.use_trans)
 			serv2user (line);
 #endif
-		private_msg (dcc->serv, tbuf, dcc->nick, "", line);
+		inbound_privmsg (dcc->serv, tbuf, dcc->nick, "", line);
 	}
 	return 0;
 }
@@ -519,11 +548,8 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	char host[128];
 	struct sockaddr_in addr;
 
-	if (dcc->iotag)
-	{
-		fe_input_remove (dcc->iotag);
-		dcc->iotag = 0;
-	}
+	fe_input_remove (dcc->iotag);
+	dcc->iotag = 0;
 
 	memset (&addr, 0, sizeof (addr));
 	addr.sin_port = htons (dcc->port);
@@ -567,57 +593,26 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	dcc->lasttime = dcc->starttime;
 
 	snprintf (host, sizeof host, "%s:%d", net_ip (dcc->addr), dcc->port);
-	EMIT_SIGNAL (XP_TE_DCCCON, dcc->serv->front_session,
-					 dcctypes[(int) dcc->type], dcc->nick, host, "to", 0);
+	EMIT_SIGNAL (XP_TE_DCCCONRECV, dcc->serv->front_session,
+					 dcc->nick, host, dcc->file, NULL, 0);
 
 	return TRUE;
 }
 
-static int dcc_listen_init (struct DCC *, struct session *);
-
 static void
 dcc_connect (struct DCC *dcc)
 {
-	int ret;
-	char tbuf[400];
-
 	if (dcc->dccstat == STAT_CONNECTING)
 		return;
 	dcc->dccstat = STAT_CONNECTING;
-
-	if (dcc->pasvid)
+	dcc->sok = dcc_connect_sok (dcc);
+	if (dcc->sok == -1)
 	{
-		/* accepted a passive dcc send */
-		ret = dcc_listen_init (dcc, dcc->serv->front_session);
-		if (!ret)
-		{
-			dcc_close (dcc, STAT_FAILED, FALSE);
-			return;
-		}
-		if (dcc->type == TYPE_RECV)
-			snprintf (tbuf, sizeof (tbuf), strchr (dcc->file, ' ') ?
-					"PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %u %d\001\r\n" :
-					"PRIVMSG %s :\001DCC SEND %s %lu %d %u %d\001\r\n",
-					dcc->nick, dcc->file, dcc->addr, dcc->port, dcc->size,
-					dcc->pasvid);
-		else
-			snprintf (tbuf, sizeof (tbuf),
-						 "PRIVMSG %s :\001DCC CHAT chat %lu %d %d\001\r\n",
-						 dcc->nick, dcc->addr, dcc->port, dcc->pasvid);
-		tcp_send (dcc->serv, tbuf);
+		dcc->dccstat = STAT_FAILED;
+		update_dcc_window (dcc->type);
+		return;
 	}
-	else
-	{
-		dcc->sok = dcc_connect_sok (dcc);
-		if (dcc->sok == -1)
-		{
-			dcc->dccstat = STAT_FAILED;
-			update_dcc_window (dcc->type);
-			return;
-		}
-		dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
-	}
-
+	dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
 	if (dcc->type == TYPE_RECV)
 		fe_dcc_update_recv (dcc);
 	else
@@ -649,10 +644,10 @@ dcc_send_data (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	if (sent < 0 && !(would_block ()))
 	{
 abortit:
+		free (buf);
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
-		free (buf);
 		return TRUE;
 	}
 	if (sent > 0)
@@ -750,13 +745,12 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	set_nonblocking (sok);
 	dcc->sok = sok;
 	dcc->addr = ntohl (CAddr.sin_addr.s_addr);
-
-	if (dcc->pasvid)
-		return dcc_connect_finished (NULL, 0, dcc);
-
 	dcc->dccstat = STAT_ACTIVE;
 	dcc->lasttime = dcc->starttime = time (0);
 	dcc->fastsend = prefs.fastdccsend;
+
+	snprintf (host, sizeof (host), "%s:%d", net_ip (dcc->addr), dcc->port);
+
 	switch (dcc->type)
 	{
 	case TYPE_SEND:
@@ -764,21 +758,21 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 			dcc->wiotag = fe_input_add (sok, 0, 1, 0, dcc_send_data, dcc);
 		dcc->iotag = fe_input_add (sok, 1, 0, 1, dcc_read_ack, dcc);
 		dcc_send_data (NULL, 0, (gpointer)dcc);
+		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
+						 dcc->nick, host, dcc->file, NULL, 0);
 		break;
 
 	case TYPE_CHATSEND:
 		dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
+		EMIT_SIGNAL (XP_TE_DCCCONCHAT, dcc->serv->front_session,
+						 dcc->nick, host, NULL, NULL, 0);
 		break;
 	}
 
 	update_dcc_window (dcc->type);
 
-	snprintf (host, sizeof host, "%s:%d", net_ip (dcc->addr), dcc->port);
-	EMIT_SIGNAL (XP_TE_DCCCON, dcc->serv->front_session,
-					 dcctypes[(int) dcc->type], dcc->nick, host, "from", 0);
-
 	return TRUE;
 }
 
@@ -880,7 +874,7 @@ dcc_send_wild (char *file)
 /* tbuf is at least 400 bytes */
 
 void
-dcc_send (struct session *sess, char *tbuf, char *to, char *file)
+dcc_send (session *sess, char *tbuf, char *to, char *file)
 {
 	struct stat st;
 	struct DCC *dcc;
@@ -950,23 +944,13 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 						else
 							fe_dcc_update_send_win ();
 						
-						if (havespaces)
-						{
-							snprintf (tbuf, 400,
-									 "PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %u\001\r\n",
-									 to, file_part (dcc->file), dcc->addr, dcc->port,
-									 dcc->size);
-						}
-						else
-						{
-							snprintf (tbuf, 400,
-									 "PRIVMSG %s :\001DCC SEND %s %lu %d %u\001\r\n",
-									 to, file_part (dcc->file), dcc->addr, dcc->port,
-									 dcc->size);
-						}
-						tcp_send (sess->server, tbuf);
+						tcp_sendf (sess->server, (havespaces) ?
+								"PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %d\001\r\n" :
+								"PRIVMSG %s :\001DCC SEND %s %lu %d %d\001\r\n",
+								 to, file_part (dcc->file), dcc->addr, dcc->port,
+								 dcc->size);
 						EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
-										 to, NULL, NULL, 0);
+										 to, dcc->file, NULL, 0);
 					} else
 					{
 						dcc_close (dcc, 0, TRUE);
@@ -976,8 +960,7 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 			}
 		}
 	}
-	snprintf (tbuf, 400, _("Cannot access %s\n"), dcc->file);
-	PrintText (sess, tbuf);
+	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
 	dcc_close (dcc, 0, TRUE);
 }
 
@@ -1005,7 +988,7 @@ find_dcc (char *nick, char *file, int type)
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
-		if (nick == NULL || !strcasecmp (nick, dcc->nick))
+		if (nick == NULL || !rfc_casecmp (nick, dcc->nick))
 		{
 			if (type == -1 || dcc->type == type)
 			{
@@ -1035,7 +1018,7 @@ dcc_change_nick (struct server *serv, char *oldnick, char *newnick)
 		dcc = (struct DCC *) list->data;
 		if (dcc->serv == serv)
 		{
-			if (!strcasecmp (dcc->nick, oldnick))
+			if (!rfc_casecmp (dcc->nick, oldnick))
 			{
 				if (dcc->nick)
 					free (dcc->nick);
@@ -1075,7 +1058,7 @@ dcc_get_nick (struct session *sess, char *nick)
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
-		if (!strcasecmp (nick, dcc->nick))
+		if (!rfc_casecmp (nick, dcc->nick))
 		{
 			if (dcc->dccstat == STAT_QUEUED && dcc->type == TYPE_RECV)
 			{
@@ -1108,7 +1091,6 @@ new_dcc (void)
 void
 dcc_chat (struct session *sess, char *nick)
 {
-	char outbuf[400];
 	struct DCC *dcc;
 
 	dcc = find_dcc (nick, "", TYPE_CHATSEND);
@@ -1156,10 +1138,9 @@ dcc_chat (struct session *sess, char *nick)
 			fe_dcc_open_chat_win (TRUE);
 		else
 			fe_dcc_update_chat_win ();
-		snprintf (outbuf, sizeof (outbuf),
-					 "PRIVMSG %s :\001DCC CHAT chat %lu %d\001\r\n", nick,
-					 dcc->addr, dcc->port);
-		tcp_send (dcc->serv, outbuf);
+		tcp_sendf (dcc->serv,
+					  "PRIVMSG %s :\001DCC CHAT chat %lu %d\001\r\n", nick,
+					  dcc->addr, dcc->port);
 		EMIT_SIGNAL (XP_TE_DCCCHATOFFERING, sess, nick, NULL, NULL, NULL, 0);
 	} else
 	{
@@ -1170,26 +1151,19 @@ dcc_chat (struct session *sess, char *nick)
 static void
 dcc_malformed (struct session *sess, char *nick, char *data)
 {
-	EMIT_SIGNAL (XP_TE_MALFORMED_FROM, sess, nick, NULL, NULL, NULL, 0);
-	EMIT_SIGNAL (XP_TE_MALFORMED_PACKET, sess, data, NULL, NULL, NULL, 0);
+	EMIT_SIGNAL (XP_TE_MALFORMED, sess, nick, data, NULL, NULL, 0);
 }
 
 void
 dcc_resume (struct DCC *dcc)
 {
-	char tbuf[410];
-
 	if (dcc->dccstat == STAT_QUEUED && dcc->resumable)
 	{
-		snprintf (tbuf, sizeof (tbuf) - 10, strchr (dcc->file, ' ') ?
-					 "PRIVMSG %s :\001DCC RESUME \"%s\" %d %u\001\r\n" :
-					 "PRIVMSG %s :\001DCC RESUME %s %d %u\001\r\n",
-					 dcc->nick, dcc->file, dcc->port, dcc->resumable);
-
-		if (dcc->pasvid)
-			sprintf (tbuf + strlen (tbuf) - 3, " %d\001\r\n", dcc->pasvid);
-
-		tcp_send (dcc->serv, tbuf);
+		/* filename contains spaces? Quote them! */
+		tcp_sendf (dcc->serv, (strchr (dcc->file, ' ')) ?
+					  "PRIVMSG %s :\001DCC RESUME \"%s\" %d %d\001\r\n" :
+					  "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n",
+					  dcc->nick, dcc->file, dcc->port, dcc->resumable);
 	}
 }
 
@@ -1199,19 +1173,15 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 {
 	struct DCC *dcc;
 	char *type = word[5];
-	int port, pasvid = 0;
-	unsigned long size, addr;
+	int port, size;
+	unsigned long addr;
 
 	if (!strcasecmp (type, "CHAT"))
 	{
 		port = atoi (word[8]);
 		sscanf (word[7], "%lu", &addr);
 
-		if (port == 0)
-			pasvid = atoi (word[9]);
-
-		if (!addr || (port < 1024 && port != 0)
-			|| port > 0xffff || (port == 0 && pasvid == 0))
+		if (!addr || port < 1024)
 		{
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
@@ -1232,7 +1202,6 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->dccstat = STAT_QUEUED;
 			dcc->addr = addr;
 			dcc->port = port;
-			dcc->pasvid = pasvid;
 			dcc->nick = strdup (nick);
 			dcc->starttime = time (0);
 
@@ -1255,18 +1224,17 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			sscanf (word[8], "%lu", &size);
-			dcc->resumable = size;
+			dcc->resumable = atoi (word[8]);
 			if (dcc->resumable < dcc->size)
 			{
 				dcc->pos = dcc->resumable;
 				dcc->ack = dcc->resumable;
 				lseek (dcc->fp, dcc->pos, SEEK_SET);
-				snprintf (outbuf, 400, strchr (file_part (dcc->file), ' ') ?
-							 "PRIVMSG %s :\001DCC ACCEPT \"%s\" %d %u\001\r\n" :
-							 "PRIVMSG %s :\001DCC ACCEPT %s %d %u\001\r\n",
-							 dcc->nick, file_part (dcc->file), port, dcc->resumable);
-				tcp_send (dcc->serv, outbuf);
+				/* filename contains spaces? */
+				tcp_sendf (dcc->serv, (strchr (file_part (dcc->file), ' ')) ?
+							"PRIVMSG %s :\001DCC ACCEPT \"%s\" %d %d\001\r\n" :
+							"PRIVMSG %s :\001DCC ACCEPT %s %d %d\001\r\n",
+							dcc->nick, file_part (dcc->file), port, dcc->resumable);
 			}
 			sprintf (outbuf, "%d", dcc->pos);
 			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
@@ -1281,22 +1249,18 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 		if (dcc && dcc->dccstat == STAT_QUEUED)
 		{
 			dcc_connect (dcc);
-			return;
 		}
+		return;
 	}
 	if (!strcasecmp (type, "SEND"))
 	{
 		char *file = file_part (word[6]);
 		port = atoi (word[8]);
+		size = atoi (word[9]);
 
 		sscanf (word[7], "%lu", &addr);
-		sscanf (word[9], "%lu", &size);
 
-		if (port == 0)
-			pasvid = atoi (word[10]);
-
-		if (!addr || !size || (port < 1024 && port != 0)
-			|| port > 0xffff || (port == 0 && pasvid == 0))
+		if (!addr || !size || port < 1024)
 		{
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
@@ -1317,18 +1281,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 				strcat (dcc->destfile, "/");
 			if (prefs.dccwithnick)
 			{
-#ifdef WIN32
-				char *t = strlen (dcc->destfile) + dcc->destfile;
-				strcpy (t, nick);
-				while (*t)
-				{
-					if (*t == '\\' || *t == '|')
-						*t = '_';
-					t++;
-				}
-#else
 				strcat (dcc->destfile, nick);
-#endif
 				strcat (dcc->destfile, ".");
 			}
 			strcat (dcc->destfile, file);
@@ -1346,7 +1299,6 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->dccstat = STAT_QUEUED;
 			dcc->addr = addr;
 			dcc->port = port;
-			dcc->pasvid = pasvid;
 			dcc->size = size;
 			dcc->nick = strdup (nick);
 			if (prefs.autodccsend)
@@ -1381,7 +1333,7 @@ dontresume:
 			else
 				fe_dcc_update_recv_win ();
 		}
-		sprintf (outbuf, "%lu", size);
+		sprintf (outbuf, "%d", size);
 		EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
 						 file, outbuf, NULL, 0);
 	} else
@@ -1401,11 +1353,10 @@ dcc_show_list (struct session *sess, char *outbuf)
 	{
 		dcc = (struct DCC *) list->data;
 		i++;
-		snprintf (outbuf, 255, " %-5.5s %-10.10s %-7.7s %-7u %-7u %s\n",
+		PrintTextf (sess, " %s  %-10.10s %-7.7s %-7d %-7d %s\n",
 					 dcctypes[(int) dcc->type], dcc->nick,
 					 _(dccstat[(int) dcc->dccstat].name), dcc->size, dcc->pos,
 					 file_part (dcc->file));
-		PrintText (sess, outbuf);
 		list = list->next;
 	}
 	if (!i)
