@@ -24,12 +24,11 @@ struct DCC
 	int iotag;
 	int wiotag;
 	int port;
-	int pasvid;
 	int cps;
-	unsigned int size;
-	unsigned int resumable;
-	unsigned int ack;
-	unsigned int pos;
+	int size;
+	int resumable;
+	int ack;
+	int pos;
 	time_t starttime;
 	time_t offertime;
 	time_t lasttime;
@@ -55,21 +54,19 @@ struct dccstat_info
 };
 
 extern struct dccstat_info dccstat[];
-extern char *dcctypes[];
 
+void dcc_abort (session *sess, struct DCC *dcc);
 void dcc_get (struct DCC *dcc);
 void dcc_resume (struct DCC *dcc);
 void dcc_check_timeouts (void);
 void dcc_change_nick (server *serv, char *oldnick, char *newnick);
-void dcc_send_filereq (session *sess, char *nick);
-void dcc_close (struct DCC *dcc, int stat, int destroy);
 void dcc_notify_kill (struct server *serv);
 struct DCC *dcc_write_chat (char *nick, char *text);
 void dcc_send (struct session *sess, char *tbuf, char *to, char *file);
 struct DCC *find_dcc (char *nick, char *file, int type);
 void dcc_get_nick (struct session *sess, char *nick);
 void dcc_chat (session *sess, char *nick);
-void handle_dcc (session *sess, char *outbuf, char *nick, char *word[],
+void handle_dcc (session *sess, char *outbudcc_chatf, char *nick, char *word[],
 					  char *word_eol[]);
 void dcc_show_list (session *sess, char *outbuf);
 void open_dcc_recv_window (void);
