@@ -31,10 +31,10 @@ void fe_notify_update (char *name);
 void fe_text_clear (struct session *sess);
 void fe_close_window (struct session *sess);
 void fe_progressbar_start (struct session *sess);
-void fe_progressbar_end (struct session *sess);
+void fe_progressbar_end (struct server *serv);
 void fe_print_text (struct session *sess, char *text);
-void fe_userlist_insert (struct session *sess, struct User *newuser, int row);
-void fe_userlist_remove (struct session *sess, struct User *user);
+void fe_userlist_insert (struct session *sess, struct User *newuser, int row, int sel);
+int fe_userlist_remove (struct session *sess, struct User *user);
 void fe_userlist_move (struct session *sess, struct User *user, int new_row);
 void fe_userlist_numbers (struct session *sess);
 void fe_userlist_clear (struct session *sess);
@@ -61,8 +61,6 @@ void fe_set_nick (struct server *serv, char *newnick);
 void fe_change_nick (struct server *serv, char *nick, char *newnick);
 void fe_ignore_update (int level);
 void fe_beep (void);
-int fe_is_confmode (struct session *);
-int fe_is_beep (struct session *);
 void fe_lastlog (session *sess, session *lastlog_sess, char *sstr);
 void fe_set_lag (server *serv, int lag);
 void fe_set_throttle (server *serv);
