@@ -23,10 +23,73 @@
 #include "xchat.h"
 #include "xchatc.h"
 #include "modes.h"
-#include "plugin.h"
+#include "server.h"
+#include "text.h"
 #include "fe.h"
 
 
+/* word[] - list of nicks.
+   wpos   - index into word[]. Where nicks really start.
+   end    - index into word[]. Last entry plus one.
+   sign   - a char, e.g. '+' or '-'
+   mode   - a mode, e.g. 'o' or 'v'	*/
+void
+send_channel_modes (session *sess, char *tbuf, char *word[], int wpos,
+						  int end, char sign, char mode)
+{
+	int usable_modes, orig_len, len, wlen, i;
+
+	/* sanity check. IRC RFC says three per line. */
+	if (sess->server->modes_per_line < 3)
+		sess->server->modes_per_line = 3;
+
+	while (wpos < end)
+	{
+		orig_len = len = sprintf (tbuf, "MODE %s ", sess->channel);
+
+		/* we'll need this many modechars too */
+		len += sess->server->modes_per_line;
+
+		/* how many can we fit? */
+		for (i = 0; i < sess->server->modes_per_line; i++)
+		{
+			/* no more nicks left? */
+			if (wpos + i >= end)
+				break;
+			wlen = strlen (word[wpos + i]) + 1;
+			/* 512 is IRC max, minus 2 for \r\n, 1 for sign, 1 for OBO error */
+			if (wlen + len > 508)
+				break;
+			len += wlen; /* length of our whole string so far */
+		}
+		if (i < 1)
+			return;
+		usable_modes = i;	/* this is how many we'll send on this line */
+
+		/* add the +/-modemodemodemode */
+		len = orig_len;
+		tbuf[len] = sign;
+		len++;
+		for (i = 0; i < usable_modes; i++)
+		{
+			tbuf[len] = mode;
+			len++;
+		}
+		tbuf[len] = 0;	/* null terminate for the strcat() to work */
+
+		/* add all the nicknames */
+		for (i = 0; i < usable_modes; i++)
+		{
+			strcat (tbuf, " ");
+			strcat (tbuf, word[wpos + i]);
+		}
+		strcat (tbuf, "\r\n");
+		tcp_send (sess->server, tbuf);
+
+		wpos += usable_modes;
+	}
+}
+
 /* does 'chan' have a valid prefix? e.g. # or & */
 
 int
@@ -135,7 +198,7 @@ mode_access (server * serv, char mode, char *prefix)
 		pos++;
 	}
 
-	*prefix = ' ';
+	*prefix = 0;
 
 	return -1;
 }
@@ -143,13 +206,10 @@ mode_access (server * serv, char mode, char *prefix)
 static int
 mode_timeout_cb (session *sess)
 {
-	char outbuf[512];
-
 	if (is_session (sess))
 	{
 		sess->mode_timeout_tag = 0;
-		sprintf (outbuf, "MODE %s\r\n", sess->channel);
-		tcp_send (sess->server, outbuf);
+		tcp_sendf (sess->server, "MODE %s\r\n", sess->channel);
 		sess->ignore_mode = TRUE;
 		sess->ignore_date = TRUE;
 	}
@@ -182,7 +242,7 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 	outbuf[2] = mode;
 	outbuf[3] = 0;
 
-	sess = find_channel (chan, serv);
+	sess = find_session_from_channel (chan, serv);
 	if (!sess || !is_channel (serv, chan))
 	{
 		/* got modes for a chan we're not in! probably nickmode +isw etc */
@@ -207,8 +267,6 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 		switch (mode)
 		{
 		case 'k':
-			if (EMIT_SIGNAL (XP_CHANSETKEY, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			safe_strcpy (sess->channelkey, arg, sizeof (sess->channelkey));
 			fe_update_channel_key (sess);
 			fe_update_mode_buttons (sess, mode, sign);
@@ -216,8 +274,6 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 				EMIT_SIGNAL (XP_TE_CHANSETKEY, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'l':
-			if (EMIT_SIGNAL (XP_CHANSETLIMIT, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			sess->limit = atoi (arg);
 			fe_update_channel_limit (sess);
 			fe_update_mode_buttons (sess, mode, sign);
@@ -225,38 +281,26 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 				EMIT_SIGNAL (XP_TE_CHANSETLIMIT, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'o':
-			if (EMIT_SIGNAL (XP_CHANOP, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANOP, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'h':
-			if (EMIT_SIGNAL (XP_CHANHOP, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANHOP, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'v':
-			if (EMIT_SIGNAL (XP_CHANVOICE, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANVOICE, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'b':
-			if (EMIT_SIGNAL (XP_CHANBAN, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANBAN, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'e':
-			if (EMIT_SIGNAL (XP_CHANEXEMPT, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANEXEMPT, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'I':
-			if (EMIT_SIGNAL (XP_CHANINVITE, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANINVITE, sess, nick, arg, NULL, NULL, 0);
 			return;
@@ -266,8 +310,6 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 		switch (mode)
 		{
 		case 'k':
-			if (EMIT_SIGNAL (XP_CHANRMKEY, sess, chan, nick, NULL, NULL, 0) == 1)
-				return;
 			sess->channelkey[0] = 0;
 			fe_update_channel_key (sess);
 			fe_update_mode_buttons (sess, mode, sign);
@@ -275,8 +317,6 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 				EMIT_SIGNAL (XP_TE_CHANRMKEY, sess, nick, NULL, NULL, NULL, 0);
 			return;
 		case 'l':
-			if (EMIT_SIGNAL (XP_CHANRMLIMIT, sess, chan, nick, NULL, NULL, 0) == 1)
-				return;
 			sess->limit = 0;
 			fe_update_channel_limit (sess);
 			fe_update_mode_buttons (sess, mode, sign);
@@ -284,38 +324,26 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 				EMIT_SIGNAL (XP_TE_CHANRMLIMIT, sess, nick, NULL, NULL, NULL, 0);
 			return;
 		case 'o':
-			if (EMIT_SIGNAL (XP_CHANDEOP, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANDEOP, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'h':
-			if (EMIT_SIGNAL (XP_CHANDEHOP, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANDEHOP, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'v':
-			if (EMIT_SIGNAL (XP_CHANDEVOICE, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANDEVOICE, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'b':
-			if (EMIT_SIGNAL (XP_CHANUNBAN, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANUNBAN, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'e':
-			if (EMIT_SIGNAL (XP_CHANRMEXEMPT, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANRMEXEMPT, sess, nick, arg, NULL, NULL, 0);
 			return;
 		case 'I':
-			if (EMIT_SIGNAL (XP_CHANRMINVITE, sess, chan, nick, arg, NULL, 0) == 1)
-				return;
 			if (!quiet)
 				EMIT_SIGNAL (XP_TE_CHANRMINVITE, sess, nick, arg, NULL, NULL, 0);
 			return;
@@ -408,7 +436,7 @@ handle_mode (server * serv, char *outbuf, char *word[], char *word_eol[],
 	if (*modes == 0)
 		return;	/* beyondirc's blank modes */
 
-	sess = find_channel (chan, serv);
+	sess = find_session_from_channel (chan, serv);
 	if (!sess)
 		sess = serv->front_session;
 	/* remove trailing space */
@@ -421,7 +449,8 @@ handle_mode (server * serv, char *outbuf, char *word[], char *word_eol[],
 
 	if (numeric_324)
 	{
-		free (sess->current_modes);
+		if (sess->current_modes)
+			free (sess->current_modes);
 		sess->current_modes = strdup (word_eol[offset+1]);
 		fe_set_title (sess);
 	}
@@ -483,7 +512,7 @@ handle_mode (server * serv, char *outbuf, char *word[], char *word_eol[],
 /* handle the 005 numeric */
 
 void
-handle_005 (server * serv, char *word[])
+inbound_005 (server * serv, char *word[])
 {
 	int w;
 	char *pre;
@@ -524,11 +553,6 @@ handle_005 (server * serv, char *word[])
 		} else if (strncmp (word[w], "WATCH=", 6) == 0)
 		{
 			serv->supports_watch = TRUE;
-		} else if (strncmp (word[w], "NETWORK=", 8) == 0)
-		{
-			if (serv->network)
-				free (serv->network);
-			serv->network = strdup (word[w] + 8);
 		}
 
 		w++;
