@@ -32,8 +32,8 @@
 #include "xchat.h"
 #include "notify.h"
 #include "cfgfiles.h"
-#include "plugin.h"
 #include "fe.h"
+#include "server.h"
 #include "text.h"
 #include "util.h"
 #include "xchatc.h"
@@ -128,7 +128,7 @@ notify_find (server *serv, char *nick)
 			continue;
 		}
 
-		if (!strcasecmp (notify->name, nick))
+		if (!rfc_casecmp (notify->name, nick))
 			return servnot;
 
 		list = list->next;
@@ -144,8 +144,6 @@ notify_announce_offline (server * serv, struct notify_per_server *servnot,
 	session *sess;
 
 	sess = serv->front_session;
-	if (prefs.show_notify_in_front_session && menu_sess)
-		sess = menu_sess;
 
 	servnot->ison = FALSE;
 	servnot->lastoff = time (0);
@@ -160,12 +158,9 @@ static void
 notify_announce_online (server * serv, struct notify_per_server *servnot,
 								char *nick)
 {
-	char buf[256];
 	session *sess;
 
 	sess = serv->front_session;
-	if (prefs.show_notify_in_front_session && menu_sess)
-		sess = menu_sess;
 
 	servnot->lastseen = time (0);
 	if (servnot->ison)
@@ -179,10 +174,7 @@ notify_announce_online (server * serv, struct notify_per_server *servnot,
 	fe_notify_update (0);
 
 	if (prefs.whois_on_notifyonline)
-	{
-		snprintf (buf, sizeof (buf), "WHOIS %s\r\n", nick);
-		tcp_send (serv, buf);
-	}
+		tcp_sendf (serv, "WHOIS %s\r\n", nick);
 }
 
 /* handles numeric 601 */
@@ -314,7 +306,7 @@ notify_markonline (server *serv, char *word[])
 		seen = FALSE;
 		while (*word[i])
 		{
-			if (!strcasecmp (notify->name, word[i]))
+			if (!rfc_casecmp (notify->name, word[i]))
 			{
 				seen = TRUE;
 				notify_announce_online (serv, servnot, notify->name);
@@ -427,7 +419,7 @@ notify_deluser (char *name)
 	while (list)
 	{
 		notify = (struct notify *) list->data;
-		if (!strcasecmp (notify->name, name))
+		if (!rfc_casecmp (notify->name, name))
 		{
 			fe_notify_update (notify->name);
 			/* Remove the records for each server */
@@ -477,7 +469,7 @@ notify_isnotify (struct session *sess, char *name)
 	while (list)
 	{
 		notify = (struct notify *) list->data;
-		if (!strcasecmp (notify->name, name))
+		if (!rfc_casecmp (notify->name, name))
 		{
 			servnot = notify_find_server_entry (notify, sess->server);
 			if (servnot && servnot->ison)
