@@ -35,7 +35,6 @@
 #endif
 
 #include "xchat.h"
-#include "plugin.h"
 #include "fe.h"
 #include "cfgfiles.h"
 #include "network.h"
@@ -67,10 +66,213 @@ extern SSL_CTX *ctx;				  /* xchat.c */
 static struct session *g_sess = NULL;
 #endif
 
-static void server_stopconnecting (server * serv);
-static gboolean
-read_data (GIOChannel *source, GIOCondition condition, server *serv);
 
+static int
+tcp_send_real_real (server *serv, char *buf, int len)
+{
+	int ret = 0;
+	char *locale = NULL;
+
+	if (!serv->use_utf8)
+		locale = g_locale_from_utf8 (buf, len, NULL, &len, NULL);
+	if (locale)
+	{
+#ifdef USE_OPENSSL
+		if (!serv->ssl)
+			ret = send (serv->sok, locale, len, 0);
+		else
+			ret = _SSL_send (serv->ssl, locale, len);
+#else
+		ret = send (serv->sok, locale, len, 0);
+#endif
+		g_free (locale);
+	} else
+	{
+#ifdef USE_OPENSSL
+		if (!serv->ssl)
+			ret = send (serv->sok, buf, len, 0);
+		else
+			ret = _SSL_send (serv->ssl, buf, len);
+#else
+		ret = send (serv->sok, buf, len, 0);
+#endif
+	}
+
+	return ret;
+}
+
+/* actually send to the socket. This might do a character translation or
+   send via SSL */
+
+static int
+tcp_send_real (server *serv, char *buf, int len)
+{
+	int ret;
+	unsigned char *tbuf = buf;
+#ifdef USE_TRANS
+#define TRANS_STAT_BUF_LEN 1024
+	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
+
+	if (prefs.use_trans)
+	{
+		if (len >= TRANS_STAT_BUF_LEN)
+			tbuf = malloc (len + 1);
+		else
+			tbuf = sbuf;
+		if (!tbuf)
+			return -1;
+		memcpy (tbuf, buf, len + 1);
+		user2serv (tbuf);
+	}
+#endif
+
+	fe_add_rawlog (serv, tbuf, TRUE);
+	ret = tcp_send_real_real (serv, tbuf, len);
+
+#ifdef USE_TRANS
+	if (prefs.use_trans)
+	{
+		if (tbuf != sbuf)
+			free (tbuf);
+	}
+#endif
+
+	return ret;
+}
+
+/* new throttling system, uses the same method as the Undernet
+   ircu2.10 server; under test, a 200-line paste didn't flood
+   off the client */
+
+static int
+tcp_send_queue (server *serv)
+{
+	char *buf, *p;
+	int len, i, pri;
+	GSList *list;
+	time_t now = time (0);
+
+	/* did the server close since the timeout was added? */
+	if (!is_server (serv))
+		return 0;
+
+	/* first try priority 1 entries */
+	pri = 1;
+	while (pri >= 0)
+	{
+		list = serv->outbound_queue;
+		while (list)
+		{
+			buf = (char *) list->data;
+			if (buf[0] == pri)
+			{
+				buf++;	/* skip the priority byte */
+				len = strlen (buf);
+
+				if (serv->next_send < now)
+					serv->next_send = now;
+				if (serv->next_send - now >= 10)
+					return 1;				  /* don't remove the timeout handler */
+
+				for (p = buf, i = len; i && *p != ' '; p++, i--);
+				serv->next_send += (2 + i / 120);
+				serv->sendq_len -= len;
+				fe_set_throttle (serv);
+
+				tcp_send_real (serv, buf, len);
+
+				buf--;
+				serv->outbound_queue = g_slist_remove (serv->outbound_queue, buf);
+				free (buf);
+				list = serv->outbound_queue;
+			} else
+			{
+				list = list->next;
+			}
+		}
+		/* now try pri 0 */
+		pri--;
+	}
+	return 0;						  /* remove the timeout handler */
+}
+
+int
+tcp_send_len (server *serv, char *buf, int len)
+{
+	char *dbuf;
+	int noqueue = !serv->outbound_queue;
+
+#ifdef USE_JCODE
+	if (prefs.kanji_conv)
+	{
+		buf = kanji_conv_auto (buf, "ISO-2022-JP");
+		if (!buf)
+			return -1;
+		len = strlen (buf);
+	}
+
+	if (!prefs.throttle)
+	{
+		int ret = tcp_send_real (serv, buf, len);
+		if (prefs.kanji_conv)
+			free (buf);
+		return ret;
+	}
+#else
+
+	if (!prefs.throttle)
+		return tcp_send_real (serv, buf, len);
+
+#endif
+
+	dbuf = malloc (len + 2);	/* first byte is the priority */
+	dbuf[0] = 1;	/* pri 1 for most things */
+	memcpy (dbuf + 1, buf, len);
+	dbuf[len + 1] = 0;
+
+#ifdef USE_JCODE
+	if (prefs.kanji_conv)
+		free (buf);
+#endif
+
+	/* privmsg and notice get a lower priority */
+	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
+		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
+		dbuf[0] = 0;
+
+	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
+	serv->sendq_len += len; /* tcp_send_queue uses strlen */
+
+	if (tcp_send_queue (serv) && noqueue)
+		fe_timeout_add (500, tcp_send_queue, serv);
+
+	return 1;
+}
+
+int
+tcp_send (server *serv, char *buf)
+{
+	return tcp_send_len (serv, buf, strlen (buf));
+}
+
+void
+tcp_sendf (server *serv, char *fmt, ...)
+{
+	va_list args;
+	/* keep this buffer in BSS */
+	static char send_buf[520];	/* good code hey (no it's not overflowable) */
+	int len;
+
+	va_start (args, fmt);
+	len = vsnprintf (send_buf, sizeof (send_buf) - 1, fmt, args);
+	va_end (args);
+
+	send_buf[sizeof (send_buf) - 1] = '\0';
+	if (len < 0 || len > (sizeof (send_buf) - 1))
+		len = strlen (send_buf);
+
+	tcp_send_len (serv, send_buf, len);
+}
 
 static int
 close_socket_cb (int sok)
@@ -86,10 +288,127 @@ close_socket (int sok)
 	fe_timeout_add (5000, close_socket_cb, GINT_TO_POINTER (sok));
 }
 
+/* handle 1 line of text received from the server */
+
+static void
+server_inline (server *serv, char *line)
+{
+	char *utf = NULL;
+#ifdef USE_JCODE
+	char *oline;
+
+	oline = line;
+	if (prefs.kanji_conv)
+		line = kanji_conv_to_locale (line);
+#endif
+
+	if (!serv->use_utf8)
+		utf = g_locale_to_utf8 (line, -1, NULL, NULL, NULL);
+	if (utf)
+	{
+		fe_add_rawlog (serv, utf, FALSE);
+		fe_checkurl (utf);
+
+		/* let proto-irc.c handle it */
+		serv->p_inline (serv, utf);
+
+		g_free (utf);
+	} else
+	{
+		fe_add_rawlog (serv, line, FALSE);
+		fe_checkurl (line);
+
+		/* let proto-irc.c handle it */
+		serv->p_inline (serv, line);
+	}
+
+#ifdef USE_JCODE
+	if (oline != line)
+		free (line);
+#endif
+}
+
+/* read data from socket */
+
+static gboolean
+server_read (GIOChannel *source, GIOCondition condition, server *serv)
+{
+	int sok = serv->sok;
+	int error, i, len;
+	char lbuf[2050];
+	char *temp;
+
+	while (1)
+	{
+#ifdef USE_OPENSSL
+		if (!serv->ssl)
+#endif
+			len = recv (sok, lbuf, sizeof (lbuf) - 2, 0);
+#ifdef USE_OPENSSL
+		else
+			len = _SSL_recv (serv->ssl, lbuf, sizeof (lbuf) - 2);
+#endif
+		if (len < 1)
+		{
+			error = 0;
+			if (len < 0)
+			{
+				if (would_block_again ())
+					return TRUE;
+				error = sock_error ();
+			}
+			if (prefs.autoreconnect)
+				auto_reconnect (serv, FALSE, error);
+			else
+				server_disconnect (serv->server_session, FALSE, error);
+			return TRUE;
+		}
+
+		i = 0;
+
+		lbuf[len] = 0;
+
+		while (i < len)
+		{
+			switch (lbuf[i])
+			{
+			case '\r':
+				break;
+
+			case '\n':
+				serv->linebuf[serv->pos] = 0;
+#ifdef USE_TRANS
+				if (prefs.use_trans)
+					serv2user (serv->linebuf);
+#endif
+				if (prefs.stripcolor)
+				{
+					temp = strip_color (serv->linebuf);
+					server_inline (serv, temp);
+					free (temp);
+				} else
+				{
+					server_inline (serv, serv->linebuf);
+				}
+				serv->pos = 0;
+				break;
+
+			default:
+				serv->linebuf[serv->pos] = lbuf[i];
+				if (serv->pos > 519)
+					fprintf (stderr,
+								"*** XCHAT WARNING: Buffer overflow - shit server!\n");
+				else
+					serv->pos++;
+			}
+			i++;
+		}
+	}
+}
+
 static void
 server_connected (server * serv)
 {
-	char hostname[256];
 	char outbuf[512];
 
 	serv->ping_recv = time (0);
@@ -100,43 +419,64 @@ server_connected (server * serv)
 	sprintf (outbuf, "%s/auth/xchat_auth", g_get_home_dir ());
 	if (access (outbuf, X_OK) == 0)
 	{
-		sprintf (outbuf, "/exec -d %s/auth/xchat_auth %s", g_get_home_dir (),
+		sprintf (outbuf, "exec -d %s/auth/xchat_auth %s", g_get_home_dir (),
 					prefs.username);
-		handle_command (outbuf, serv->front_session, FALSE, FALSE);
+		handle_command (serv->server_session, outbuf, FALSE);
 	}
 #endif
 
+	prefs.wait_on_exit = TRUE;
 	serv->connected = TRUE;
-	serv->iotag = fe_input_add (serv->sok, 1, 0, 1, read_data, serv);
+	set_nonblocking (serv->sok);
+	serv->iotag = fe_input_add (serv->sok, 1, 0, 1, server_read, serv);
 	if (!serv->no_login)
 	{
-		EMIT_SIGNAL (XP_TE_CONNECTED, serv->front_session, NULL, NULL, NULL,
+		EMIT_SIGNAL (XP_TE_CONNECTED, serv->server_session, NULL, NULL, NULL,
 						 NULL, 0);
-		if (serv->password[0])
-		{
-			sprintf (outbuf, "PASS %s\r\n", serv->password);
-			tcp_send (serv, outbuf);
-		}
-		gethostname (hostname, sizeof (hostname) - 1);
-		hostname[sizeof (hostname) - 1] = 0;
-		if (hostname[0] == 0)
-			strcpy (hostname, "0");
-		snprintf (outbuf, sizeof (outbuf),
-					 "NICK %s\r\n"
-					 "USER %s %s %s :%s\r\n",
-					 serv->nick, prefs.username, hostname,
-					 serv->servername, prefs.realname);
-		tcp_send (serv, outbuf);
+		serv->p_login (serv, prefs.username, prefs.realname);
 	} else
 	{
-		EMIT_SIGNAL (XP_TE_SERVERCONNECTED, serv->front_session, NULL, NULL,
+		EMIT_SIGNAL (XP_TE_SERVERCONNECTED, serv->server_session, NULL, NULL,
 						 NULL, NULL, 0);
 	}
 
-	set_nonblocking (serv->sok);
 	set_server_name (serv, serv->servername);
 }
 
+static void
+server_stopconnecting (server * serv)
+{
+	if (serv->iotag)
+	{
+		fe_input_remove (serv->iotag);
+		serv->iotag = 0;
+	}
+
+#ifndef WIN32
+	/* kill the child process trying to connect */
+	kill (serv->childpid, SIGKILL);
+	waitpid (serv->childpid, NULL, 0);
+#else
+	TerminateThread ((HANDLE)serv->childpid, 0);
+	CloseHandle ((HANDLE)serv->childpid);
+#endif
+
+	close (serv->childwrite);
+	close (serv->childread);
+
+#ifdef USE_OPENSSL
+	if (serv->ssl_do_connect_tag)
+	{
+		fe_timeout_remove (serv->ssl_do_connect_tag);
+		serv->ssl_do_connect_tag = 0;
+	}
+#endif
+
+	fe_progressbar_end (serv);
+
+	serv->connecting = FALSE;
+}
+
 #ifdef USE_OPENSSL
 #define	SSLTMOUT	10				  /* seconds */
 void
@@ -180,8 +520,7 @@ ssl_do_connect (server * serv)
 {
 	char buf[128];
 
-
-	g_sess = serv->front_session;
+	g_sess = serv->server_session;
 	if (SSL_connect (serv->ssl) <= 0)
 	{
 		char err_buf[128];
@@ -192,7 +531,7 @@ ssl_do_connect (server * serv)
 		{
 			ERR_error_string (err, err_buf);
 			snprintf (buf, sizeof (buf), "(%d) %s", err, err_buf);
-			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->front_session, buf, NULL,
+			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL,
 							 NULL, NULL, 0);
 
 			server_cleanup (serv);
@@ -215,61 +554,61 @@ ssl_do_connect (server * serv)
 		if (!_SSL_get_cert_info (&cert_info, serv->ssl))
 		{
 			snprintf (buf, sizeof (buf), "* Certification info:");
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 			snprintf (buf, sizeof (buf), "  Subject:");
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 			for (i = 0; cert_info.subject_word[i]; i++)
 			{
 				snprintf (buf, sizeof (buf), "    %s", cert_info.subject_word[i]);
-				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 								 NULL, 0);
 			}
 			snprintf (buf, sizeof (buf), "  Issuer:");
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 			for (i = 0; cert_info.issuer_word[i]; i++)
 			{
 				snprintf (buf, sizeof (buf), "    %s", cert_info.issuer_word[i]);
-				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 								 NULL, 0);
 			}
 			snprintf (buf, sizeof (buf), "  Public key algorithm: %s (%d bits)",
 						 cert_info.algorithm, cert_info.algorithm_bits);
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 			if (cert_info.rsa_tmp_bits)
 			{
 				snprintf (buf, sizeof (buf),
 							 "  Public key algorithm uses ephemeral key with %d bits",
 							 cert_info.rsa_tmp_bits);
-				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 								 NULL, 0);
 			}
 			snprintf (buf, sizeof (buf), "  Sign algorithm %s (%d bits)",
 						 cert_info.sign_algorithm, cert_info.sign_algorithm_bits);
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 			snprintf (buf, sizeof (buf), "  Valid since %s to %s",
 						 cert_info.notbefore, cert_info.notafter);
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 		} else
 		{
 			snprintf (buf, sizeof (buf), " * No Certificate");
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 		}
 
 		chiper_info = _SSL_get_cipher_info (serv->ssl);	/* static buffer */
 		snprintf (buf, sizeof (buf), "* Chiper info:");
-		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL,
+		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL, NULL,
 						 0);
 		snprintf (buf, sizeof (buf), "  Version: %s, cipher %s (%u bits)",
 					 chiper_info->version, chiper_info->chiper,
 					 chiper_info->chiper_bits);
-		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL,
+		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL, NULL,
 						 0);
 
 		verify_error = SSL_get_verify_result (serv->ssl);
@@ -277,7 +616,7 @@ ssl_do_connect (server * serv)
 		{
 		case X509_V_OK:
 			/* snprintf (buf, sizeof (buf), "* Verify OK (?)"); */
-			/* EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0); */
+			/* EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL, NULL, 0); */
 			break;
 		case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
 		case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
@@ -288,7 +627,7 @@ ssl_do_connect (server * serv)
 				snprintf (buf, sizeof (buf), "* Verify E: %s.? (%d) -- Ignored",
 							 X509_verify_cert_error_string (verify_error),
 							 verify_error);
-				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL,
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL,
 								 NULL, 0);
 				break;
 			}
@@ -296,7 +635,7 @@ ssl_do_connect (server * serv)
 			snprintf (buf, sizeof (buf), "%s.? (%d)",
 						 X509_verify_cert_error_string (verify_error),
 						 verify_error);
-			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->front_session, buf, NULL, NULL,
+			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL, NULL,
 							 NULL, 0);
 
 			server_cleanup (serv);
@@ -315,7 +654,7 @@ ssl_do_connect (server * serv)
 		if (serv->ssl->session->time + SSLTMOUT < time (NULL))
 		{
 			snprintf (buf, sizeof (buf), "SSL handshake timed out");
-			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->front_session, buf, NULL,
+			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL,
 							 NULL, NULL, 0);
 			server_cleanup (serv); /* ->connecting = FALSE */
 
@@ -331,29 +670,31 @@ ssl_do_connect (server * serv)
 #endif
 
 static int
-timeout_auto_reconnect (struct server *serv)
+timeout_auto_reconnect (server *serv)
 {
-	if (is_server (serv))		  /* make sure it hasnt been closed during the delay */
+	if (is_server (serv))  /* make sure it hasnt been closed during the delay */
 	{
 		serv->recondelay_tag = 0;
-		if (!serv->connected && !serv->connecting && serv->front_session)
+		if (!serv->connected && !serv->connecting && serv->server_session)
 		{
-			connect_server (serv->front_session, serv->hostname, serv->port,
-								 FALSE);
+			server_connect (serv, serv->hostname, serv->port, FALSE);
 		}
 	}
-	return 0;						  /* returning 0 should remove the timeout handler */
+	return 0;			  /* returning 0 should remove the timeout handler */
 }
 
 void
-auto_reconnect (struct server *serv, int send_quit, int err)
+auto_reconnect (server *serv, int send_quit, int err)
 {
 	session *s;
 	GSList *list;
 	int del;
 
-	if (serv->front_session == NULL)
+	if (serv->server_session == NULL)
+	{
+		puts("server_sess was 0!");
 		return;
+	}
 
 	list = sess_list;
 	while (list)				  /* make sure auto rejoin can work */
@@ -368,7 +709,7 @@ auto_reconnect (struct server *serv, int send_quit, int err)
 	}
 
 	if (serv->connected)
-		disconnect_server (serv->front_session, send_quit, err);
+		server_disconnect (serv->server_session, send_quit, err);
 
 	del = prefs.recon_delay * 1000;
 	if (del < 1000)
@@ -384,122 +725,8 @@ auto_reconnect (struct server *serv, int send_quit, int err)
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
 }
 
-static gboolean
-read_data (GIOChannel *source, GIOCondition condition, server *serv)
-{
-	int sok = serv->sok;
-	int error, i, len;
-	char lbuf[2050];
-	char *temp;
-#ifdef USE_JCODE
-	char *jtemp;
-#endif
-
-	while (1)
-	{
-		if (!EMIT_SIGNAL (XP_IF_RECV, &len, GINT_TO_POINTER (sok), &lbuf,
-								GINT_TO_POINTER (sizeof (lbuf) - 2), NULL, 0))
-		{
-#ifdef USE_OPENSSL
-			if (!serv->ssl)
-#endif
-				len = recv (sok, lbuf, sizeof (lbuf) - 2, 0);
-#ifdef USE_OPENSSL
-			else
-				len = _SSL_recv (serv->ssl, lbuf, sizeof (lbuf) - 2);
-#endif
-		}
-		if (len < 1)
-		{
-			if (len < 0)
-			{
-				if (would_block_again ())
-					return TRUE;
-				error = sock_error ();
-			} else
-			{
-				error = 0;
-			}
-			if (prefs.autoreconnect)
-				auto_reconnect (serv, FALSE, error);
-			else
-				disconnect_server (serv->front_session, FALSE, error);
-			return TRUE;
-		} else
-		{
-			i = 0;
-
-			lbuf[len] = 0;
-
-			while (i < len)
-			{
-				switch (lbuf[i])
-				{
-				case '\r':
-					break;
-
-				case '\n':
-					serv->linebuf[serv->pos] = 0;
-#ifdef USE_TRANS
-					if (prefs.use_trans)
-						serv2user (serv->linebuf);
-#endif
-					if (prefs.stripcolor)
-					{
-						temp = strip_color (serv->linebuf);
-#ifdef USE_JCODE
-						if (prefs.kanji_conv)
-						{
-							jtemp = kanji_conv_to_locale (temp);
-							if (jtemp)
-							{
-								process_line (serv, jtemp);
-								free (jtemp);
-							} else
-								process_line (serv, temp);
-						} else
-							process_line (serv, temp);
-#else
-						process_line (serv, temp);
-#endif
-						free (temp);
-					} else
-					{
-#ifdef USE_JCODE
-						if (prefs.kanji_conv)
-						{
-							jtemp = kanji_conv_to_locale (serv->linebuf);
-							if (jtemp)
-							{
-								process_line (serv, jtemp);
-								free (jtemp);
-							} else
-								process_line (serv, serv->linebuf);
-						} else
-							process_line (serv, serv->linebuf);
-#else
-						process_line (serv, serv->linebuf);
-#endif
-					}
-					serv->pos = 0;
-					break;
-
-				default:
-					serv->linebuf[serv->pos] = lbuf[i];
-					if (serv->pos > 519)
-						fprintf (stderr,
-									"*** XCHAT WARNING: Buffer overflow - shit server!\n");
-					else
-						serv->pos++;
-				}
-				i++;
-			}
-		}
-	}
-}
-
 void
-flush_server_queue (struct server *serv)
+server_flush_queue (server *serv)
 {
 	list_free (&serv->outbound_queue);
 	serv->sendq_len = 0;
@@ -533,10 +760,48 @@ waitline2 (GIOChannel *source, char *buf, int bufsize)
 
 #endif
 
-static gboolean
-connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
+/* connect() successed */
+
+static void
+server_connect_success (server *serv)
 {
-	session *sess = serv->front_session;
+#ifdef USE_OPENSSL
+#define	SSLDOCONNTMOUT	300
+	if (serv->use_ssl)
+	{
+		char *err;
+
+		/* it'll be a memory leak, if connection isn't terminated by
+		   server_cleanup() */
+		serv->ssl = _SSL_socket (ctx, serv->sok);
+		if ((err = _SSL_set_verify (ctx, ssl_cb_verify, NULL)))
+		{
+			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, err, NULL,
+							 NULL, NULL, 0);
+			server_cleanup (serv);	/* ->connecting = FALSE */
+			return;
+		}
+		/* FIXME: it'll be needed by new servers */
+		/* send(serv->sok, "STLS\r\n", 6, 0); sleep(1); */
+		set_nonblocking (serv->sok);
+		serv->ssl_do_connect_tag = fe_timeout_add (SSLDOCONNTMOUT,
+																 ssl_do_connect, serv);
+		return;
+	}
+
+	serv->ssl = NULL;
+#endif
+	server_stopconnecting (serv);	/* ->connecting = FALSE */
+	/* activate glib poll */
+	server_connected (serv);
+}
+
+/* receive info from the child-process about connection progress */
+
+static gboolean
+server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
+{
+	session *sess = serv->server_session;
 	char tbuf[128];
 	char outbuf[512];
 	char host[100];
@@ -549,8 +814,10 @@ connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
 	case '1':						  /* unknown host */
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
-		if (serv->sok6 >= 0)
+#ifdef USE_IPV6
+		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
+#endif
 		EMIT_SIGNAL (XP_TE_UKNHOST, sess, NULL, NULL, NULL, NULL, 0);
 		if (prefs.autoreconnectonfail)
 			auto_reconnect (serv, FALSE, -1);
@@ -559,8 +826,10 @@ connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
 		waitline2 (source, tbuf, sizeof tbuf);
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
-		if (serv->sok6 >= 0)
+#ifdef USE_IPV6
+		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
+#endif
 		EMIT_SIGNAL (XP_TE_CONNFAIL, sess, errorstring (atoi (tbuf)), NULL,
 						 NULL, NULL, 0);
 		if (prefs.autoreconnectonfail)
@@ -582,39 +851,7 @@ connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
 		else
 			closesocket (serv->sok4);
 #endif
-#ifdef USE_OPENSSL
-#define	SSLDOCONNTMOUT	300
-		if (serv->use_ssl)
-		{
-			char *err;
-
-			/* it'll be a memory leak, if connection isn't terminated by
-			   server_cleanup() */
-			serv->ssl = _SSL_socket (ctx, serv->sok);
-			if ((err = _SSL_set_verify (ctx, ssl_cb_verify, NULL)))
-			{
-				EMIT_SIGNAL (XP_TE_CONNFAIL, serv->front_session, err, NULL,
-								 NULL, NULL, 0);
-
-				server_cleanup (serv);	/* ->connecting = FALSE */
-				return TRUE;
-			}
-
-			/* FIXME: it'll be needed by new servers */
-			/* send(serv->sok, "STLS\r\n", 6, 0); sleep(1); */
-			set_nonblocking (serv->sok);
-			serv->ssl_do_connect_tag = fe_timeout_add (SSLDOCONNTMOUT,
-																	 ssl_do_connect, serv);
-		} else
-		{
-			serv->ssl = NULL;
-#endif
-			server_stopconnecting (serv);	/* ->connecting = FALSE */
-			/* activate gtk poll */
-			server_connected (serv);
-#ifdef USE_OPENSSL
-		}
-#endif
+		server_connect_success (serv);
 		break;
 	case '5':						  /* prefs ip discovered */
 		waitline2 (source, tbuf, sizeof tbuf);
@@ -622,13 +859,13 @@ connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
 		break;
 	case '7':						  /* gethostbyname (prefs.hostname) failed */
 		sprintf (outbuf,
-					"Cannot resolve hostname %s\nCheck your IP Settings!\n",
+					_("Cannot resolve hostname %s\nCheck your IP Settings!\n"),
 					prefs.hostname);
 		PrintText (sess, outbuf);
 		break;
 	case '8':
-		PrintText (sess, "Proxy traversal failed.\n");
-		disconnect_server (sess, FALSE, -1);
+		PrintText (sess, _("Proxy traversal failed.\n"));
+		server_disconnect (sess, FALSE, -1);
 		break;
 	case '9':
 		waitline2 (source, tbuf, sizeof tbuf);
@@ -639,40 +876,6 @@ connected_signal (GIOChannel *source, GIOCondition condition, server *serv)
 	return TRUE;
 }
 
-static void
-server_stopconnecting (server * serv)
-{
-	if (serv->iotag)
-	{
-		fe_input_remove (serv->iotag);
-		serv->iotag = 0;
-	}
-
-#ifndef WIN32
-	/* kill the child process trying to connect */
-	kill (serv->childpid, SIGKILL);
-	waitpid (serv->childpid, NULL, 0);
-#else
-	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
-	CloseHandle ((HANDLE)serv->childhandle);
-#endif
-
-	close (serv->childwrite);
-	close (serv->childread);
-
-#ifdef USE_OPENSSL
-	if (serv->ssl_do_connect_tag)
-	{
-		fe_timeout_remove (serv->ssl_do_connect_tag);
-		serv->ssl_do_connect_tag = 0;
-	}
-#endif
-
-	fe_progressbar_end (serv->front_session);
-
-	serv->connecting = FALSE;
-}
-
 /* kill all sockets & iotags of a server. Stop a connection attempt, or
    disconnect if already connected. */
 
@@ -686,6 +889,7 @@ server_cleanup (server * serv)
 		fe_input_remove (serv->iotag);
 		serv->iotag = 0;
 	}
+
 #ifdef USE_OPENSSL
 	if (serv->ssl)
 	{
@@ -698,7 +902,7 @@ server_cleanup (server * serv)
 	{
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
-		if (serv->sok6 >= 0)
+		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
 		return 1;
 	}
@@ -722,7 +926,7 @@ server_cleanup (server * serv)
 }
 
 void
-disconnect_server (session * sess, int sendquit, int err)
+server_disconnect (session * sess, int sendquit, int err)
 {
 	server *serv = sess->server;
 	GSList *list;
@@ -747,19 +951,24 @@ disconnect_server (session * sess, int sendquit, int err)
 		return;
 	case 1:							  /* it was in the process of connecting */
 		sprintf (tbuf, "%d", sess->server->childpid);
-		EMIT_SIGNAL (XP_TE_SCONNECT, sess, tbuf, NULL, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_STOPCONNECT, sess, tbuf, NULL, NULL, NULL, 0);
 		return;
 	}
 
-	flush_server_queue (serv);
+	server_flush_queue (serv);
 
 	list = sess_list;
-	while (list)					  /* print "Disconnected" to each window using this server */
+	while (list)
 	{
 		sess = (struct session *) list->data;
 		if (sess->server == serv)
+		{
+			/* print "Disconnected" to each window using this server */
 			EMIT_SIGNAL (XP_TE_DISCON, sess, errorstring (err), NULL, NULL, NULL,
 							 0);
+			if (!sess->channel[0] || sess->type == SESS_CHANNEL)
+				clear_channel (sess);
+		}
 		list = list->next;
 	}
 
@@ -768,25 +977,6 @@ disconnect_server (session * sess, int sendquit, int err)
 	serv->no_login = FALSE;
 	serv->servername[0] = 0;
 
-	list = sess_list;
-	while (list)
-	{
-		sess = (struct session *) list->data;
-		if (sess->server == serv)
-		{
-			if (sess->channel[0])
-			{
-				if (sess->type == SESS_CHANNEL)
-				{
-					clear_channel (sess);
-				}
-			} else
-			{
-				clear_channel (sess);
-			}
-		}
-		list = list->next;
-	}
 	notify_cleanup ();
 }
 
@@ -897,71 +1087,15 @@ traverse_wingate (int sok, char *serverAddr, int port)
 	return 0;
 }
 
-static void
-three_to_four (char *from, char *to)
-{
-	static const char tab64[64]={  'A','B','C','D','E','F','G','H',
-					 'I','J','K','L','M','N','O','P',
-					 'Q','R','S','T','U','V','W','X',
-					 'Y','Z','a','b','c','d','e','f',
-					 'g','h','i','j','k','l','m','n',
-					 'o','p','q','r','s','t','u','v',
-					 'w','x','y','z','0','1','2','3',
-					 '4','5','6','7','8','9','+','/'
-	};
-
-	to[0] = tab64 [ (from[0] >> 2) & 63 ];
-	to[1] = tab64 [ ((from[0] << 4) | (from[1] >> 4)) & 63 ];
-	to[2] = tab64 [ ((from[1] << 2) | (from[2] >> 6)) & 63 ];
-	to[3] = tab64 [ from[2] & 63 ];
-};
-
-static void
-base64_encode (char *to, char *from, unsigned int len)
-{
-	while (len >= 3)
-	{
-		three_to_four (from, to);
-		len -= 3;
-		from += 3;
-		to += 4;
-	}
-	if (len)
-	{
-		char three[3]={0,0,0};
-		int i=0;
-		for (i=0;i<len;i++)
-			three[i] = *from++;
-		three_to_four (three, to);
-		if (len == 1)
-			to[2] = to[3] = '=';
-		else if (len == 2)
-			to[3] = '=';
-		to += 4;
-	};
-	to[0] = 0;
-}
-
 static int
 traverse_http (int sok, char *serverAddr, int port)
 {
-	char buf[256];
-	char auth_data[128];
-	char auth_data2[68];
-	int n, n2;
+	char buf[128];
 
-	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n",
+	snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n\r\n",
 					  serverAddr, port);
-	if (prefs.proxy_auth)
-	{
-		n2 = snprintf (auth_data2, sizeof (auth_data2), "%s:%s",
-							prefs.proxy_user, prefs.proxy_pass);
-		base64_encode (auth_data, auth_data2, n2);
-		n += snprintf (buf+n, sizeof (buf)-n, "Proxy-Authorization: Basic %s\r\n", auth_data);
-	}
-	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
-	send (sok, buf, n, 0);
-#ifndef WIN32
+	send (sok, buf, strlen (buf), 0);
+#if 0
 	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
 	/* "HTTP/1.0 200 OK" */
 	if (strlen (buf) < 12)
@@ -1104,11 +1238,15 @@ server_child (server * serv)
 
 xit:
 
+	/* 'fd' is leaked on win32 */
 	fflush (fd);
 
+#if defined (USE_IPV6) || defined (WIN32)
+	/* this is probably not needed */
 	net_store_destroy (ns_server);
 	if (ns_proxy)
 		net_store_destroy (ns_proxy);
+#endif
 
 	/* no need to free ip/real_hostname, this process is exiting */
 #ifdef WIN32
@@ -1119,43 +1257,31 @@ xit:
 		free (ip);
 	if (real_hostname)
 		free (real_hostname);
-	fclose (fd);
 #endif
 
 	return 0;
 }
 
 void
-connect_server (session * sess, char *hostname, int port, int no_login)
+server_connect (server *serv, char *hostname, int port, int no_login)
 {
 	int pid, read_des[2];
-	server *serv = sess->server;
-
-#ifdef USE_OPENSSL
-	if (!ctx && serv->use_ssl)
-	{
-		if (!(ctx = _SSL_context_init (ssl_cb_info, FALSE)))
-		{
-			fprintf (stderr, "_SSL_context_init failed\n");
-			exit (1);
-		}
-	}
-#endif
+	session *sess;
 
 	if (!hostname[0])
 		return;
 
-	sess = serv->front_session;
+	sess = serv->server_session;
 
 	if (serv->connected || serv->connecting || serv->recondelay_tag)
-		disconnect_server (sess, TRUE, -1);
+		server_disconnect (sess, TRUE, -1);
 
 	fe_progressbar_start (sess);
 
 	EMIT_SIGNAL (XP_TE_SERVERLOOKUP, sess, hostname, NULL, NULL, NULL, 0);
 
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
-	strcpy (serv->hostname, serv->servername);
+	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
 
 	set_server_defaults (serv);
 	serv->connecting = TRUE;
@@ -1163,7 +1289,7 @@ connect_server (session * sess, char *hostname, int port, int no_login)
 	serv->no_login = no_login;
 
 	fe_set_away (serv);
-	flush_server_queue (serv);
+	server_flush_queue (serv);
 
 	/* pipe is #defined on win32 in gwin32.h to use _pipe and O_BINARY */
 	if (pipe (read_des) < 0)
@@ -1179,9 +1305,8 @@ connect_server (session * sess, char *hostname, int port, int no_login)
 	net_sockets (&serv->sok4, &serv->sok6);
 
 #ifdef WIN32
-	serv->childhandle = (int)CreateThread (NULL, 0,
-													(LPTHREAD_START_ROUTINE)server_child,
-													serv, 0, (DWORD *)&pid);
+	pid = (int)CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE)server_child,
+									 serv, 0, (DWORD *)&pid);
 #else
 	switch (pid = fork ())
 	{
@@ -1198,5 +1323,5 @@ connect_server (session * sess, char *hostname, int port, int no_login)
 	serv->childpid = pid;
 	/* the 3 tells input_add that it's a fd, not a socket */
 	serv->iotag =
-		fe_input_add (serv->childread, 3, 0, 0, connected_signal, serv);
+		fe_input_add (serv->childread, 3, 0, 0, server_read_child, serv);
 }
