@@ -32,11 +32,18 @@
 #include "fe.h"
 #include "util.h"
 #include "outbound.h"
-#include "perlc.h"
 #include "xchatc.h"
 #include "text.h"
 
 
+struct pevt_stage1
+{
+	int len;
+	char *data;
+	struct pevt_stage1 *next;
+};
+
+
 #ifdef USE_HEBREW
 
 static inline int iswhitespace(const char ch)
@@ -54,12 +61,6 @@ static inline int iseng(const char ch)
 	return ((ch >= 'A'  &&  ch <= 'z')  ||  (ch >= 'a'  &&  ch <= 'z'));
 }
 
-/* this is already in ctype.h */
-/*static inline int isdigit(const char ch)
-{
-	return (ch >= '0'  &&  ch <= '9');
-}*/
-
 static char *strhebpatch(char *dest, const char *src)
 {
 	short int mode = 0, imode;
@@ -141,71 +142,30 @@ static char *strhebpatch(char *dest, const char *src)
 #endif /* !USE_HEBREW */
 
 void
-PrintText (struct session *sess, unsigned char *text)
-{
-	if (!sess)
-	{
-		if (!sess_list)
-			return;
-		sess = (struct session *) sess_list->data;
-	}
-
-	if (sess->logfd != -1 && prefs.logging)
-	{
-		time_t timval = time (0);
-		char *temp = strip_color (text);
-		char tim[32];
-		strftime (tim, 32, prefs.timestamp_log_format, localtime (&timval));
-		if (prefs.timestamp || prefs.timestamp_logs)
-			write (sess->logfd, tim, strlen(tim));
-		write (sess->logfd, temp, strlen (temp));
-		free (temp);
-	}
-
-#ifdef USE_HEBREW
-	if (prefs.hebrew)
-	{
-   	const char *orgtext = NULL;
-   	if (text)
-   	{
-			orgtext = text;
-			text = strdup(text);
-			strhebpatch (text, orgtext);
-   	}
-		fe_print_text (sess, text);
-   	if (orgtext)
-			free(text);
-	} else
-	{
-#endif
-		fe_print_text (sess, text);
-#ifdef USE_HEBREW
-	}
-#endif
-}
-
-void
-end_logging (int fd)
+log_close (session *sess)
 {
 	char obuf[512];
 	time_t currenttime;
 
-	currenttime = time (NULL);
-	write (fd, obuf,
-			 snprintf (obuf, 510, _("**** ENDING LOGGING AT %s\n"),
+	if (sess->logfd != -1)
+	{
+		currenttime = time (NULL);
+		write (sess->logfd, obuf,
+			 snprintf (obuf, sizeof (obuf) - 1, _("**** ENDING LOGGING AT %s\n"),
 						  ctime (&currenttime)));
-	close (fd);
+		close (sess->logfd);
+		sess->logfd = -1;
+	}
 }
 
-static int
-open_log_file (char *servname, char *channame, char *netname)
+static void
+log_create_filename (char *buf, char *servname, char *channame)
 {
-	char buf[512], fn[256];
+	char fn[256];
 	char *dir, *sep;
-	int fd, pathlen=510, c=0;
-	time_t currenttime;
+	int pathlen=510, c=0;
 
-	if (!strcasecmp (channame, servname))
+	if (!rfc_casecmp (channame, servname))
 	{
 		channame = strdup ("server");
 	} else
@@ -226,16 +186,17 @@ open_log_file (char *servname, char *channame, char *netname)
 		}
 	}
 
-	snprintf (buf, 510, "%s/xchatlogs", get_xdir ());
+	snprintf (buf, 512, "%s/xchatlogs", get_xdir ());
 	if (access (buf, F_OK) != 0)
 #ifdef WIN32
 		mkdir (buf);
 #else
 		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
-	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
+	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", "", servname);
+	free (channame);
 
-	snprintf (buf, 510, "%s/xchatlogs/%s", get_xdir (), fn);
+	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir (), fn);
 
  	/* The following code handles subdirectories in logpath and creates
  	 * them, if they don't exist. Useful with logmasks like "%c/%y.log" in
@@ -243,8 +204,9 @@ open_log_file (char *servname, char *channame, char *netname)
  	 *                       -- patch by tvk <tvk@von-koch.com>
  	 */
 
-	if (access (buf, F_OK) != 0) {
-		snprintf (buf, 510, "%s/xchatlogs/", get_xdir ());
+	if (access (buf, F_OK) != 0)
+	{
+		snprintf (buf, 512, "%s/xchatlogs/", get_xdir ());
 		pathlen -= strlen(buf);
  
 		/* how many sub-directories do we have? */
@@ -278,8 +240,16 @@ open_log_file (char *servname, char *channame, char *netname)
 		}
 
 	}
+}
 
-	free (channame);
+static int
+log_open_file (char *servname, char *channame)
+{
+	char buf[512];
+	int fd;
+	time_t currenttime;
+
+	log_create_filename (buf, servname, channame);
 #ifdef WIN32
 	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY | OFLAGS, S_IREAD|S_IWRITE);
 #else
@@ -296,11 +266,80 @@ open_log_file (char *servname, char *channame, char *netname)
 }
 
 void
-setup_logging (struct session *sess)
+log_open (session *sess)
 {
-	if (sess->logfd != -1)
-		end_logging (sess->logfd);
-	sess->logfd = open_log_file (sess->server->servername, sess->channel, sess->server->network);
+	log_close (sess);
+	sess->logfd = log_open_file (sess->server->servername, sess->channel);
+}
+
+static void
+log_write (session *sess, char *text)
+{
+	time_t timval;
+	char *temp;
+	char tim[32];
+
+	if (sess->logfd != -1 && prefs.logging)
+	{
+		if (prefs.timestamp_logs)
+		{
+			timval = time (0);
+			strftime (tim, sizeof (tim), prefs.timestamp_log_format,
+						 localtime (&timval));
+			write (sess->logfd, tim, strlen (tim));
+		}
+		temp = strip_color (text);
+		write (sess->logfd, temp, strlen (temp));
+		free (temp);
+	}
+}
+
+void
+PrintText (session *sess, unsigned char *text)
+{
+	if (!sess)
+	{
+		if (!sess_list)
+			return;
+		sess = (session *) sess_list->data;
+	}
+
+	log_write (sess, text);
+
+#ifdef USE_HEBREW
+	if (prefs.hebrew)
+	{
+   	const char *orgtext = NULL;
+   	if (text)
+   	{
+			orgtext = text;
+			text = strdup(text);
+			strhebpatch (text, orgtext);
+   	}
+		fe_print_text (sess, text);
+   	if (orgtext)
+			free(text);
+	} else
+	{
+#endif
+		fe_print_text (sess, text);
+#ifdef USE_HEBREW
+	}
+#endif
+}
+
+void
+PrintTextf (session *sess, char *format, ...)
+{
+	va_list args;
+	char *buf;
+
+	va_start (args, format);
+	buf = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	PrintText (sess, buf);
+	g_free (buf);
 }
 
 /* Print Events stuff here --AGL */
@@ -325,22 +364,16 @@ setup_logging (struct session *sess)
 /* These lists are thus:
    pntevts_text[] are the strings the user sees (WITH %x etc)
    pntevts[] are the data strings with \000 etc
-   evtnames[] are the names of the events
  */
 
 /* To add a new event:
 
-   Think up a name (like JOIN)
-   Make up a pevt_name_help struct (with a NULL at the end)
-   add a struct to the end of te with,
-   {"Name", help_struct, "Default text", num_of_args, NULL}
-   Open up plugin.h
-   add one to NUM_XP
-   add a #define XP_TE_NAME with a value of +1 on the last XP_
+   Think up a name (like "Join")
+   Make up a pevt_name_help struct
+	Add an entry to textevents.in
+	Type: make textevents
  */
 
-/* As of 990528 this code is in BETA --AGL */
-
 /* Internals:
 
    On startup ~/.xchat/printevents.conf is loaded if it doesn't exist the
@@ -356,7 +389,7 @@ setup_logging (struct session *sess)
    (byte) number of varable to insert
    2 = end of buffer
 
-   Each XP_TE_* signal is hard coded to call textsignal_handler which calls
+   Each XP_TE_* signal is hard coded to call text_emit which calls
    display_event which decodes the data
 
    This means that this system *should be faster* than snprintf because
@@ -366,8 +399,8 @@ setup_logging (struct session *sess)
    --AGL
  */
 
-char **pntevts_text;
-char **pntevts;
+char *pntevts_text[NUM_XP];
+char *pntevts[NUM_XP];
 
 #define pevt_generic_none_help NULL
 
@@ -511,8 +544,8 @@ static char *pevt_chanop_help[] = {
 };
 
 static char *pevt_chanhop_help[] = {
-	N_("The nick of the person who did the halfop'ing"),
 	N_("The nick of the person who has been halfop'ed"),
+	N_("The nick of the person who did the halfop'ing"),
 };
 
 static char *pevt_chanvoice_help[] = {
@@ -622,6 +655,7 @@ static char *pevt_generic_channel_help[] = {
 
 static char *pevt_servertext_help[] = {
 	N_("Text"),
+	N_("Server Name")
 };
 
 static char *pevt_invited_help[] = {
@@ -708,11 +742,15 @@ static char *pevt_dccconfail_help[] = {
 	N_("Error string"),
 };
 
-static char *pevt_dcccon_help[] = {
-	N_("DCC Type"),
+static char *pevt_dccchatcon_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
-	N_("{to|from}"),
+};
+
+static char *pevt_dcccon_help[] = {
+	N_("Nickname"),
+	N_("IP address"),
+	N_("Filename"),
 };
 
 static char *pevt_dccsendfail_help[] = {
@@ -729,11 +767,15 @@ static char *pevt_dccsendcomp_help[] = {
 static char *pevt_dccoffer_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
+	N_("Pathname"),
 };
 
-static char *pevt_dccabort_help[] = {
-	N_("DCC type"),
-	N_("Filename"),
+static char *pevt_dccfileabort_help[] = {
+	N_("Nickname"),
+	N_("Filename")
+};
+
+static char *pevt_dccchatabort_help[] = {
 	N_("Nickname"),
 };
 
@@ -780,16 +822,6 @@ static char *pevt_ctcpsend_help[] = {
 	N_("Message"),
 };
 
-static char *pevt_ignorelist_help[] = {
-	N_("Hostmask"),
-	N_("Priv Yes/No"),
-	N_("Noti Yes/No"),
-	N_("Chan Yes/No"),
-	N_("Ctcp Yes/No"),
-	N_("Invi Yes/No"),
-	N_("Unig Yes/No"),
-};
-
 static char *pevt_ignoreaddremove_help[] = {
 	N_("Hostmask"),
 };
@@ -799,15 +831,11 @@ static char *pevt_resolvinguser_help[] = {
 	N_("Hostname"),
 };
 
-static char *pevt_dccmalformed_help[] = {
+static char *pevt_malformed_help[] = {
+	N_("Nickname"),
 	N_("The Packet"),
 };
 
-static char *pevt_newmail_help[] = {
-	N_("Number of messages"),
-	N_("Bytes in mailbox"),
-};
-
 static char *pevt_pingtimeout_help[] = {
 	N_("Seconds"),
 };
@@ -829,314 +857,7 @@ static char *pevt_discon_help[] = {
 	N_("Error"),
 };
 
-/* *BBIIGG* struct ahead!! --AGL */
-
-struct text_event te[] = {
-	/* Padding for all the non-text signals */
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-	{NULL, NULL, NULL, 0, NULL},
-
-	/* Now we get down to business */
-
-	
-	{"Join", pevt_join_help,
-	 N_("-%C10-%C11>%O$t%B$1%B %C14(%C10$3%C14)%C has joined $2"), 3, NULL},
-	{"Channel Action", pevt_chanaction_help, N_("%C13*%O$t$1 $2%O"), 2, NULL},
-	{"Channel Action Hilight", pevt_chanaction_help, N_("%C13*%O$t%C8%B$1%B%O $2%O"), 2, NULL},
-	{"Channel Message", pevt_chanmsg_help, N_("%C2<%O$1%C2>%O$t$2%O"), 3, NULL},
-	{"Channel Msg Hilight", pevt_chanmsg_help, N_("%C2<%C8%B$1%B%C2>%O$t$2%O"), 3, NULL},
-	{"Private Message", pevt_privmsg_help, N_("%C12*%C13$1%C12*$t%O$2%O"), 2, NULL},
-	{"Change Nick", pevt_changenick_help,
-	 N_("-%C10-%C11-%O$t$1 is now known as $2"), 2, NULL},
-	{"New Topic", pevt_newtopic_help,
-	 N_("-%C10-%C11-%O$t$1 has changed the topic to: $2%O"), 3, NULL},
-	{"Topic", pevt_topic_help, N_("-%C10-%C11-%O$tTopic for %C11$1%C is %C11$2%O"),
-	 2, NULL},
-	{"Kick", pevt_kick_help, N_("<%C10-%C11-%O$t$1 has kicked $2 from $3 ($4%O)"),
-	 4, NULL},
-	{"Part", pevt_part_help, N_("<%C10-%C11-%O$t$1 %C14(%O$2%C14)%C has left $3"),
-	 3, NULL},
-	{"Channel Creation", pevt_chandate_help,
-	 N_("-%C10-%C11-%O$tChannel $1 created on $2"), 2, NULL},
-	{"Topic Creation", pevt_topicdate_help,
-	 N_("-%C10-%C11-%O$tTopic for %C11$1%C set by %C11$2%C at %C11$3%O"), 3, NULL},
-	{"Quit", pevt_quit_help, N_("<%C10-%C11-%O$t$1 has quit %C14(%O$2%O%C14)%O"),
-	 3, NULL},
-	{"Ping Reply", pevt_pingrep_help,
-	 N_("-%C10-%C11-%O$tPing reply from $1 : $2 second(s)"), 2, NULL},
-	{"Notice", pevt_notice_help, N_("%C12-%C13$1%C12-%O$t$2%O"), 2, NULL},
-
-	{"You Joining", pevt_join_help,
-	 N_("-%C10-%C11>%O$t%BYou%B are now talking on %C11$2%O"), 3, NULL},
-	{"Your Message", pevt_chanmsg_help, N_("%C6<%O$1%C6>%O$t$2%O"), 3, NULL},
-
-	{"Private Message to Dialog", pevt_dprivmsg_help,
-	 N_("%C2<%O$1%C2>%O$t$2%O"), 2, NULL},
-	{"Your Nick Changing", pevt_uchangenick_help,
-	 N_("-%C10-%C11-%O$tYou are now known as $2"), 2, NULL},
-	{"You're Kicked", pevt_ukick_help,
-	 N_("-%C10-%C11-%O$tYou have been kicked from $2 by $3 ($4%O)"), 4, NULL},
-	{"You Parting", pevt_part_help, N_("-%C10-%C11-%O$tYou have left channel $3"),
-	 3, NULL},
-	{"CTCP Sound", pevt_ctcpsnd_help,
-	 N_("-%C10-%C11-%O$tReceived a CTCP Sound $1 from $2"), 2, NULL},
-	{"CTCP Generic", pevt_ctcpgen_help,
-	 N_("-%C10-%C11-%O$tReceived a CTCP $1 from $2"), 2, NULL},
-	{"CTCP Generic to Channel", pevt_ctcpgenc_help,
-	 N_("-%C10-%C11-%O$tReceived a CTCP $1 from $2 (to $3)"), 3, NULL},
-	{"Channel Set Key", pevt_chansetkey_help,
-	 N_("-%C10-%C11-%O$t$1 sets channel keyword to $2"), 2, NULL},
-	{"Channel Set Limit", pevt_chansetlimit_help,
-	 N_("-%C10-%C11-%O$t$1 sets channel limit to $2"), 2, NULL},
-	{"Channel Half-Operator", pevt_chanhop_help,
-	 N_("-%C10-%C11-%O$t$1 gives channel half-operator status to $2"), 2, NULL},
-	{"Channel Operator", pevt_chanop_help,
-	 N_("-%C10-%C11-%O$t$1 gives channel operator status to $2"), 2, NULL},
-	{"Channel Voice", pevt_chanvoice_help,
-	 N_("-%C10-%C11-%O$t$1 gives voice to $2"), 2, NULL},
-	{"Channel Ban", pevt_chanban_help, N_("-%C10-%C11-%O$t$1 sets ban on $2"), 2,
-	 NULL},
-	{"Channel Remove Keyword", pevt_chanrmkey_help,
-	 N_("-%C10-%C11-%O$t$1 removes channel keyword"), 1, NULL},
-	{"Channel Remove Limit", pevt_chanrmlimit_help,
-	 N_("-%C10-%C11-%O$t$1 removes user limit"), 1, NULL},
-	{"Channel DeHalfOp", pevt_chandehop_help,
-	 N_("-%C10-%C11-%O$t$1 removes channel half-operator status from $2"), 2,
-	 NULL},
-	{"Channel DeOp", pevt_chandeop_help,
-	 N_("-%C10-%C11-%O$t$1 removes channel operator status from $2"), 2, NULL},
-	{"Channel DeVoice", pevt_chandevoice_help,
-	 N_("-%C10-%C11-%O$t$1 removes voice from $2"), 2, NULL},
-	{"Channel UnBan", pevt_chanunban_help,
-	 N_("-%C10-%C11-%O$t$1 removes ban on $2"), 2, NULL},
-	{"Channel Exempt", pevt_chanexempt_help,
-	 N_("-%C10-%C11-%O$t$1 sets exempt on $2"), 2, NULL},
-	{"Channel Remove Exempt", pevt_chanrmexempt_help,
-	 N_("-%C10-%C11-%O$t$1 removes exempt on $2"), 2, NULL},
-	{"Channel INVITE", pevt_chaninvite_help,
-	 N_("-%C10-%C11-%O$t$1 sets invite on $2"), 2, NULL},
-	{"Channel Remove INVITE", pevt_chanrminvite_help,
-	 N_("-%C10-%C11-%O$t$1 removes invite on $2"), 2, NULL},
-	{"Channel Mode Generic", pevt_chanmodegen_help,
-	 N_("-%C10-%C11-%O$t$1 sets mode $2$3 $4"), 4, NULL},
-	{"WhoIs Name Line", pevt_whois1_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12] %C14(%O$2@$3%C14) %O: $4%O"), 4, NULL},
-	{"WhoIs Channel/Oper Line", pevt_whois2_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12]%C $2"), 2, NULL},
-	{"WhoIs Server Line", pevt_whois3_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12]%O $2"), 2, NULL},
-	{"WhoIs Idle Line", pevt_whois4_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12]%O idle %C11$2%O"), 2, NULL},
-	{"WhoIs Idle Line with Signon", pevt_whois4t_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12]%O idle %C11$2%O, signon: %C11$3%O"), 3,
-	 NULL},
-	{"WhoIs Away Line", pevt_whois5_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12] %Cis away %C14(%O$2%O%C14)"), 2, NULL},
-	{"WhoIs End", pevt_whois6_help,
-	 N_("-%C10-%C11-%O$t%C12[%O$1%C12] %CEnd of WHOIS list."), 1, NULL},
-	{"User Limit", pevt_generic_channel_help,
-	 N_("-%C10-%C11-%O$tCannot join%C11 %B$1 %O(User limit reached)."), 1, NULL},
-	{"Banned", pevt_generic_channel_help,
-	 N_("-%C10-%C11-%O$tCannot join%C11 %B$1 %O(You are banned)."), 1, NULL},
-	{"Invite", pevt_generic_channel_help,
-	 N_("-%C10-%C11-%O$tCannot join%C11 %B$1 %O(Channel is invite only)."), 1,
-	 NULL},
-	{"Keyword", pevt_generic_channel_help,
-	 N_("-%C10-%C11-%O$tCannot join%C11 %B$1 %O(Requires keyword)."), 1, NULL},
-	{"MOTD Skipped", pevt_generic_none_help, N_("-%C10-%C11-%O$tMOTD Skipped."), 0,
-	 NULL},
-	{"Server Text", pevt_servertext_help, N_("-%C10-%C11-%O$t$1%O"), 1, NULL},
-
-	{"Invited", pevt_invited_help,
-	 N_("-%C10-%C11-%O$tYou have been invited to %C11$1%C by %C11$2%C (%C11$3%C)"),
-	 3, NULL},
-	{"Users On Channel", pevt_usersonchan_help,
-	 N_("-%C10-%C11-%O$t%C11Users on $1:%C $2"), 2, NULL},
-	{"Nick Clash", pevt_nickclash_help,
-	 N_("-%C10-%C11-%O$t$1 already in use. Retrying with $2.."), 2, NULL},
-	{"Nick Failed", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tNickname already in use. Use /NICK to try another."), 0,
-	 NULL},
-	{"Unknown Host", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tUnknown host. Maybe you misspelled it?"), 0, NULL},
-	{"Connection Failed", pevt_connfail_help,
-	 N_("-%C10-%C11-%O$tConnection failed. Error: $1"), 1, NULL},
-	{"Connecting", pevt_connect_help,
-	 N_("-%C10-%C11-%O$tConnecting to %C11$1 %C14(%C11$2%C14)%C port %C11$3%C.."),
-	 3, NULL},
-	{"Connected", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tConnected. Now logging in.."), 0, NULL},
-	{"Stop Connection", pevt_sconnect_help,
-	 N_("-%C10-%C11-%O$tStopped previous connection attempt (pid=$1)"), 1, NULL},
-	{"Disconnected", pevt_discon_help,
-	 N_("-%C10-%C11-%O$tDisconnected ($1)."), 1, NULL},
-	{"No DCC", pevt_generic_none_help, N_("-%C10-%C11-%O$tNo such DCC."), 0, NULL},
-	
-	{"Delete Notify", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$t$1 deleted from notify list."), 1, NULL},
-	{"Add Notify", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$t$1 added to notify list."), 1, NULL},
-	{"Channel Modes", pevt_chanmodes_help,
-	 N_("-%C10-%C11-%O$tChannel $1 modes: $2"), 2, NULL},
-	{"Raw Modes", pevt_rawmodes_help,
-	 N_("-%C10-%C11-%O$t$1 sets modes%B %C14[%O$2%B%C14]%O"), 2, NULL},
-	{"Killed", pevt_kill_help,
-	 N_("-%C10-%C11-%O$tYou have been killed by $1 ($2%O)"), 2, NULL},
-	{"DCC Stall", pevt_dccstall_help,
-	 N_("-%C10-%C11-%O$tDCC $1 %C11$2 %Cto %C11$3 %Cstalled - aborting."), 3,
-	 NULL},
-	{"DCC Timeout", pevt_dccstall_help,
-	 N_("-%C10-%C11-%O$tDCC $1 %C11$2 %Cto %C11$3 %Ctimed out - aborting."), 3,
-	 NULL},
-	{"DCC CHAT Failed", pevt_dccchaterr_help,
-	 N_("-%C10-%C11-%O$tDCC CHAT failed. Connection to $1 %C14[%O$2:$3%C14]%O lost."),
-	 3, NULL},
-	{"DCC RECV File Open Error", pevt_generic_file_help,
-	 N_("-%C10-%C11-%O$tDCC RECV: Cannot open $1 for writing - aborting."), 1,
-	 NULL},
-	{"DCC RECV Failed", pevt_dccrecverr_help,
-	 N_("-%C10-%C11-%O$tDCC RECV $1 ($2) failed. Connection to $3 lost."), 3,
-	 NULL},
-	{"DCC RECV Complete", pevt_dccrecvcomp_help,
-	 N_("-%C10-%C11-%O$tDCC RECV %C11$1%O from %C11$3%O complete %C14[%C11$4%O cps%C14]%O."),
-	 4, NULL},
-	{"DCC Conection Failed", pevt_dccconfail_help,
-	 N_("-%C10-%C11-%O$tDCC $1 connect attempt to %C11$2%O failed (err=$3)."), 3,
-	 NULL},
-	{"DCC Connected", pevt_dcccon_help,
-	 N_("-%C10-%C11-%O$tDCC $1 connection established $4 %C11$2 %C14[%O$3%C14]%O"),
-	 4, NULL},
-	{"DCC SEND Failed", pevt_dccsendfail_help,
-	 N_("-%C10-%C11-%O$tDCC SEND %C11$1%O failed. Connection to %C11$2%O lost."),
-	 2, NULL},
-	{"DCC SEND Complete", pevt_dccsendcomp_help,
-	 N_("-%C10-%C11-%O$tDCC SEND %C11$1%O to %C11$2%O complete %C14[%C11$3%O cps%C14]%O."),
-	 3, NULL},
-	{"DCC Offer", pevt_dccoffer_help,
-	 N_("-%C10-%C11-%O$tOffering %C11$1 %Cto %C11$2%O"), 2, NULL},
-	{"DCC Abort", pevt_dccabort_help,
-	 N_("-%C10-%C11-%O$tDCC $1 %C11$2 %Cto %C11$3 %Caborted."), 3, NULL},
-	{"DCC Offer Not Valid", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tNo such DCC offer."), 0, NULL},
-	{"DCC CHAT Reoffer", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tAlready offering CHAT to $1"), 1, NULL},
-	{"DCC CHAT Offering", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tOffering DCC CHAT to $1"), 1, NULL},
-	{"DCC CHAT Offer", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tReceived a DCC CHAT offer from $1"), 1, NULL},
-	{"DCC RESUME Request", pevt_dccresumeoffer_help,
-	 N_("-%C10-%C11-%O$t%C11$1 %Chas requested to resume %C11$2 %Cfrom %C11$3%C."),
-	 3, NULL},
-	{"DCC SEND Offer", pevt_dccsendoffer_help,
-	 N_("-%C10-%C11-%O$t%C11$1 %Chas offered %C11$2 %C(%C11$3 %Cbytes)"), 3, NULL},
-	{"DCC Generic Offer", pevt_dccgenericoffer_help,
-	 N_("-%C10-%C11-%O$tReceived '$1%O' from $2"), 2, NULL},
-	{"Notify Online", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tNotify: $1 is online ($2)."), 2, NULL},
-	{"Notify Number", pevt_notifynumber_help,
-	 N_("-%C10-%C11-%O$t$1 users in notify list."), 1, NULL},
-	{"Notify Empty", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tNotify list is empty."), 0, NULL},
-	{"No Running Process", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tNo process is currently running"), 0, NULL},
-	{"Process Already Running", pevt_generic_none_help,
-	 N_("-%C10-%C11-%O$tA process is already running"), 0, NULL},
-	{"Server Lookup", pevt_serverlookup_help,
-	 N_("-%C10-%C11-%O$tLooking up %C11$1%C.."), 1, NULL},
-	{"Server Connected", pevt_generic_none_help, N_("-%C10-%C11-%O$tConnected."),
-	 0, NULL},
-	{"Server Error", pevt_servererror_help, N_("-%C10-%C11-%O$t$1%O"), 1, NULL},
-	
-	{"Server Generic Message", pevt_servererror_help, N_("-%C10-%C11-%O$t$1%O"),
-	 1, NULL},
-	{"Found IP", pevt_foundip_help, N_("-%C10-%C11-%O$tFound your IP: [$1]"), 1,
-	 NULL},
-	{"DCC Rename", pevt_dccrename_help,
-	 N_("-%C10-%C11-%O$tThe file %C11$1%C already exists, saving it as %C11$2%O instead."),
-	 2, NULL},
-	{"CTCP Send", pevt_ctcpsend_help, N_("%C3>%O$1%C3<%O$tCTCP $2%O"), 2, NULL},
-	{"Message Send", pevt_ctcpsend_help, N_("%C3>%O$1%C3<%O$t$2%O"), 2, NULL},
-	{"Notice Send", pevt_ctcpsend_help, N_("%C3>%O$1%C3<%O$t$2%O"), 2, NULL},
-	
-	{"Receive Wallops", pevt_dprivmsg_help,
-	 N_("%C12-%C13$1/Wallops%C12-%O$t$2%O"), 2, NULL},
-
-	{"Ignore Header", pevt_generic_none_help,
-	 N_("%C08,02 Hostmask             PRIV NOTI CHAN CTCP INVI UNIG %O"), 0, NULL},
-	{"Ignore List", pevt_ignorelist_help, N_(" $1 $2  $3  $4  $5  $6  $7"), 7,
-	 NULL},
-	{"Ignore Footer", pevt_generic_none_help,
-	 N_("%C08,02                                                    %O"), 0, NULL},
-	{"Ignore Add", pevt_ignoreaddremove_help,
-	 N_("%O%C11$1%O added to ignore list."), 1, NULL},
-	{"Ignore Remove", pevt_ignoreaddremove_help,
-	 N_("%O%C11$1%O removed from ignore list."), 1, NULL},
-	{"Resolving User", pevt_resolvinguser_help,
-	 N_("-%C10-%C11-%O$tLooking up IP number for%C11 $1%O.."), 2, NULL},
-	{"Ignorelist Empty", pevt_generic_none_help, N_("  Ignore list is empty."), 0,
-	 NULL},
-	{"Ignore Changed", pevt_ignoreaddremove_help,
-	 N_("Ignore on %C11$1%O changed."), 1, NULL},
-	{"Notify Offline", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tNotify: $1 is offline ($2)."), 2, NULL},
-	{"DCC Malformed From", pevt_generic_nick_help,
-	 N_("-%C10-%C11-%O$tReceived a malformed DCC request from %C11$1%O."), 1,
-	 NULL},
-	{"DCC Malformed Packet", pevt_dccmalformed_help,
-	 N_("-%C10-%C11-%O$tContents of packet: \"$1%O\"."), 1, NULL},
-	{"Part with Reason", pevt_partreason_help,
-	 N_("<%C10-%C11-%O$t$1 %C14(%O$2%C14)%C has left $3 %C14(%O$4%C14)%O"), 4,
-	 NULL},
-	{"You Part with Reason", pevt_partreason_help,
-	 N_("-%C10-%C11-%O$tYou have left channel $3 %C14(%O$4%C14)%O"), 4, NULL},
-	{"New Mail", pevt_newmail_help,
-	 N_("-%C3-%C9-%O$tYou have new mail ($1 messages, $2 bytes total)."), 2, NULL},
-	{"Motd", pevt_servertext_help, N_("-%C10-%C11-%O$t$1%O"), 1, NULL},
-	{"Ping Timeout", pevt_pingtimeout_help,
-		N_("-%C10-%C11-%O$tNo ping reply for $1 seconds, disconnecting."), 1, NULL},
-	{"Your Invitation", pevt_uinvite_help,
-		N_("-%C10-%C11-%O$tYou're inviting %C11$1%C to %C11$2%C (%C11$3%C)"), 3,
-		NULL},
-	{"Ban List", pevt_banlist_help, N_("-%C10-%C11-%O$t$1 Banlist: %C3$4 %C4$2 %C5$3%O"), 4, NULL},
-	{"Channel List", pevt_generic_none_help, N_("%UChannel          Users   Topic%O"), 0, NULL},
-	{"Notify List",  pevt_generic_none_help, N_("%C08,02 %B-- Notify List --------------- %O"), 0, NULL},
-	{"DCC List", pevt_generic_none_help, N_("%C8,2 Type  To/From    Status  Size    Pos     File    %O%010%B%C9--------------------------------------------------%O"), 0, NULL},
-	{NULL, NULL, NULL, 0, NULL}, /* XP_IF_SEND */
-	{NULL, NULL, NULL, 0, NULL}, /* XP_IF_RECV */
-	{"Channel Notice", pevt_channotice_help, N_("%C12-%C13$1/$2%C12-%O$t$3%O"), 3, NULL},
-
-};
-
-int
-text_event (int i)
-{
-	if (te[i].name == NULL)
-		return 0;
-	else
-		return 1;
-}
+#include "textevents.h"
 
 static void
 pevent_load_defaults ()
@@ -1145,8 +866,6 @@ pevent_load_defaults ()
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		if (!text_event (i))
-			continue;
 		len = strlen (_(te[i].def));
 		len++;
 		if (pntevts_text[i])
@@ -1164,8 +883,6 @@ pevent_make_pntevts ()
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		if (!text_event (i))
-			continue;
 		if (pntevts[i] != NULL)
 			free (pntevts[i]);
 		if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
@@ -1241,11 +958,6 @@ pevent_find (char *name, int *i_i)
 			j = 0;
 		if (j == i)
 			return -1;
-		if (!text_event (j))
-		{
-			j++;
-			continue;
-		}
 		if (strcmp (te[j].name, name) == 0)
 		{
 			*i_i = j;
@@ -1273,7 +985,7 @@ pevent_load (char *filename)
 	if (filename == NULL)
 		snprintf (buf, 1000, "%s/pevents.conf", get_xdir ());
    else
-      strncpy (buf, filename, 1000);
+      safe_strcpy (buf, filename, 1000);
 
 	fd = open (buf, O_RDONLY | OFLAGS);
 	free (buf);
@@ -1328,7 +1040,6 @@ pevent_load (char *filename)
 	return 0;
 }
 
-
 static void
 pevent_check_all_loaded ()
 {
@@ -1336,8 +1047,6 @@ pevent_check_all_loaded ()
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		if (!text_event (i))
-			continue;
 		if (pntevts_text[i] == NULL)
 		{
 			/*printf ("%s\n", te[i].name);
@@ -1353,13 +1062,8 @@ pevent_check_all_loaded ()
 void
 load_text_events ()
 {
-	/* I don't free these as the only time when we don't need them
-	   is once XChat has exit(2)'ed, so why bother ?? --AGL */
-
-	pntevts_text = malloc (sizeof (char *) * (NUM_XP));
-	memset (pntevts_text, 0, sizeof (char *) * (NUM_XP));
-	pntevts = malloc (sizeof (char *) * (NUM_XP));
-	memset (pntevts, 0, sizeof (char *) * (NUM_XP));
+	memset (&pntevts_text, 0, sizeof (char *) * (NUM_XP));
+	memset (&pntevts, 0, sizeof (char *) * (NUM_XP));
 
 	if (pevent_load (NULL))
 		pevent_load_defaults ();
@@ -1368,7 +1072,7 @@ load_text_events ()
 }
 
 static void
-display_event (char *i, struct session *sess, int numargs, char **args)
+display_event (char *i, session *sess, int numargs, char **args)
 {
 	int len, oi, ii;
 	char *ar, o[4096], d, a, done_all = FALSE;
@@ -1404,7 +1108,7 @@ display_event (char *i, struct session *sess, int numargs, char **args)
 							a, numargs, i);
 				break;
 			}
-			ar = args[(int) a];
+			ar = args[(int) a + 1];
 			if (ar == NULL)
 			{
 				printf ("Error args[a] == NULL in display_event\n");
@@ -1420,19 +1124,19 @@ display_event (char *i, struct session *sess, int numargs, char **args)
 			done_all = TRUE;
 			continue;
 		case 3:
-			if (sess->type == SESS_DIALOG)
+/*			if (sess->type == SESS_DIALOG)
 			{
 				if (prefs.dialog_indent_nicks)
 					o[oi++] = '\t';
 				else
 					o[oi++] = ' ';
 			} else
-			{
+			{*/
 				if (prefs.indent_nicks)
 					o[oi++] = '\t';
 				else
 					o[oi++] = ' ';
-			}
+			/*}*/
 			break;
 		}
 	}
@@ -1443,7 +1147,7 @@ display_event (char *i, struct session *sess, int numargs, char **args)
 }
 
 int
-pevt_build_string (char *input, char **output, int *max_arg)
+pevt_build_string (const char *input, char **output, int *max_arg)
 {
 	struct pevt_stage1 *s = NULL, *base = NULL, *last = NULL, *next;
 	int clen;
@@ -1530,7 +1234,6 @@ pevt_build_string (char *input, char **output, int *max_arg)
 		if (d == 't')
 		{
 			/* Tab - if tabnicks is set then write '\t' else ' ' */
-			/*s = g_new (struct pevt_stage1, 1); */
 			s = (struct pevt_stage1 *) malloc (sizeof (struct pevt_stage1));
 			if (base == NULL)
 				base = s;
@@ -1622,79 +1325,44 @@ pevt_build_string (char *input, char **output, int *max_arg)
 static void
 play_wave (char *file)
 {
-	char *buf = malloc (512);
+	char buf[512];
 
-	snprintf (buf, 512, "%s/%s", prefs.sounddir, file);
+	snprintf (buf, sizeof (buf), "%s/%s", prefs.sounddir, file);
 	if (access (buf, R_OK) == 0)
 	{
-		snprintf (buf, 512, "%s %s/%s", prefs.soundcmd, prefs.sounddir, file);
+		snprintf (buf, sizeof (buf), "%s %s/%s", prefs.soundcmd, prefs.sounddir, file);
+		buf[sizeof (buf) - 1] = '\0';
 		xchat_exec (buf);
-	}
-	free (buf);
-}
-
-static int
-textsignal_handler (struct session *sess, void *b, void *c,
-						  void *d, void *e, char f)
-{
-	/* This handler *MUST* always be the last in the chain
-	   because it doesn't call the next handler
-	 */
-
-	char *args[8];
-	int numargs;
-
-	if (!sess)
-		return 0;
-
-	if (!text_event (current_signal))
+	} else
 	{
-		printf ("error, textsignal_handler passed non TE signal (%d)\n",
-				  current_signal);
-		abort ();
+		snprintf (buf, sizeof (buf), "Cannot read %s/%s", prefs.sounddir, file);
+		fe_message (buf, FALSE);
 	}
-	numargs = te[current_signal].num_args;
-
-#ifdef USE_PERL
-	if (perl_print (te[current_signal].name, sess, b, c, d, e))
-		return TRUE;
-#endif
-
-	if (te[current_signal].sound)
-		play_wave (te[current_signal].sound);
-
-	args[0] = b;
-	args[1] = c;
-	args[2] = d;
-	args[3] = e;
-
-	display_event (pntevts[(int) current_signal], sess, numargs, args);
-	return 0;
 }
 
+/* called by EMIT_SIGNAL macro */
+
 void
-printevent_setup ()
+text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 {
-	int evt;
-	struct xp_signal *sig;
+	char *word[PDIWORDS];
+	int i;
 
-	sig = (struct xp_signal *) malloc (sizeof (struct xp_signal));
+	word[0] = te[index].name;
+	word[1] = a;
+	word[2] = b;
+	word[3] = c;
+	word[4] = d;
+	for (i = 5; i < PDIWORDS; i++)
+		word[i] = "\000";
 
-	sig->signal = -1;
-	sig->naddr = NULL;
-	sig->callback = XP_CALLBACK (textsignal_handler);
+	if (plugin_emit_print (sess, word))
+		return;
 
-#ifdef USE_PLUGIN
-	sig->mod = NULL;
-#endif
+	if (te[index].sound)
+		play_wave (te[index].sound);
 
-	for (evt = 0; evt < NUM_XP; evt++)
-	{
-		if (!text_event (evt))
-			continue;
-		g_assert (!sigroots[evt]);
-		sigroots[evt] = g_slist_prepend (sigroots[evt], sig);
-	}
+	display_event (pntevts[index], sess, te[index].num_args, word);
 }
 
 void
@@ -1706,7 +1374,7 @@ pevent_save (char *fn)
 	if (!fn)
 		snprintf (buf, sizeof (buf), "%s/pevents.conf", get_xdir ());
 	else
-		snprintf (buf, sizeof (buf), "%s", fn);
+		safe_strcpy (buf, fn, sizeof (buf));
 	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
 	if (fd < 0)
 	{
@@ -1722,8 +1390,6 @@ pevent_save (char *fn)
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		if (!text_event (i))
-			continue;
 		write (fd, buf, snprintf (buf, sizeof (buf),
 										  "event_name=%s\n", te[i].name));
 		write (fd, buf, snprintf (buf, sizeof (buf),
