@@ -31,30 +31,6 @@
 #include "util.h"
 
 
-static void update_entry (struct session *sess, struct User *user);
-
-
-int
-userlist_add_hostname (struct session *sess, char *nick, char *hostname,
-							  char *realname, char *servername)
-{
-	struct User *user;
-
-	user = find_name (sess, nick);
-	if (user && !user->hostname)
-	{
-		user->hostname = strdup (hostname);
-		if (!user->realname)
-			user->realname = strdup (realname);
-		if (!user->servername)
-			user->servername = strdup (servername);
-		if (prefs.showhostname_in_userlist)
-			update_entry (sess, user);
-		return 1;
-	}
-	return 0;
-}
-
 static int
 nick_cmp_az_ops (struct User *user1, struct User *user2)
 {
@@ -75,7 +51,7 @@ nick_cmp_az_ops (struct User *user1, struct User *user2)
 		}
 	}
 
-	return strcasecmp (user1->nick, user2->nick);
+	return rfc_casecmp (user1->nick, user2->nick);
 }
 
 static int
@@ -86,86 +62,16 @@ nick_cmp (struct User *user1, struct User *user2)
 	case 0:
 		return nick_cmp_az_ops (user1, user2);
 	case 1:
-		return strcasecmp (user1->nick, user2->nick);
+		return rfc_casecmp (user1->nick, user2->nick);
 	case 2:
 		return -1 * nick_cmp_az_ops (user1, user2);
 	case 3:
-		return -1 * strcasecmp (user1->nick, user2->nick);
+		return -1 * rfc_casecmp (user1->nick, user2->nick);
 	default:
 		return 1;
 	}
 }
 
-void
-free_userlist (session *sess)
-{
-	struct User *user;
-	GSList *list = sess->userlist;
-
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (user->realname)
-			free (user->realname);
-		if (user->hostname)
-			free (user->hostname);
-		if (user->servername)
-			free (user->servername);
-		free (user);
-		list = g_slist_remove (list, user);
-	}
-	sess->userlist = 0;
-	sess->ops = 0;
-	sess->hops = 0;
-	sess->voices = 0;
-	sess->total = 0;
-}
-
-void
-clear_user_list (session *sess)
-{
-	fe_userlist_clear (sess);
-	free_userlist (sess);
-	fe_userlist_numbers (sess);
-}
-
-struct User *
-find_name (struct session *sess, char *name)
-{
-	struct User *user;
-	GSList *list;
-
-	list = sess->userlist;
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (!strcasecmp (user->nick, name))
-			return user;
-		list = list->next;
-	}
-	return FALSE;
-}
-
-struct User *
-find_name_global (struct server *serv, char *name)
-{
-	struct User *user;
-	session *sess;
-	GSList *list = sess_list;
-	while (list)
-	{
-		sess = (session *) list->data;
-		if (sess->server == serv)
-		{
-			user = find_name (sess, name);
-			if (user)
-				return user;
-		}
-		list = list->next;
-	}
-	return 0;
-}
-
 static int
 userlist_insertname (struct session *sess, struct User *newuser)
 {
@@ -200,6 +106,101 @@ update_entry (struct session *sess, struct User *user)
 	fe_userlist_numbers (sess);
 }
 
+int
+userlist_add_hostname (struct session *sess, char *nick, char *hostname,
+							  char *realname, char *servername)
+{
+	struct User *user;
+
+	user = find_name (sess, nick);
+	if (user && !user->hostname)
+	{
+		user->hostname = strdup (hostname);
+		if (!user->realname)
+			user->realname = strdup (realname);
+		if (!user->servername)
+			user->servername = strdup (servername);
+		if (prefs.showhostname_in_userlist)
+			update_entry (sess, user);
+		return 1;
+	}
+	return 0;
+}
+
+static void
+free_user (struct User *user, GSList **list)
+{
+	if (user->realname)
+		free (user->realname);
+	if (user->hostname)
+		free (user->hostname);
+	if (user->servername)
+		free (user->servername);
+	free (user);
+	*list = g_slist_remove (*list, user);
+}
+
+void
+free_userlist (session *sess)
+{
+	struct User *user;
+
+	while (sess->userlist)
+	{
+		user = (struct User *) sess->userlist->data;
+		free_user (user, &sess->userlist);
+	}
+	sess->ops = 0;
+	sess->hops = 0;
+	sess->voices = 0;
+	sess->total = 0;
+}
+
+void
+clear_user_list (session *sess)
+{
+	fe_userlist_clear (sess);
+	free_userlist (sess);
+	fe_userlist_numbers (sess);
+}
+
+struct User *
+find_name (struct session *sess, char *name)
+{
+	struct User *user;
+	GSList *list;
+
+	list = sess->userlist;
+	while (list)
+	{
+		user = (struct User *) list->data;
+		if (!rfc_casecmp (user->nick, name))
+			return user;
+		list = list->next;
+	}
+	return FALSE;
+}
+
+struct User *
+find_name_global (struct server *serv, char *name)
+{
+	struct User *user;
+	session *sess;
+	GSList *list = sess_list;
+	while (list)
+	{
+		sess = (session *) list->data;
+		if (sess->server == serv)
+		{
+			user = find_name (sess, name);
+			if (user)
+				return user;
+		}
+		list = list->next;
+	}
+	return 0;
+}
+
 static void
 update_counts (session *sess, struct User *user, char prefix,
 					int level, int offset)
@@ -260,7 +261,7 @@ ul_update_entry (session *sess, char *name, char mode, char sign)
 	}
 
 	/* now what is this users highest prefix? e.g. @ for ops */
-	user->prefix = get_nick_prefix (sess->server, user->access);
+	user->prefix[0] = get_nick_prefix (sess->server, user->access);
 
 	/* update the various counts using the CHANGED prefix only */
 	update_counts (sess, user, prefix, level, offset);
@@ -298,14 +299,7 @@ sub_name (struct session *sess, char *name)
 	fe_userlist_numbers (sess);
 	fe_userlist_remove (sess, user);
 
-	if (user->realname)
-		free (user->realname);
-	if (user->hostname)
-		free (user->hostname);
-	if (user->servername)
-		free (user->servername);
-	free (user);
-	sess->userlist = g_slist_remove (sess->userlist, user);
+	free_user (user, &sess->userlist);
 
 	return TRUE;
 }
@@ -329,11 +323,10 @@ add_name (struct session *sess, char *name, char *hostname)
 	memset (user, 0, sizeof (struct User));
 
 	user->access = acc;
-	user->prefix = ' ';
 
 	/* assume first char is the highest level nick prefix */
 	if (prefix_chars)
-		user->prefix = name[0];
+		user->prefix[0] = name[0];
 
 	sess->total++;
 
@@ -351,7 +344,7 @@ add_name (struct session *sess, char *name, char *hostname)
 	safe_strcpy (user->nick, name, NICKLEN);
 	row = userlist_insertname (sess, user);
 
-	fe_userlist_insert (sess, user, row);
+	fe_userlist_insert (sess, user, row, FALSE);
 	fe_userlist_numbers (sess);
 }
 
