@@ -187,6 +187,12 @@ xchat_free (void *buf, char *file, int line)
 {
 	struct mem_block *cur, *last;
 
+	if (buf == NULL)
+	{
+		printf ("%s:%d \033[33mTried to free NULL\033[m\n", file, line);
+		return;
+	}
+
 	last = NULL;
 	cur = mroot;
 	while (cur)
@@ -256,7 +262,7 @@ path_part (char *file, char *path, int pathlen)
 char *				/* like strstr(), but nocase */
 nocasestrstr (char *s, char *wanted)
 {
-	register const size_t len = strlen (wanted);
+	register const int len = strlen (wanted);
 
 	if (len == 0)
 		return (char *)s;
@@ -387,6 +393,22 @@ strip_color (unsigned char *text)
 	return new_str;
 }
 
+char *
+strip_spaces (char *buf)
+{
+	char *s = buf;
+	size_t len = strlen (buf) + 1;
+	size_t ofs;
+
+	while (s[ofs = strcspn (s, " \t")])
+	{
+		size_t cnt = strspn (s+= ofs, " \t");
+		memmove (s, s + cnt, len-= ofs + cnt);
+	}
+
+	return buf;
+}
+
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 
 static void
@@ -1078,8 +1100,8 @@ util_exec (char *cmd)
 #else
 	spawnvp (_P_DETACH, argv[0], argv);
 	free (argv);
-#endif
 	return 0;
+#endif
 }
 
 unsigned long
