@@ -14,9 +14,6 @@
 #undef tolower
 #define tolower(c) (tolowertab[(unsigned char)(c)])
 
-#define strcasecmp      rfc_casecmp
-#define strncasecmp     rfc_ncasecmp
-
 extern const unsigned char tolowertab[];
 
 int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr);
@@ -27,7 +24,6 @@ char *file_part (char *file);
 int for_files (char *dirname, char *mask, void callback (char *file));
 int rfc_casecmp (char *, char *);
 int rfc_ncasecmp (char *, char *, int);
-void toupperStr (char *str);
 void tolowerStr (char *str);
 int buf_get_line (char *, char **, int *, int len);
 char *nocasestrstr (char *text, char *tofind);
@@ -35,6 +31,7 @@ char *country (char *);
 char *get_cpu_str (void);
 int util_exec (char *cmd);
 unsigned char *strip_color (unsigned char *text);
+char *strip_spaces (char *buf);
 char *errorstring (int err);
 int waitline (int sok, char *buf, int bufsize);
 unsigned long make_ping_time (void);
