@@ -40,10 +40,10 @@
 #include "plugin.h"
 #include "notify.h"
 #include "server.h"
-#include "pythonc.h"
 #include "outbound.h"
 #include "text.h"
-#include "perlc.h"
+#include "proto-irc.h"
+#include "plugin-timer.h"
 #include "xchatc.h"
 
 #ifdef USE_OPENSSL
@@ -51,10 +51,6 @@
 #include "ssl.h"
 #endif
 
-#ifdef USE_JCODE
-#include "jcode.h"
-#endif
-
 GSList *popup_list = 0;
 GSList *button_list = 0;
 GSList *dlgbutton_list = 0;
@@ -72,9 +68,10 @@ static GSList *away_list = 0;
 static int in_xchat_exit = FALSE;
 int xchat_is_quitting = FALSE;
 int auto_connect = TRUE;
+char *connect_url = NULL;
 
 struct session *current_tab;
-struct session *menu_sess = 0;
+struct session *current_sess = 0;
 struct xchatprefs prefs;
 
 #ifdef USE_OPENSSL
@@ -82,212 +79,31 @@ SSL_CTX *ctx = NULL;
 void ssl_cb_info (SSL * s, int where, int ret);
 #endif
 
-static void free_away_messages (server *serv);
-
-
-/* actually send to the socket. This might do a character translation or
-   send via SSL */
-
-static int
-tcp_send_real (struct server *serv, char *buf, int len)
-{
-	int ret = 1;
-	unsigned char *tbuf = buf;
-#ifdef USE_TRANS
-#define TRANS_STAT_BUF_LEN 1024
-	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
-
-	if (prefs.use_trans)
-	{
-		if (len >= TRANS_STAT_BUF_LEN)
-			tbuf = malloc (len + 1);
-		else
-			tbuf = sbuf;
-		if (!tbuf)
-			return -1;
-		memcpy (tbuf, buf, len); /* buf is not necessarily NULL terminated! */
-		tbuf[len] = 0;
-		user2serv (tbuf);
-	}
-#endif
-
-	fe_add_rawlog (serv, tbuf, len, TRUE);
-
-	if (!EMIT_SIGNAL (XP_IF_SEND, GINT_TO_POINTER (serv->sok), tbuf,
-							GINT_TO_POINTER (len), NULL, NULL, 0))
-	{
-#ifdef USE_OPENSSL
-		if (!serv->ssl)
-			ret = send (serv->sok, tbuf, len, 0);
-		else
-			ret = _SSL_send (serv->ssl, tbuf, len);
-#else
-		ret = send (serv->sok, tbuf, len, 0);
-#endif
-	}
-
-#ifdef USE_TRANS
-	if (prefs.use_trans)
-	{
-		if (tbuf != sbuf)
-			free (tbuf);
-	}
-#endif
-
-	return ret;
-}
-
-/* new throttling system, uses the same method as the Undernet
-   ircu2.10 server; under test, a 200-line paste didn't flood
-   off the client */
-
-static int
-tcp_send_queue (struct server *serv)
-{
-	char *buf, *p;
-	int len, i, pri;
-	GSList *list;
-	time_t now = time (0);
-
-	/* did the server close since the timeout was added? */
-	if (!is_server (serv))
-		return 0;
-
-	/* first try priority 1 entries */
-	pri = 1;
-	while (pri >= 0)
-	{
-		list = serv->outbound_queue;
-		while (list)
-		{
-			buf = (char *) list->data;
-			if (buf[0] == pri)
-			{
-				buf++;	/* skip the priority byte */
-				len = strlen (buf);
-
-				if (serv->next_send < now)
-					serv->next_send = now;
-				if (serv->next_send - now >= 10)
-					return 1;				  /* don't remove the timeout handler */
-
-				for (p = buf, i = len; i && *p != ' '; p++, i--);
-				serv->next_send += (2 + i / 120);
-				serv->sendq_len -= len;
-				fe_set_throttle (serv);
-
-				tcp_send_real (serv, buf, len);
-
-				buf--;
-				serv->outbound_queue = g_slist_remove (serv->outbound_queue, buf);
-				free (buf);
-				list = serv->outbound_queue;
-			} else
-			{
-				list = list->next;
-			}
-		}
-		/* now try pri 0 */
-		pri--;
-	}
-	return 0;						  /* remove the timeout handler */
-}
-
-int
-tcp_send_len (server *serv, char *buf, int len)
-{
-	char *dbuf;
-	int noqueue = !serv->outbound_queue;
-
-#ifdef USE_JCODE
-	if (prefs.kanji_conv)
-	{
-		buf = kanji_conv_auto (buf, "ISO-2022-JP");
-		if (!buf)
-			return 0;
-		len = strlen (buf);
-	}
-
-	if (!prefs.throttle)
-	{
-		int ret = tcp_send_real (serv, buf, len);
-		if (prefs.kanji_conv)
-			free (buf);
-		return ret;
-	}
-#else
-
-	if (!prefs.throttle)
-		return tcp_send_real (serv, buf, len);
-
-#endif
-
-	dbuf = malloc (len + 2);	/* first byte is the priority */
-	dbuf[0] = 1;	/* pri 1 for most things */
-	memcpy (dbuf + 1, buf, len);
-	dbuf[len + 1] = 0;
-
-#ifdef USE_JCODE
-	if (prefs.kanji_conv)
-		free (buf);
-#endif
-
-	/* privmsg and notice get a lower priority */
-	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
-		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
-		dbuf[0] = 0;
-
-	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
-	serv->sendq_len += len; /* tcp_send_queue uses strlen */
-
-	if (tcp_send_queue (serv) && noqueue)
-		fe_timeout_add (500, tcp_send_queue, serv);
-
-	return 1;
-}
-
-int
-tcp_send (struct server *serv, char *buf)
-{
-	return tcp_send_len (serv, buf, strlen (buf));
-}
-
-static int
-is_in_list (GSList *list, void *data)
-{
-	while (list)
-	{
-		if (list->data == data)
-			return TRUE;
-		list = list->next;
-	}
-	return FALSE;
-}
 
 int
 is_server (server * serv)
 {
-	return is_in_list (serv_list, serv);
+	return g_slist_find (serv_list, serv) ? 1 : 0;
 }
 
 int
 is_session (session * sess)
 {
-	return is_in_list (sess_list, sess);
+	return g_slist_find (sess_list, sess) ? 1 : 0;
 }
 
-struct session *
-find_dialog (struct server *serv, char *nick)
+session *
+find_dialog (server *serv, char *nick)
 {
 	GSList *list = sess_list;
-	struct session *sess;
+	session *sess;
 
 	while (list)
 	{
-		sess = (struct session *) list->data;
+		sess = list->data;
 		if (sess->server == serv && sess->type == SESS_DIALOG)
 		{
-			if (!strcasecmp (nick, sess->channel))
+			if (!rfc_casecmp (nick, sess->channel))
 				return (sess);
 		}
 		list = list->next;
@@ -296,17 +112,16 @@ find_dialog (struct server *serv, char *nick)
 }
 
 session *
-find_channel (char *chan, server *serv)
+find_session_from_channel (char *chan, server *serv)
 {
 	session *sess;
 	GSList *list = sess_list;
 	while (list)
 	{
 		sess = list->data;
-		if (sess->type != SESS_SHELL && sess->type != SESS_DIALOG &&
-			 !strcasecmp (chan, sess->channel))
+		if (!serv || serv == sess->server)
 		{
-			if (!serv || serv == sess->server)
+			if (!rfc_casecmp (chan, sess->channel))
 				return sess;
 		}
 		list = list->next;
@@ -314,78 +129,14 @@ find_channel (char *chan, server *serv)
 	return 0;
 }
 
-#ifndef WIN32
-
-static int
-mail_items (char *file)
-{
-	FILE *fp;
-	int items;
-	char buf[512];
-
-	fp = fopen (file, "r");
-	if (!fp)
-		return 1;
-
-	items = 0;
-	while (fgets (buf, sizeof buf, fp))
-	{
-		if (!strncmp (buf, "From ", 5))
-			items++;
-	}
-	fclose (fp);
-
-	return items;
-}
-
 static void
-xchat_mail_check (void)
-{
-	static int last_size = -1;
-	int size;
-	struct stat st;
-	char buf[512];
-	char *maildir;
-
-	maildir = getenv ("MAIL");
-	if (!maildir)
-	{
-		snprintf (buf, sizeof (buf), "/var/spool/mail/%s", g_get_user_name ());
-		maildir = buf;
-	}
-
-	if (stat (maildir, &st) < 0)
-		return;
-
-	size = st.st_size;
-
-	if (last_size == -1)
-	{
-		last_size = size;
-		return;
-	}
-
-	if (size > last_size)
-	{
-		sprintf (buf, "%d", mail_items (maildir));
-		sprintf (buf + 16, "%d", size);
-		if (menu_sess && menu_sess->type != SESS_SHELL)
-			EMIT_SIGNAL (XP_TE_NEWMAIL, menu_sess, buf, buf + 16, NULL, NULL, 0);
-	}
-
-	last_size = size;
-}
-
-#endif
-
-static int
 lagcheck_update (void)
 {
 	server *serv;
 	GSList *list = serv_list;
 	
 	if (!prefs.lagometer)
-		return 1;
+		return;
 
 	while (list)
 	{
@@ -395,7 +146,6 @@ lagcheck_update (void)
 
 		list = list->next;
 	}
-	return 1;
 }
 
 void
@@ -404,7 +154,7 @@ lag_check (void)
 	server *serv;
 	GSList *list = serv_list;
 	unsigned long tim;
-	char tbuf[256];
+	char tbuf[128];
 	time_t now = time (0);
 	int lag;
 
@@ -419,13 +169,12 @@ lag_check (void)
 			if (prefs.pingtimeout && lag > prefs.pingtimeout && lag > 0)
 			{
 				sprintf (tbuf, "%d", lag);
-				EMIT_SIGNAL (XP_TE_PINGTIMEOUT, serv->front_session, tbuf, NULL,
+				EMIT_SIGNAL (XP_TE_PINGTIMEOUT, serv->server_session, tbuf, NULL,
 								 NULL, NULL, 0);
 				auto_reconnect (serv, FALSE, -1);
 			} else
 			{
-				sprintf (tbuf, "PING LAG%lu\r\n", tim);
-				tcp_send (serv, tbuf);
+				tcp_sendf (serv, "PING LAG%lu\r\n", tim);
 				serv->lag_sent = tim;
 				fe_set_lag (serv, -1);
 			}
@@ -435,24 +184,22 @@ lag_check (void)
 }
 
 static int
-xchat_misc_checks (void)		  /* this gets called every 2 seconds */
+xchat_misc_checks (void)		/* this gets called every 1/2 second */
 {
 	static int count = 0;
 
-	dcc_check_timeouts ();
-
 	count++;
 
-	if (count == 7 && prefs.lagometer)	/* every 30 seconds */
-		lag_check ();
+	lagcheck_update ();			/* every 500ms */
 
-	if (count == 15)							/* every 30 seconds */
+	if (count % 2)
+		dcc_check_timeouts ();	/* every 1 second */
+
+	if (count >= 60)				/* every 30 seconds */
 	{
+		if (prefs.lagometer)
+			lag_check ();
 		count = 0;
-#ifndef WIN32
-		if (prefs.mail_check)
-			xchat_mail_check ();
-#endif
 	}
 
 	return 1;
@@ -470,22 +217,24 @@ irc_init (session *sess)
 
 	done_init = TRUE;
 
-#ifdef USE_PYTHON
-	pys_init ();
-#endif
 #ifdef USE_PLUGIN
-	module_setup ();
-#endif
-#ifdef USE_PERL
-	perl_auto_load (sess);
+	plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
 #endif
+	plugin_add (sess, NULL, NULL, timer_plugin_init, NULL, NULL, FALSE);
 
 	if (prefs.notify_timeout)
 		notify_tag = fe_timeout_add (prefs.notify_timeout * 1000,
 											  notify_checklist, 0);
 
-	fe_timeout_add (2000, xchat_misc_checks, 0);
-	fe_timeout_add (500, lagcheck_update, 0);
+	fe_timeout_add (500, xchat_misc_checks, 0);
+
+	if (connect_url != NULL)
+	{
+		char buf[512];
+		snprintf (buf, sizeof (buf), "server %s", connect_url);
+		handle_command (sess, buf, FALSE);
+		free (connect_url);
+	}
 }
 
 static session *
@@ -499,7 +248,6 @@ new_session (server *serv, char *from, int type)
 	sess->server = serv;
 	sess->logfd = -1;
 	sess->type = type;
-	sess->current_modes = strdup ("");
 
 	if (from != NULL)
 		safe_strcpy (sess->channel, from, CHANLEN);
@@ -522,14 +270,11 @@ set_server_defaults (server *serv)
 		free (serv->nick_prefixes);
 	if (serv->nick_modes)
 		free (serv->nick_modes);
-	if (serv->network)
-		free (serv->network);
 
 	serv->chantypes = strdup ("#&!+");
 	serv->chanmodes = strdup ("beI,k,l");
 	serv->nick_prefixes = strdup ("@%+");
 	serv->nick_modes = strdup ("ohv");
-	serv->network = strdup (serv->servername);
 
 	serv->nickcount = 1;
 	serv->end_of_motd = FALSE;
@@ -546,6 +291,9 @@ new_server (void)
 	serv = malloc (sizeof (struct server));
 	memset (serv, 0, sizeof (struct server));
 
+	/* use proto-irc.c functions */
+	proto_fill_her_up (serv);
+
 	serv->sok = -1;
 	strcpy (serv->nick, prefs.nick1);
 	set_server_defaults (serv);
@@ -564,9 +312,6 @@ new_ircwindow (server *serv, char *name, int type)
 
 	switch (type)
 	{
-	case SESS_SHELL:
-		sess = new_session (new_server (), name, SESS_SHELL);
-		break;
 	case SESS_SERVER:
 		serv = new_server ();
 		if (prefs.use_server_tab)
@@ -575,6 +320,8 @@ new_ircwindow (server *serv, char *name, int type)
 			prefs.hideuserlist = 1;
 			sess = new_session (serv, name, SESS_SERVER);
 			prefs.hideuserlist = oldh;
+
+			serv->server_session = sess;
 			serv->front_session = sess;
 		} else
 		{
@@ -595,6 +342,53 @@ new_ircwindow (server *serv, char *name, int type)
 	return sess;
 }
 
+static void
+free_away_messages (server *serv)
+{
+	GSList *list, *next;
+	struct away_msg *away;
+
+	list = away_list;
+	while (list)
+	{
+		away = list->data;
+		next = list->next;
+		if (away->server == serv)
+		{
+			away_list = g_slist_remove (away_list, away);
+			if (away->message)
+				free (away->message);
+			free (away);
+			next = away_list;
+		}
+		list = next;
+	}
+}
+
+void
+save_away_message (struct server *serv, char *nick, char *msg)
+{
+	struct away_msg *away = find_away_message (serv, nick);
+
+	if (away)						  /* Change message for known user */
+	{
+		if (away->message)
+			free (away->message);
+		away->message = strdup (msg);
+	} else
+		/* Create brand new entry */
+	{
+		away = malloc (sizeof (struct away_msg));
+		if (away)
+		{
+			away->server = serv;
+			safe_strcpy (away->nick, nick, sizeof (away->nick));
+			away->message = strdup (msg);
+			away_list = g_slist_prepend (away_list, away);
+		}
+	}
+}
+
 static void
 kill_server_callback (server * serv)
 {
@@ -605,14 +399,13 @@ kill_server_callback (server * serv)
 	serv_list = g_slist_remove (serv_list, serv);
 
 	dcc_notify_kill (serv);
-	flush_server_queue (serv);
+	server_flush_queue (serv);
 	free_away_messages (serv);
 
 	free (serv->nick_modes);
 	free (serv->nick_prefixes);
 	free (serv->chanmodes);
 	free (serv->chantypes);
-	free (serv->network);
 	if (serv->bad_nick_prefixes)
 		free (serv->bad_nick_prefixes);
 	if (serv->last_away_reason)
@@ -625,13 +418,6 @@ kill_server_callback (server * serv)
 	notify_cleanup ();
 }
 
-static void
-log_notify_kill (session * sess)
-{
-	if (sess->logfd != -1)
-		end_logging (sess->logfd);
-}
-
 static void
 exec_notify_kill (session * sess)
 {
@@ -682,7 +468,7 @@ send_quit_or_part (session * killsess)
 		{
 			if (!killserv->sent_quit)
 			{
-				flush_server_queue (killserv);
+				server_flush_queue (killserv);
 				server_sendquit (killsess);
 				killserv->sent_quit = TRUE;
 			}
@@ -706,6 +492,9 @@ kill_session_callback (session * killsess)
 	if (current_tab == killsess)
 		current_tab = NULL;
 
+	if (killserv->server_session == killsess)
+		killserv->server_session = NULL;
+
 	if (killserv->front_session == killsess)
 	{
 		/* front_session is closed, find a valid replacement */
@@ -717,6 +506,8 @@ kill_session_callback (session * killsess)
 			if (sess != killsess && sess->server == killserv)
 			{
 				killserv->front_session = sess;
+				if (!killserv->server_session)
+					killserv->server_session = sess;
 				break;
 			}
 			list = list->next;
@@ -732,19 +523,21 @@ kill_session_callback (session * killsess)
 
 	exec_notify_kill (killsess);
 
-#ifdef USE_PERL
+/*#ifdef USE_PERL
 	perl_notify_kill (killsess);
-#endif
+#endif*/
 
-	log_notify_kill (killsess);
+	log_close (killsess);
 
 	send_quit_or_part (killsess);
 
 	history_free (&killsess->history);
 	free (killsess->gui);
+	free (killsess->res);
 	if (killsess->topic)
 		free (killsess->topic);
-	free (killsess->current_modes);
+	if (killsess->current_modes)
+		free (killsess->current_modes);
 	free (killsess);
 
 	if (!sess_list && !in_xchat_exit)
@@ -782,136 +575,91 @@ find_away_message (struct server *serv, char *nick)
 	while (list)
 	{
 		away = (struct away_msg *) list->data;
-		if (away->server == serv && !strcasecmp (nick, away->nick))
+		if (away->server == serv && !rfc_casecmp (nick, away->nick))
 			return away;
 		list = list->next;
 	}
 	return 0;
 }
 
-static void
-free_away_messages (server *serv)
-{
-	GSList *list, *next;
-	struct away_msg *away;
-
-	list = away_list;
-	while (list)
-	{
-		away = list->data;
-		next = list->next;
-		if (away->server == serv)
-		{
-			away_list = g_slist_remove (away_list, away);
-			if (away->message)
-				free (away->message);
-			free (away);
-			next = away_list;
-		}
-		list = next;
-	}
-}
-
-void
-save_away_message (struct server *serv, char *nick, char *msg)
-{
-	struct away_msg *away = find_away_message (serv, nick);
-
-	if (away)						  /* Change message for known user */
-	{
-		if (away->message)
-			free (away->message);
-		away->message = strdup (msg);
-	} else
-		/* Create brand new entry */
-	{
-		away = malloc (sizeof (struct away_msg));
-		if (away)
-		{
-			away->server = serv;
-			safe_strcpy (away->nick, nick, sizeof (away->nick));
-			away->message = strdup (msg);
-			away_list = g_slist_prepend (away_list, away);
-		}
-	}
-}
+#define XTERM "rxvt"
 
 #define defaultconf_ctcp \
-	"NAME TIME\n"				"CMD /nctcp %s TIME %t\n\n"\
-	"NAME PING\n"				"CMD /nctcp %s PING %d\n\n"
+	"NAME TIME\n"				"CMD nctcp %s TIME %t\n\n"\
+	"NAME PING\n"				"CMD nctcp %s PING %d\n\n"
 
-#define defaultconf_popup _( \
+#define defaultconf_popup \
 	"NAME SUB\n"				"CMD Direct Client-To-Client\n\n"\
-	"NAME Send File\n"		"CMD /dcc send %s\n\n"\
-	"NAME Offer Chat\n"		"CMD /dcc chat %s\n\n"\
-	"NAME Abort Chat\n"		"CMD /dcc close chat %s\n\n"\
+		"NAME Send File\n"	"CMD dcc send %s\n\n"\
+		"NAME Offer Chat\n"	"CMD dcc chat %s\n\n"\
+		"NAME Abort Chat\n"	"CMD dcc close chat %s\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD CTCP\n\n"\
-	"NAME Version\n"			"CMD /ctcp %s VERSION\n\n"\
-	"NAME Userinfo\n"			"CMD /ctcp %s USERINFO\n\n"\
-	"NAME Clientinfo\n"		"CMD /ctcp %s CLIENTINFO\n\n"\
-	"NAME Ping\n"				"CMD /ping %s\n\n"\
-	"NAME Time\n"				"CMD /ctcp %s TIME\n\n"\
-	"NAME Finger\n"			"CMD /ctcp %s FINGER\n\n"\
-	"NAME XDCC List\n"		"CMD /ctcp %s XDCC LIST\n\n"\
-	"NAME CDCC List\n"		"CMD /ctcp %s CDCC LIST\n\n"\
+		"NAME Version\n"		"CMD ctcp %s VERSION\n\n"\
+		"NAME Userinfo\n"		"CMD ctcp %s USERINFO\n\n"\
+		"NAME Clientinfo\n"	"CMD ctcp %s CLIENTINFO\n\n"\
+		"NAME Ping\n"			"CMD ping %s\n\n"\
+		"NAME Time\n"			"CMD ctcp %s TIME\n\n"\
+		"NAME Finger\n"		"CMD ctcp %s FINGER\n\n"\
+		"NAME XDCC List\n"	"CMD ctcp %s XDCC LIST\n\n"\
+		"NAME CDCC List\n"	"CMD ctcp %s CDCC LIST\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Oper\n\n"\
-	"NAME Kill this user\n"	"CMD /quote KILL %s :die!\n\n"\
-	"NAME ENDSUB\n"			"CMD \n\n"\
-	"NAME SUB\n"				"CMD Mode\n\n"\
-	"NAME Give Voice\n"		"CMD /voice %s\n\n"\
-	"NAME Take Voice\n"		"CMD /devoice %s\n"\
-	"NAME SEP\n"				"CMD \n\n"\
-	"NAME Give Ops\n"			"CMD /op %s\n\n"\
-	"NAME Take Ops\n"			"CMD /deop %s\n\n"\
+		"NAME Kill this user\n""CMD quote KILL %s :die!\n\n"\
+		"NAME ENDSUB\n"		"CMD \n\n"\
+		"NAME SUB\n"			"CMD Mode\n\n"\
+		"NAME Give Voice\n"	"CMD voice %a\n\n"\
+		"NAME Take Voice\n"	"CMD devoice %a\n"\
+		"NAME SEP\n"			"CMD \n\n"\
+		"NAME Give Ops\n"		"CMD op %a\n\n"\
+		"NAME Take Ops\n"		"CMD deop %a\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Ignore\n\n"\
-	"NAME Ignore User\n"		"CMD /ignore %s!*@* ALL\n\n"\
-	"NAME UnIgnore User\n"	"CMD /unignore %s!*@*\n\n"\
+		"NAME Ignore User\n"	"CMD ignore %s!*@* ALL\n\n"\
+		"NAME UnIgnore User\n""CMD unignore %s!*@*\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Kick/Ban\n\n"\
-	"NAME Kick\n"				"CMD /kick %s\n\n"\
-	"NAME Ban\n"				"CMD /ban %s\n\n"\
-	"NAME SEP\n"				"CMD \n\n"\
-	"NAME Ban *!*@*.host\n"	"CMD /ban %s 0\n\n"\
-	"NAME Ban *!*@domain\n"	"CMD /ban %s 1\n\n"\
-	"NAME Ban *!*user@*.host\n""CMD /ban %s 2\n\n"\
-	"NAME Ban *!*user@domain\n""CMD /ban %s 3\n\n"\
-	"NAME SEP\n"				"CMD \n\n"\
-	"NAME KickBan *!*@*.host\n""CMD /kickban %s 0\n\n"\
-	"NAME KickBan *!*@domain\n""CMD /kickban %s 1\n\n"\
-	"NAME KickBan *!*user@*.host\n""CMD /kickban %s 2\n\n"\
-	"NAME KickBan *!*user@domain\n""CMD /kickban %s 3\n\n"\
+		"NAME Kick\n"			"CMD kick %s\n\n"\
+		"NAME Ban\n"			"CMD ban %s\n\n"\
+		"NAME SEP\n"			"CMD \n\n"\
+		"NAME Ban *!*@*.host\n""CMD ban %s 0\n\n"\
+		"NAME Ban *!*@domain\n""CMD ban %s 1\n\n"\
+		"NAME Ban *!*user@*.host\n""CMD ban %s 2\n\n"\
+		"NAME Ban *!*user@domain\n""CMD ban %s 3\n\n"\
+		"NAME SEP\n"			"CMD \n\n"\
+		"NAME KickBan *!*@*.host\n""CMD kickban %s 0\n\n"\
+		"NAME KickBan *!*@domain\n""CMD kickban %s 1\n\n"\
+		"NAME KickBan *!*user@*.host\n""CMD kickban %s 2\n\n"\
+		"NAME KickBan *!*user@domain\n""CMD kickban %s 3\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Info\n\n"\
-	"NAME Who\n"				"CMD /quote WHO %s\n\n"\
-	"NAME Whois\n"				"CMD /quote WHOIS %s\n\n"\
-	"NAME DNS Lookup\n"		"CMD /dns %s\n\n"\
-	"NAME Trace\n"				"CMD /quote TRACE %s\n\n"\
-	"NAME UserHost\n"			"CMD /quote USERHOST %s\n\n"\
+		"NAME Who\n"			"CMD quote WHO %s\n\n"\
+		"NAME Whois\n"			"CMD quote WHOIS %s\n\n"\
+		"NAME DNS Lookup\n"	"CMD dns %s\n\n"\
+		"NAME Trace\n"			"CMD quote TRACE %s\n\n"\
+		"NAME UserHost\n"		"CMD quote USERHOST %s\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD External\n\n"\
-	"NAME Traceroute\n"		"CMD !rxvt -e /bin/sh -c \"/usr/sbin/traceroute %h ; sleep 30\"\n\n"\
-	"NAME Ping\n"				"CMD !rxvt -e /bin/sh -c \"ping -c 4 %h ; sleep 30\"\n\n"\
-	"NAME Telnet\n"			"CMD !rxvt -e telnet %h\n\n"\
+		"NAME Traceroute\n"	"CMD !"XTERM" -e /bin/sh -c \"/usr/sbin/traceroute %h ; sleep 30\"\n\n"\
+		"NAME Ping\n"			"CMD !"XTERM" -e /bin/sh -c \"ping -c 4 %h ; sleep 30\"\n\n"\
+		"NAME Telnet\n"		"CMD !"XTERM" -e telnet %h\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
-	"NAME Open Query\n"		"CMD /query %s\n\n")
+	"NAME Open Query\n"		"CMD query %s\n\n"
 
-#define defaultconf_buttons _( \
-	"NAME Op\n"					"CMD /op %a\n\n"\
-	"NAME DeOp\n"				"CMD /deop %a\n\n"\
-	"NAME Ban\n"				"CMD /ban %s\n\n"\
-	"NAME Kick\n"				"CMD /kick %s\n\n"\
-	"NAME Sendfile\n"			"CMD /dcc send %s\n\n"\
-	"NAME Dialog\n"			"CMD /query %s\n\n")
+#define defaultconf_buttons \
+	"NAME Op\n"					"CMD op %a\n\n"\
+	"NAME DeOp\n"				"CMD deop %a\n\n"\
+	"NAME Ban\n"				"CMD ban %s\n\n"\
+	"NAME Kick\n"				"CMD kick %s\n\n"\
+	"NAME Sendfile\n"			"CMD dcc send %s\n\n"\
+	"NAME Dialog\n"			"CMD query %s\n\n"
 
-#define defaultconf_dlgbuttons _( \
-	"NAME Whois\n"				"CMD /whois %s\n\n"\
-	"NAME Send\n"				"CMD /dcc send %s\n\n"\
-	"NAME Chat\n"				"CMD /dcc chat %s\n\n"\
-	"NAME Ping\n"				"CMD /ping %s\n\n"\
-	"NAME Clear\n"				"CMD /clear\n\n")
+#define defaultconf_dlgbuttons \
+	"NAME Whois\n"				"CMD whois %s\n\n"\
+	"NAME Send\n"				"CMD dcc send %s\n\n"\
+	"NAME Chat\n"				"CMD dcc chat %s\n\n"\
+	"NAME Ping\n"				"CMD ping %s\n\n"\
+	"NAME Clear\n"				"CMD clear\n\n"
 
 #define defaultconf_replace \
 	"NAME teh\n"				"CMD the\n\n"\
@@ -919,123 +667,99 @@ save_away_message (struct server *serv, char *nick, char *msg)
 	"NAME u\n"					"CMD you\n\n"
 
 #define defaultconf_commands \
-   "NAME ACTION\n"		"CMD /me &2\n\n"\
-	"NAME AME\n"			"CMD /allchan /me &2\n\n"\
-	"NAME AMSG\n"			"CMD /allchan &2\n\n"\
-	"NAME BACK\n"			"CMD /away\n\n"\
-   "NAME BANLIST\n"		"CMD /quote MODE %c +b\n\n"\
-   "NAME CHAT\n"			"CMD /dcc chat %2\n\n"\
-   "NAME DIALOG\n"		"CMD /query %2\n\n"\
-   "NAME DMSG\n"			"CMD /msg =%2 &3\n\n"\
-   "NAME EXIT\n"			"CMD /quit\n\n"\
-   "NAME J\n"				"CMD /join &2\n\n"\
-   "NAME KILL\n"			"CMD /quote KILL %2 :&3\n\n"\
-   "NAME LEAVE\n"			"CMD /part &2\n\n"\
-   "NAME M\n"				"CMD /msg &2\n\n"\
-   "NAME ONOTICE\n"		"CMD /notice @%c &2\n\n"\
-   "NAME RAW\n"			"CMD /quote &2\n\n"\
-   "NAME SERVHELP\n"		"CMD /quote HELP\n\n"\
-	"NAME SPING\n"			"CMD /ping\n\n"\
-	"NAME SSLSERVER\n"	"CMD /server -ssl &2\n\n"\
-   "NAME SV\n"				"CMD /echo xchat %v %m\n\n"\
-   "NAME UMODE\n"			"CMD /mode %n &2\n\n"\
-   "NAME UPTIME\n"		"CMD /quote STATS u\n\n"\
-   "NAME VER\n"			"CMD /ctcp %2 VERSION\n\n"\
-   "NAME VERSION\n"		"CMD /ctcp %2 VERSION\n\n"\
-   "NAME WALLOPS\n"		"CMD /quote WALLOPS :&2\n\n"\
-   "NAME WII\n"			"CMD /quote WHOIS %2 %2\n\n"
+	"NAME ACTION\n"		"CMD me &2\n\n"\
+	"NAME AME\n"			"CMD allchan me &2\n\n"\
+	"NAME AMSG\n"			"CMD allchan say &2\n\n"\
+	"NAME BACK\n"			"CMD away\n\n"\
+	"NAME BANLIST\n"		"CMD quote MODE %c +b\n\n"\
+	"NAME CHAT\n"			"CMD dcc chat %2\n\n"\
+	"NAME DIALOG\n"		"CMD query %2\n\n"\
+	"NAME DMSG\n"			"CMD msg =%2 &3\n\n"\
+	"NAME EXIT\n"			"CMD quit\n\n"\
+	"NAME J\n"				"CMD join &2\n\n"\
+	"NAME KILL\n"			"CMD quote KILL %2 :&3\n\n"\
+	"NAME LEAVE\n"			"CMD part &2\n\n"\
+	"NAME M\n"				"CMD msg &2\n\n"\
+	"NAME ONOTICE\n"		"CMD notice @%c &2\n\n"\
+	"NAME RAW\n"			"CMD quote &2\n\n"\
+	"NAME SERVHELP\n"		"CMD quote HELP\n\n"\
+	"NAME SPING\n"			"CMD ping\n\n"\
+	"NAME SQUERY\n"		"CMD quote SQUERY %2 :&3\n\n"\
+	"NAME SSLSERVER\n"	"CMD server -ssl &2\n\n"\
+	"NAME SV\n"				"CMD echo xchat %v %m\n\n"\
+	"NAME UMODE\n"			"CMD mode %n &2\n\n"\
+	"NAME UPTIME\n"		"CMD quote STATS u\n\n"\
+	"NAME VER\n"			"CMD ctcp %2 VERSION\n\n"\
+	"NAME VERSION\n"		"CMD ctcp %2 VERSION\n\n"\
+	"NAME WALLOPS\n"		"CMD quote WALLOPS :&2\n\n"\
+	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n"
 
-#define defaultconf_usermenu _( \
-	"NAME SUB\n"					"CMD IRC Stuff\n\n"\
-   "NAME Disconnect\n"			"CMD /discon\n\n"\
-   "NAME Reconnect\n"			"CMD /reconnect\n\n"\
-	"NAME Part Channel\n"		"CMD /part\n\n"\
-   "NAME Cycle Channel\n"		"CMD /cycle\n\n"\
-   "NAME Server Map\n"			"CMD /quote MAP\n\n"\
-	"NAME Server Links\n"		"CMD /quote LINKS\n\n"\
-   "NAME Ping Server\n"			"CMD /ping\n\n"\
-   "NAME ENDSUB\n"				"CMD \n\n"\
-	"NAME SUB\n"					"CMD Connect\n\n"\
-   "NAME irc.xchat.org #Linux\n""CMD /servchan irc.xchat.org 6667 #linux\n\n"\
- 	"NAME Go to EFNet\n"			"CMD /newserver irc.efnet.net\n\n"\
-   "NAME ENDSUB\n"				"CMD \n\n"\
-	"NAME SUB\n"					"CMD Settings\n\n"\
-	"NAME TOGGLE Hide Version\n"		"CMD hide_version\n\n"\
-	"NAME TOGGLE Colored Nicks\n"		"CMD colorednicks\n\n"\
-	"NAME TOGGLE 1.4.x Nick Comp.\n"	"CMD old_nickcompletion\n\n"\
-	"NAME TOGGLE Strip mIRC color\n"	"CMD stripcolor\n\n"\
-	"NAME TOGGLE Filter Beeps\n"		"CMD filterbeep\n\n"\
-	"NAME TOGGLE Raw MODE Display\n"	"CMD raw_modes\n\n"\
-	"NAME TOGGLE Perl Warnings\n"		"CMD perlwarnings\n\n"\
-	"NAME TOGGLE Mail Checker\n"		"CMD mail_check\n\n"\
-   "NAME ENDSUB\n"				"CMD \n\n"\
-	"NAME SUB\n"					"CMD External\n\n"\
-   "NAME Run XMMS\n"				"CMD !xmms\n\n"\
-	"NAME Run RXVT\n"				"CMD !rxvt\n\n"\
-   "NAME ENDSUB\n"				"CMD \n\n")
+#define defaultconf_usermenu \
+	"NAME SUB\n"						"CMD IRC Stuff\n\n"\
+		"NAME Disconnect\n"			"CMD discon\n\n"\
+		"NAME Reconnect\n"			"CMD reconnect\n\n"\
+		"NAME Part Channel\n"		"CMD part\n\n"\
+		"NAME Cycle Channel\n"		"CMD cycle\n\n"\
+		"NAME Server Map\n"			"CMD quote MAP\n\n"\
+		"NAME Server Links\n"		"CMD quote LINKS\n\n"\
+		"NAME Ping Server\n"			"CMD ping\n\n"\
+	"NAME ENDSUB\n"					"CMD \n\n"\
+	"NAME SUB\n"						"CMD Connect\n\n"\
+		"NAME irc.xchat.org #Linux\n""CMD servchan irc.xchat.org 6667 #linux\n\n"\
+ 		"NAME Go to EFNet\n"			"CMD newserver irc.efnet.net\n\n"\
+	"NAME ENDSUB\n"					"CMD \n\n"\
+	"NAME SUB\n"						"CMD Settings\n\n"\
+		"NAME TOGGLE Hide Version\n""CMD irc_hide_version\n\n"\
+		"NAME TOGGLE Colored Nicks\n""CMD text_color_nicks\n\n"\
+		"NAME TOGGLE 1.4.x Nick Comp.\n""CMD completion_old\n\n"\
+		"NAME TOGGLE Strip mIRC color\n""CMD text_stripcolor\n\n"\
+		"NAME TOGGLE Filter Beeps\n""CMD input_filter_beep\n\n"\
+		"NAME TOGGLE Raw MODE Display\n""CMD irc_raw_modes\n\n"\
+		"NAME TOGGLE Perl Warnings\n""CMD perl_warnings\n\n"\
+	"NAME ENDSUB\n"					"CMD \n\n"\
+	"NAME SUB\n"						"CMD External\n\n"\
+		"NAME Run XMMS\n"				"CMD !xmms\n\n"\
+		"NAME Run RXVT\n"				"CMD !"XTERM"\n\n"\
+	"NAME ENDSUB\n"					"CMD \n\n"
 
 #ifdef WIN32
 #define defaultconf_urlhandlers \
-	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n"
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
 #else
-#define defaultconf_urlhandlers _( \
+#define defaultconf_urlhandlers \
 	"NAME SUB\n"								"CMD Netscape...\n\n"\
-   	"NAME Open in existing\n"			"CMD !netscape -remote 'openURL(%s)'\n\n"\
-   	"NAME Open in new window\n"		"CMD !netscape -remote 'openURL(%s,new-window)'\n\n"\
-   	"NAME Run new Netscape\n"			"CMD !netscape %s\n\n"\
+		"NAME Open in existing\n"			"CMD !netscape -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !netscape -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Run new Netscape\n"			"CMD !netscape %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Mozilla...\n\n"\
-   	"NAME Open in existing\n"			"CMD !mozilla -remote 'openURL(%s)'\n\n"\
-   	"NAME Open in new window\n"		"CMD !mozilla -remote 'openURL(%s,new-window)'\n\n"\
-		"NAME Open in new tab\n"			"CMD !mozilla -remote 'openURL(%s,new-tab)'\n\n"\
-   	"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
+		"NAME Open in existing\n"			"CMD !mozilla -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !mozilla -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
 		"NAME Open in existing\n"			"CMD !galeon -x '%s'\n\n"\
-   	"NAME Open in new window\n"		"CMD !galeon -w '%s'\n\n"\
-   	"NAME Open in new tab\n"			"CMD !galeon -n '%s'\n\n"\
+		"NAME Open in new window\n"		"CMD !galeon -w '%s'\n\n"\
+		"NAME Open in new tab\n"			"CMD !galeon -n '%s'\n\n"\
 		"NAME Run new Galeon\n"				"CMD !galeon '%s'\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Opera...\n\n"\
-   	"NAME Open in existing\n"			"CMD !opera -remote 'openURL(%s)'\n\n"\
-   	"NAME Open in new window\n"		"CMD !opera -remote 'openURL(%s,new-window)'\n\n"\
-   	"NAME Run new Opera\n"				"CMD !opera %s\n\n"\
+		"NAME Open in existing\n"			"CMD !opera -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !opera -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Run new Opera\n"				"CMD !opera %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Send URL to...\n\n"\
-   	"NAME Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
-   	"NAME Lynx\n"							"CMD !rxvt -e lynx %s\n\n"\
-   	"NAME Links\n"							"CMD !rxvt -e links %s\n\n"\
-   	"NAME w3m\n"							"CMD !rxvt -e w3m %s\n\n"\
-   	"NAME NcFTP\n" 						"CMD !rxvt -e ncftp %s\n\n"\
-   	"NAME gFTP\n"							"CMD !gftp %s\n\n"\
+		"NAME Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
+		"NAME Lynx\n"							"CMD !"XTERM" -e lynx %s\n\n"\
+		"NAME Links\n"							"CMD !"XTERM" -e links %s\n\n"\
+		"NAME w3m\n"							"CMD !"XTERM" -e w3m %s\n\n"\
+		"NAME NcFTP\n" 						"CMD !"XTERM" -e ncftp %s\n\n"\
+		"NAME gFTP\n"							"CMD !gftp %s\n\n"\
 		"NAME Konqueror\n"					"CMD !konqueror %s\n\n"\
-		"NAME Telnet\n"						"CMD !rxvt -e telnet %s\n\n"\
-		"NAME Ping\n"							"CMD !rxvt -e ping -c 4 %s\n\n"\
+		"NAME Telnet\n"						"CMD !"XTERM" -e telnet %s\n\n"\
+		"NAME Ping\n"							"CMD !"XTERM" -e ping -c 4 %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
-	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n")
-#endif
-
-
-#ifndef WIN32
-/* Close and open log files on SIGUSR1. Usefull for log rotating */
-
-static void 
-sighup_handler (int signal)
-{
-	GSList *list = sess_list;
-	session *sess;
-
-	if (prefs.logging)
-	{
-		while (list)
-		{
-			sess = list->data;
-			setup_logging (sess);
-			list = list->next;
-		}
-	}
-}
-
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
 #endif
 
 static void
@@ -1053,12 +777,8 @@ xchat_init (void)
 	act.sa_flags = 0;
 	sigemptyset (&act.sa_mask);
 	sigaction (SIGPIPE, &act, NULL);
-
-	/* Deal with SIGUSR1's */
-	signal (SIGUSR1, sighup_handler);
 #endif
 
-	signal_setup ();
 	load_text_events ();
 	notify_load ();
 	ignore_load ();
@@ -1085,8 +805,10 @@ xchat_init (void)
 	{
 		/* and serverlist isn't open .... */
 		if (prefs.skipserverlist)
+		{
 			/* we'll have to open one. */
 			new_ircwindow (NULL, NULL, SESS_SERVER);
+		}
 	}
 }
 
@@ -1095,12 +817,7 @@ xchat_exit (void)
 {
 	xchat_is_quitting = TRUE;
 	in_xchat_exit = TRUE;
-#ifdef USE_PERL
-	perl_end ();
-#endif
-#ifdef USE_PYTHON
-	pys_kill ();
-#endif
+	plugin_kill_all ();
 	fe_cleanup ();
 	if (prefs.autosave)
 	{
@@ -1147,6 +864,14 @@ main (int argc, char *argv[])
 	SOCKSinit (argv[0]);
 #endif
 
+#ifdef USE_OPENSSL
+	if (!(ctx = _SSL_context_init (ssl_cb_info, FALSE)))
+	{
+		fprintf (stderr, "_SSL_context_init failed\n");
+		return 1;
+	}
+#endif
+
 	if (!fe_args (argc, argv))
 		return 0;
 
@@ -1158,14 +883,13 @@ main (int argc, char *argv[])
 
 	fe_main ();
 
+#ifdef USE_OPENSSL
+	_SSL_context_free (ctx);
+#endif
+
 #ifdef USE_DEBUG
 	xchat_mem_list ();
 #endif
 
-#ifdef USE_OPENSSL
-	if (ctx)
-		_SSL_context_free (ctx);
-#endif
-
 	return 0;
 }
