@@ -1,10 +1,9 @@
 #include "../../config.h"
 
-#ifdef USE_MYGLIB
-#include "../fe-text/myglib.h"
-#else
-#include <glib.h>
-#endif
+#include <glib/gslist.h>
+#include <glib/gutils.h>
+#include <glib/giochannel.h>
+#include <glib/gstrfuncs.h>
 #include <time.h>			/* need time_t */
 
 #include "history.h"
@@ -13,6 +12,10 @@
 #define snprintf g_snprintf
 #endif
 
+#ifndef HAVE_VSNPRINTF
+#define vsnprintf g_vsnprintf
+#endif
+
 #ifdef USE_DEBUG
 #define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
 #define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
@@ -42,8 +45,9 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define strncasecmp strnicmp
 #define PATH_MAX MAXPATHLEN
 #define FILEPATH_LEN_MAX MAXPATHLEN
-#else
-#ifdef WIN32
+#endif
+
+#ifdef WIN32						/* for win32 */
 #define OFLAGS O_BINARY
 #define sleep(t) _sleep(t*1000)
 #include <direct.h>
@@ -55,10 +59,9 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define	S_ISDIR(m)	((m) & _S_IFDIR)
 #endif
 #define NETWORK_PRIVATE
-#else
+#else									/* for unix */
 #define OFLAGS 0
 #endif
-#endif
 
 #define FONTNAMELEN	127
 #define PATHLEN		255
@@ -66,8 +69,8 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define CHANLEN		202
 #define PDIWORDS		32
 
-#define safe_strcpy(dest,src,len)	strncpy(dest,src,len); \
-												dest[len-1] = 0;
+#define safe_strcpy(dest,src,len)	{strncpy(dest,src,len); \
+												dest[len-1] = 0;}
 
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
@@ -96,9 +99,9 @@ struct nbexec
 
 struct xchatprefs
 {
-	char nick1[64];
-	char nick2[64];
-	char nick3[64];
+	char nick1[NICKLEN];
+	char nick2[NICKLEN];
+	char nick3[NICKLEN];
 	char realname[127];
 	char username[127];
 	char nick_suffix[4];			/* Only ever holds a one-character string. */
@@ -107,7 +110,9 @@ struct xchatprefs
 	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
 	char dialog_font_normal[FONTNAMELEN + 1];
+#ifdef USE_ZVT
 	char font_shell[FONTNAMELEN + 1];
+#endif
 	char doubleclickuser[256];
 	char sounddir[PATHLEN + 1];
 	char soundcmd[PATHLEN + 1];
@@ -118,7 +123,9 @@ struct xchatprefs
 	char dnsprogram[72];
 	char hostname[127];
 	char cmdchar[4];
+#ifdef USE_TRANS
 	char trans_file[256];
+#endif
 	char logmask[256];
 	char stamp_format[64];
 	char timestamp_log_format[64];
@@ -126,9 +133,6 @@ struct xchatprefs
 	char proxy_host[64];
 	int proxy_port;
 	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
-	unsigned int proxy_auth;
-	char proxy_user[32];
-	char proxy_pass[32];
 
 	int first_dcc_send_port;
 	int last_dcc_send_port;
@@ -143,8 +147,6 @@ struct xchatprefs
 
 	int tabs_position;
 	int max_auto_indent;
-	int indent_pixels;
-	int dialog_indent_pixels;
 	int dcc_blocksize;
 	int max_lines;
 	int notify_timeout;
@@ -166,7 +168,6 @@ struct xchatprefs
 	unsigned int mainwindow_save;
 	unsigned int perc_color;
 	unsigned int perc_ascii;
-	unsigned int hilightnick;
 	unsigned int use_trans;
 	unsigned int nopanel;
 	unsigned int autosave;
@@ -182,7 +183,6 @@ struct xchatprefs
 	unsigned int colorednicks;
 	unsigned int chanmodebuttons;
 	unsigned int userlistbuttons;
-	unsigned int userlist_icons;
 	unsigned int showhostname_in_userlist;
 	unsigned int nickcompletion;
 	unsigned int old_nickcompletion;
@@ -243,7 +243,6 @@ struct xchatprefs
 	unsigned int auto_indent;
 	unsigned int wordwrap;
 	unsigned int dialog_wordwrap;
-	unsigned int mail_check;
 	unsigned int throttle;
 	unsigned int fudgeservernotice;
 	unsigned int topicbar;
@@ -255,13 +254,10 @@ struct xchatprefs
 	unsigned int hebrew;
 	unsigned int limitedtabhighlight;
 	unsigned int pingtimeout;
-	unsigned int show_invite_in_front_session;
-	unsigned int show_notify_in_front_session;
 	unsigned int whois_on_notifyonline;
-	unsigned int inputgad_superfocus;
-	unsigned int nickgad;
 	unsigned int channelbox;
 	unsigned int persist_chans;
+	unsigned int wait_on_exit;
 
 	unsigned int ctcp_number_limit;	/*flood */
 	unsigned int ctcp_time_limit;	/*seconds of floods */
@@ -274,11 +270,10 @@ struct xchatprefs
 #define SESS_SERVER	1
 #define SESS_CHANNEL	2
 #define SESS_DIALOG	3
-#define SESS_SHELL	4
-#define SESS_NOTICES	5
-#define SESS_SNOTICES	6
+#define SESS_NOTICES	4
+#define SESS_SNOTICES	5
 
-struct session
+typedef struct session
 {
 	struct server *server;
 	GSList *userlist;
@@ -305,14 +300,13 @@ struct session
 	int mode_timeout_tag;
 
 	struct session *lastlog_sess;
-	struct setup *setup;
 	struct nbexec *running_exec;
 
 	struct session_gui *gui;		/* initialized by fe_new_window */
+	struct restore_gui *res;
 
 	int userlisthidden;
 	int type;
-	int is_tab:1;			/* is this a tab or toplevel window? */
 	int new_data:1;		/* new data avail? (red tab) */
 	int nick_said:1;		/* your nick mentioned? (blue tab) */
 	int ignore_date:1;
@@ -321,28 +315,50 @@ struct session
 	int end_of_names:1;
 	int doing_who:1;		/* /who sent on this channel */
 	int highlight_tab:1;	/* Highlight the channel tab (red-style) */
-};
+	/* these are in the bottom-right menu */
+	int hide_join_part:1;	/* hide join & part messages? */
+	int beep:1;				/* beep enabled? */
+	int color_paste:1;
+} session;
 
-typedef struct session session;
-
-struct server
+typedef struct server
 {
+	/* irc protocol functions (in proto-irc.c) */
+	void (*p_inline)(struct server *, char *buf);
+	void (*p_invite)(struct server *, char *channel, char *nick);
+	void (*p_cycle)(struct server *, char *channel, char *key);
+	void (*p_ctcp)(struct server *, char *to, char *msg);
+	void (*p_nctcp)(struct server *, char *to, char *msg);
+	void (*p_quit)(struct server *, char *reason);
+	void (*p_kick)(struct server *, char *channel, char *nick, char *reason);
+	void (*p_part)(struct server *, char *channel, char *reason);
+	void (*p_join)(struct server *, char *channel, char *key);
+	void (*p_login)(struct server *, char *user, char *realname);
+	void (*p_join_info)(struct server *, char *channel);
+	void (*p_user_list)(struct server *, char *channel);
+	void (*p_get_ip)(struct server *, char *nick);
+	void (*p_set_back)(struct server *);
+	void (*p_set_away)(struct server *, char *reason);
+	void (*p_message)(struct server *, char *channel, char *text);
+	void (*p_action)(struct server *, char *channel, char *act);
+	void (*p_notice)(struct server *, char *channel, char *text);
+	void (*p_topic)(struct server *, char *channel, char *topic);
+	void (*p_list_channels)(struct server *, char *arg);
+	void (*p_change_nick)(struct server *, char *new_nick);
+	void (*p_names)(struct server *, char *channel);
+/*	void (*p_set_away)(struct server *);*/
+
 	int port;
-	int sok;
-	int sok4;
-	int sok6;
+	int sok;					/* is equal to sok4 or sok6 (the one we are using) */
+	int sok4;				/* tcp4 socket */
+	int sok6;				/* tcp6 socket */
 #ifdef USE_OPENSSL
 	SSL *ssl;
 	int ssl_do_connect_tag;
-	int use_ssl:1;					  /* is server SSL capable? */
-	int accept_invalid_cert:1;	  /* ignore result of server's cert. verify */
 #endif
 	int childread;
 	int childwrite;
 	int childpid;
-#ifdef WIN32
-	int childhandle;
-#endif
 	int iotag;
 	int bartag;
 	int recondelay_tag;				/* reconnect delay timeout */
@@ -368,7 +384,8 @@ struct server
 	int next_send;						/* cptr->since in ircu */
 	int sendq_len;						/* queue size */
 
-	struct session *front_session;
+	struct session *front_session;	/* front-most window/tab */
+	struct session *server_session;	/* server window/tab */
 
 	struct server_gui *gui;		  /* initialized by fe_new_server */
 
@@ -382,7 +399,6 @@ struct server
 	time_t lag_sent;
 	time_t ping_recv;					/* when we last got a ping reply */
 	time_t away_time;					/* when we were marked away */
-	char *network;
 
 	int motd_skipped:1;
 	int connected:1;
@@ -393,15 +409,18 @@ struct server
 	int doing_who:1;				  /* /dns has been done */
 	int end_of_motd:1;			  /* end of motd reached (logged in) */
 	int sent_quit:1;				  /* sent a QUIT already? */
-	int is_newtype:1;		/* undernet and dalnet need /list >0,<10000 */
+	int use_listargs:1;			/* undernet and dalnet need /list >0,<10000 */
 	int is_away:1;
 	int reconnect_away:1;		/* whether to reconnect in is_away state */
 	int dont_use_proxy:1;		/* to proxy or not to proxy */
 	int supports_watch:1;		/* supports the WATCH command */
 	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
-};
-
-typedef struct server server;
+	int use_utf8:1;				/* send and receive utf8? */
+#ifdef USE_OPENSSL
+	int use_ssl:1;					  /* is server SSL capable? */
+	int accept_invalid_cert:1;	  /* ignore result of server's cert. verify */
+#endif
+} server;
 
 typedef int (*cmd_callback) (struct session * sess, char *tbuf, char *word[],
 									  char *word_eol[]);
