@@ -2,8 +2,9 @@ extern struct xchatprefs prefs;
 
 extern int auto_connect;
 extern int xchat_is_quitting;
+extern char *connect_url;
 
-extern session *menu_sess;
+extern session *current_sess;
 extern session *current_tab;
 
 extern GSList *popup_list;
@@ -19,9 +20,7 @@ extern GSList *ignore_list;
 extern GSList *usermenu_list;
 extern GSList *urlhandler_list;
 
-int tcp_send_len (server *serv, char *buf, int len);
-int tcp_send (server *serv, char *buf);
-session * find_channel (char *chan, server *serv);
+session * find_session_from_channel (char *chan, server *serv);
 session * find_dialog (server *serv, char *nick);
 session * new_ircwindow (server *serv, char *name, int type);
 void set_server_defaults (server *serv);
