@@ -28,13 +28,25 @@
 #include <time.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkalignment.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkstock.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtkhbbox.h>
+
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/util.h"
-#include <gdk/gdkkeysyms.h>
 #include "gtkutil.h"
-#include "wins.h"
+#include "maingui.h"
 
 
 /**
@@ -44,14 +56,11 @@
 #ifndef WIN32
 
 static int
-reg_match (server *serv, const char *str)
+reg_match (regex_t * regexptr, const char *str)
 {
 	int m;
 
-	if (!serv->gui->have_regex)
-		return 0;
-
-	m = regexec (&serv->gui->chanlist_match_regex, str, 1, NULL, REG_NOTBOL);
+	m = regexec (regexptr, str, 1, NULL, REG_NOTBOL);
 
 	/* regex returns 0 if it's a match: */
 	return m == 0 ? 1 : 0;
@@ -60,9 +69,9 @@ reg_match (server *serv, const char *str)
 #else
 
 static int
-reg_match (server *serv, const char *str)
+reg_match (char ** regexptr, const char *str)
 {
-	return match (serv->gui->chanlist_match_regex, str);
+	return match (*regexptr, str);
 }
 
 #endif
@@ -97,7 +106,7 @@ chanlist_compare_text_ignore_case (GtkCList * clist,
 	GtkCListRow *row1 = (GtkCListRow *) ptr1;
 	GtkCListRow *row2 = (GtkCListRow *) ptr2;
 
-	return strcasecmp (GTK_CELL_TEXT (row1->cell[clist->sort_column])->text,
+	return rfc_casecmp (GTK_CELL_TEXT (row1->cell[clist->sort_column])->text,
 							 GTK_CELL_TEXT (row2->cell[clist->sort_column])->text);
 }
 
@@ -191,8 +200,6 @@ static void
 chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 {
 	int num_users = atoi (next_row[1]);
-	gfloat val, end;
-	GtkAdjustment *adj;
 
 	/* First, update the 'found' counter values */
 	serv->gui->chanlist_users_found_count += num_users;
@@ -222,8 +229,8 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 			 (!serv->gui->chanlist_match_wants_channel
 			  && !serv->gui->chanlist_match_wants_topic))
 		{
-			if (!reg_match (serv, next_row[0])
-				 && !reg_match (serv, next_row[2]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0])
+				 && !reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -232,7 +239,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_channel)
 		{
-			if (!reg_match (serv, next_row[0]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -241,7 +248,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_topic)
 		{
-			if (!reg_match (serv, next_row[2]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -249,8 +256,6 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 		}
 	}
 
-	adj = gtk_clist_get_vadjustment (GTK_CLIST (serv->gui->chanlist_list));
-	val = adj->value;
 	/*
 	 * If all the above above tests passed or if no text was in the 
 	 * chanlist_wild_text, add this entry to the GUI
@@ -261,12 +266,6 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 	serv->gui->chanlist_users_shown_count += num_users;
 	serv->gui->chanlist_channels_shown_count++;
 
-	/* restore original scrollbar position */
-	end = adj->upper - adj->lower - adj->page_size;
-	if (val > end)
-		val = end;
-	gtk_adjustment_set_value (adj, val);
-
 	chanlist_update_caption (serv);
 }
 
@@ -285,7 +284,7 @@ chanlist_do_refresh (struct server *serv)
 		gtk_clist_clear (GTK_CLIST (serv->gui->chanlist_list));
 		gtk_widget_set_sensitive (serv->gui->chanlist_refresh, FALSE);
 
-		handle_command ("/LIST", serv->front_session, FALSE, FALSE);
+		handle_command (serv->server_session, "list", FALSE);
 	} else
 		gtkutil_simpledialog ("Not connected.");
 }
@@ -386,23 +385,22 @@ chanlist_apply_pressed (GtkButton * button, struct server *serv)
 static void
 chanlist_wild (GtkWidget * wid, struct server *serv)
 {
-	const char *pattern = gtk_entry_get_text (GTK_ENTRY (wid));
-
 	/* Store the pattern text in the wild_text var so next time the window is 
 	 * opened it is remembered. 
 	 */
-	safe_strcpy (serv->gui->chanlist_wild_text, pattern,
+	safe_strcpy (serv->gui->chanlist_wild_text,
+					 gtk_entry_get_text (GTK_ENTRY (wid)),
 					 sizeof (serv->gui->chanlist_wild_text));
 
 #ifdef WIN32
-	serv->gui->chanlist_match_regex = strdup (pattern);
+	serv->gui->chanlist_match_regex =
+		strdup (gtk_entry_get_text (GTK_ENTRY (wid)));
 #else
 	/* recompile the regular expression. */
-	if (serv->gui->have_regex)
-		regfree (&serv->gui->chanlist_match_regex);
-	regcomp (&serv->gui->chanlist_match_regex, pattern,
+	regfree (&serv->gui->chanlist_match_regex);
+	regcomp (&serv->gui->chanlist_match_regex,
+				gtk_entry_get_text (GTK_ENTRY (wid)),
 				REG_ICASE | REG_EXTENDED | REG_NOSUB);
-	serv->gui->have_regex = 1;
 #endif
 }
 
@@ -463,7 +461,7 @@ chanlist_join (GtkWidget * wid, struct server *serv)
 {
 	int row;
 	char *chan;
-	char tbuf[256];
+	char tbuf[CHANLEN + 6];
 
 	row = gtkutil_clist_selection (serv->gui->chanlist_list);
 	if (row != -1)
@@ -472,8 +470,8 @@ chanlist_join (GtkWidget * wid, struct server *serv)
 								  &chan);
 		if (serv->connected && (strcmp (chan, "*") != 0))
 		{
-			snprintf (tbuf, sizeof tbuf, "JOIN %s\r\n", chan);
-			tcp_send (serv, tbuf);
+			snprintf (tbuf, sizeof (tbuf), "join %s", chan);
+			handle_command (serv->server_session, tbuf, FALSE);
 		} else
 			gdk_beep ();
 	}
@@ -561,18 +559,14 @@ chanlist_destroy_widget (GtkObject * object, struct server *serv)
 {
 	chanlist_data_free (serv);
 #ifndef WIN32
-	if (serv->gui->have_regex)
-	{
-		regfree (&serv->gui->chanlist_match_regex);
-		serv->gui->have_regex = 0;
-	}
+	regfree (&serv->gui->chanlist_match_regex);
 #else
 	free (serv->gui->chanlist_match_regex);
 #endif
 }
 
 static void
-chanlist_closegui (server *serv)
+chanlist_closegui (GtkWidget *wid, server *serv)
 {
 	if (is_server (serv))
 		serv->gui->chanlist_window = 0;
@@ -581,18 +575,14 @@ chanlist_closegui (server *serv)
 void
 chanlist_opengui (struct server *serv)
 {
-	gchar *titles[3];
+	gchar *titles[] = { _("Channel"), _("Users"), _("Topic") };
 	GtkWidget *frame, *vbox, *hbox, *sortvbox, *sorthbox, *numtable,
 		*table, *wid, *real_wid;
 	char tbuf[256];
 
-	titles[0] = _("Channel");
-	titles[1] = _("Users");
-	titles[2] = _("Topic");
-
 	if (serv->gui->chanlist_window)
 	{
-		gdk_window_show (serv->gui->chanlist_window->window);
+		gtk_window_present (GTK_WINDOW (serv->gui->chanlist_window));
 		return;
 	}
 
@@ -609,9 +599,8 @@ chanlist_opengui (struct server *serv)
 
 
 	serv->gui->chanlist_window =
-		maingui_window ("chanlist", tbuf, FALSE, TRUE, chanlist_closegui, serv,
-								450, 300, &serv->gui->chanlist_window);
-	vbox = wins_get_vbox (serv->gui->chanlist_window);
+		mg_create_generic_tab ("chanlist", tbuf, FALSE, TRUE, chanlist_closegui, serv,
+								450, 300, &vbox);
 
 	frame = gtk_frame_new (_("List display options:"));
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
@@ -661,7 +650,7 @@ chanlist_opengui (struct server *serv)
 	if (serv->gui->chanlist_maxusers > 0)
 		sprintf (tbuf, "%d", serv->gui->chanlist_maxusers);
 	else
-		strcpy (tbuf, "");
+		tbuf[0] = 0;
 	gtk_entry_set_text (GTK_ENTRY (wid), tbuf);
 	gtk_signal_connect (GTK_OBJECT (wid), "changed",
 							  GTK_SIGNAL_FUNC (chanlist_maxusers), serv);
@@ -725,7 +714,7 @@ chanlist_opengui (struct server *serv)
 	gtk_widget_show (wid);
 
 	wid = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
-	real_wid = gtk_button_new_with_label (_(" Apply "));
+	real_wid = gtk_button_new_with_label (_("Apply"));
 	gtk_container_add (GTK_CONTAINER (wid), real_wid);
 	gtk_table_attach_defaults (GTK_TABLE (table), wid, 0, 3, 1, 2);
 	gtk_signal_connect (GTK_OBJECT (real_wid), "pressed",
@@ -761,18 +750,18 @@ chanlist_opengui (struct server *serv)
 
 	serv->gui->chanlist_label = wid;
 
-	hbox = gtk_hbox_new (0, 1);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
+	hbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_SPREAD);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
 	serv->gui->chanlist_refresh =
-		gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_REFRESH,
-							 _("Refresh the list"), chanlist_refresh, serv, hbox);
-	gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_SAVE,
-						 _("Save the list"), chanlist_save, serv, hbox);
-	gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_JUMP_TO,
-						 _("Join Channel"), chanlist_join, serv, hbox);
+		gtkutil_button (hbox, GTK_STOCK_REFRESH, 0, chanlist_refresh, serv,
+							_("Refresh the list"), 0, 0);
+	gtkutil_button (hbox, GTK_STOCK_SAVE, 0, chanlist_save, serv,
+						_("Save the list"), 0, 0);
+	gtkutil_button (hbox, GTK_STOCK_JUMP_TO, 0, chanlist_join, serv,
+						 _("Join Channel"), 0, 0);
 
 	/* connect a destroy event handler to this window so that the dynamic
 	   memory can be freed */
