@@ -27,15 +27,20 @@
 #define WANTARPA
 #include "../common/inet.h"
 #include "fe-gtk.h"
+
+#include <gtk/gtkclist.h>
+#include <gtk/gtkhbbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkstock.h>
+
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
-#include "../common/plugin.h"
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/network.h"
 #include "gtkutil.h"
-#include "wins.h"
 #include "palette.h"
+#include "maingui.h"
 
 struct dccwindow
 {
@@ -43,6 +48,13 @@ struct dccwindow
 	GtkWidget *list;
 };
 
+struct dcc_send
+{
+	struct session *sess;
+	char nick[64];
+	GtkWidget *freq;
+};
+
 static struct dccwindow dccrwin;	/* recv */
 static struct dccwindow dccswin;	/* send */
 static struct dccwindow dcccwin;	/* chat */
@@ -74,7 +86,7 @@ fe_dcc_send_filereq (struct session *sess, char *nick)
 void
 fe_dcc_update_recv (struct DCC *dcc)
 {
-	char pos[14], cps[14], perc[14], eta[14];
+	char pos[14], kbs[14], perc[14], eta[14];
 	gint row;
 	int to_go;
 	float per;
@@ -88,8 +100,8 @@ fe_dcc_update_recv (struct DCC *dcc)
 	/* percentage done */
 	per = (float) ((dcc->pos * 100.00) / dcc->size);
 
-	snprintf (pos, sizeof (pos), "%u", dcc->pos);
-	snprintf (cps, sizeof (cps), "%d", dcc->cps);
+	snprintf (pos, sizeof (pos), "%d", dcc->pos);
+	snprintf (kbs, sizeof (kbs), "%.1f", ((float)dcc->cps) / 1024);
 	snprintf (perc, sizeof (perc), "%.0f%%", per);
 
 	gtk_clist_freeze (GTK_CLIST (dccrwin.list));
@@ -97,7 +109,7 @@ fe_dcc_update_recv (struct DCC *dcc)
 							  _(dccstat[(int) dcc->dccstat].name));
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 3, pos);
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 4, perc);
-	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 5, cps);
+	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 5, kbs);
 	if (dcc->cps != 0)
 	{
 		to_go = (dcc->size - dcc->pos) / dcc->cps;
@@ -121,7 +133,7 @@ fe_dcc_update_recv (struct DCC *dcc)
 void
 fe_dcc_update_send (struct DCC *dcc)
 {
-	char pos[14], cps[14], ack[14], perc[14], eta[14];
+	char pos[14], kbs[14], ack[14], perc[14], eta[14];
 	gint row;
 	int to_go;
 	float per;
@@ -136,8 +148,8 @@ fe_dcc_update_send (struct DCC *dcc)
 	per = (float) ((dcc->ack * 100.00) / dcc->size);
 
 	snprintf (pos, sizeof (pos), "%d", dcc->pos);
-	snprintf (cps, sizeof (cps), "%u", dcc->cps);
-	snprintf (ack, sizeof (ack), "%u", dcc->ack);
+	snprintf (kbs, sizeof (kbs), "%.1f", ((float)dcc->cps) / 1024);
+	snprintf (ack, sizeof (ack), "%d", dcc->ack);
 	snprintf (perc, sizeof (perc), "%.0f%%", per);
 	if (dcc->cps != 0)
 	{
@@ -156,33 +168,11 @@ fe_dcc_update_send (struct DCC *dcc)
 	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 3, pos);
 	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 4, ack);
 	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 5, perc);
-	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 6, cps);
+	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 6, kbs);
 	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 7, eta);
 	gtk_clist_thaw (GTK_CLIST (dccswin.list));
 }
 
-static void
-dcc_abort (struct DCC *dcc)
-{
-	if (dcc)
-	{
-		switch (dcc->dccstat)
-		{
-		case STAT_QUEUED:
-		case STAT_CONNECTING:
-		case STAT_ACTIVE:
-			dcc_close (dcc, STAT_ABORTED, FALSE);
-
-			EMIT_SIGNAL (XP_TE_DCCABORT, dcc->serv->front_session,
-							 dcctypes[(int) dcc->type], file_part (dcc->file),
-							 dcc->nick, NULL, 0);
-			break;
-		default:
-			dcc_close (dcc, 0, TRUE);
-		}
-	}
-}
-
 static void
 close_dcc_recv_window (void)
 {
@@ -197,7 +187,7 @@ fe_dcc_update_recv_win (void)
 	gchar *nnew[9];
 	char size[16];
 	char pos[16];
-	char cps[16];
+	char kbs[16];
 	char perc[14];
 	char eta[16];
 	gint row;
@@ -214,7 +204,7 @@ fe_dcc_update_recv_win (void)
 	nnew[2] = size;
 	nnew[3] = pos;
 	nnew[4] = perc;
-	nnew[5] = cps;
+	nnew[5] = kbs;
 	nnew[6] = eta;
 	while (list)
 	{
@@ -230,12 +220,12 @@ fe_dcc_update_recv_win (void)
 			else
 				nnew[8] = "";
 #endif
-			sprintf (size, "%u", dcc->size);
+			sprintf (size, "%d", dcc->size);
 			if (dcc->dccstat == STAT_QUEUED)
-				sprintf (pos, "%u", dcc->resumable);
+				sprintf (pos, "%d", dcc->resumable);
 			else
-				sprintf (pos, "%u", dcc->pos);
-			snprintf (cps, sizeof (cps), "%d", dcc->cps);
+				sprintf (pos, "%d", dcc->pos);
+			snprintf (kbs, sizeof (kbs), "%.1f", ((float)dcc->cps) / 1024);
 			/* percentage recv'ed */
 			per = (float) ((dcc->pos * 100.00) / dcc->size);
 			snprintf (perc, sizeof (perc), "%.0f%%", per);
@@ -266,7 +256,7 @@ dcc_info (struct DCC *dcc)
 	char tbuf[256];
 	snprintf (tbuf, 255, _("      File: %s\n"
 				 "   To/From: %s\n"
-				 "      Size: %u\n"
+				 "      Size: %d\n"
 				 "      Port: %d\n"
 				 " IP Number: %s\n"
 				 "Start Time: %s"),
@@ -303,7 +293,7 @@ abort_clicked (GtkWidget * wid, gpointer none)
 	if (row != -1)
 	{
 		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), row);
-		dcc_abort (dcc);
+		dcc_abort (dcc->serv->front_session, dcc);
 	}
 }
 
@@ -392,34 +382,24 @@ fe_dcc_open_recv_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
 #ifdef USE_GNOME
-	gchar *titles[9];
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), "%", "KB/s", _("ETA"), _("From"),
+			_("MIME Type") };
 #else
-	gchar *titles[8];
-#endif
-
-	titles[0] = _("Status");
-	titles[1] = _("File");
-	titles[2] = _("Size");
-	titles[3] = _("Position");
-	titles[4] = "%";
-	titles[5] = _("CPS");
-	titles[6] = _("ETA");
-	titles[7] = _("From");
-#ifdef USE_GNOME
-	titles[8] = _("MIME Type");
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), "%", "KB/s", _("ETA"), _("From") };
 #endif
 
 	if (dccrwin.window)
 	{
 		fe_dcc_update_recv_win ();
 		if (!passive)
-			wins_bring_tofront (dccrwin.window);
+			mg_bring_tofront (dccrwin.window);
 		return;
 	}
-	dccrwin.window = maingui_window ("dccrecv", _("X-Chat: File Receive List"),
-								FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0,
-								&dccrwin.window);
-	vbox = wins_get_vbox (dccrwin.window);
+	dccrwin.window =
+			  mg_create_generic_tab ("dccrecv", _("X-Chat: File Receive List"),
+							FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0, &vbox);
 
 #ifdef USE_GNOME
 	dccrwin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
@@ -441,19 +421,20 @@ fe_dcc_open_recv_win (int passive)
 	gtk_clist_set_column_justification (GTK_CLIST (dccrwin.list), 4,
 													GTK_JUSTIFY_CENTER);
 
-	bbox = gtk_hbox_new (FALSE, 2);
+	bbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dccrwin.window, 0, _("Accept"), accept_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_clicked, 0, _("Abort"), 0, 0);
 
-	gtkutil_button (dccrwin.window, 0, _("Resume"), resume_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_clicked, 0, _("Accept"), 0, 0);
 
-	gtkutil_button (dccrwin.window, 0, _("Abort"), abort_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_REFRESH, 0, resume_clicked, 0, _("Resume"), 0, 0);
 
-	gtkutil_button (dccrwin.window, 0, _("Info"), info_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_clicked, 0, _("Info"), 0, 0);
 #ifdef USE_GNOME
-	gtkutil_button (dccrwin.window, 0, _("Open"), open_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_OPEN, 0, open_clicked, 0, _("Open"), 0, 0);
 #endif
 	gtk_widget_show (dccrwin.window);
 	fe_dcc_update_recv_win ();
@@ -473,7 +454,7 @@ fe_dcc_update_send_win (void)
 	gchar *nnew[9];
 	char size[14];
 	char pos[14];
-	char cps[14];
+	char kbs[14];
 	char ack[14];
 	char perc[14];
 	char eta[14];
@@ -492,7 +473,7 @@ fe_dcc_update_send_win (void)
 	nnew[3] = pos;
 	nnew[4] = ack;
 	nnew[5] = perc;
-	nnew[6] = cps;
+	nnew[6] = kbs;
 	while (list)
 	{
 		nnew[7] = eta;
@@ -504,11 +485,11 @@ fe_dcc_update_send_win (void)
 			nnew[8] = dcc->nick;
 			/* percentage ack'ed */
 			per = (float) ((dcc->ack * 100.00) / dcc->size);
-			snprintf (size, sizeof (size), "%u", dcc->size);
-			snprintf (pos, sizeof (pos), "%u", dcc->pos);
-			snprintf (cps, sizeof (cps), "%d", dcc->cps);
+			snprintf (size, sizeof (size), "%d", dcc->size);
+			snprintf (pos, sizeof (pos), "%d", dcc->pos);
+			snprintf (kbs, sizeof (kbs), "%.1f", ((float)dcc->cps) / 1024);
 			snprintf (perc, sizeof (perc), "%.0f%%", per);
-			snprintf (ack, sizeof (ack), "%u", dcc->ack);
+			snprintf (ack, sizeof (ack), "%d", dcc->ack);
 			if (dcc->cps != 0)
 			{
 				to_go = (dcc->size - dcc->ack) / dcc->cps;
@@ -546,7 +527,7 @@ send_row_selected (GtkWidget * clist, gint row, gint column,
 			case STAT_FAILED:
 			case STAT_ABORTED:
 			case STAT_DONE:
-				dcc_abort (dcc);
+				dcc_abort (dcc->serv->front_session, dcc);
 			}
 		}
 	}
@@ -577,7 +558,7 @@ abort_send_clicked (GtkWidget * wid, gpointer none)
 	if (row != -1)
 	{
 		dcc = gtk_clist_get_row_data (GTK_CLIST (dccswin.list), row);
-		dcc_abort (dcc);
+		dcc_abort (dcc->serv->front_session, dcc);
 	}
 }
 
@@ -585,30 +566,20 @@ void
 fe_dcc_open_send_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	gchar *titles[9];
-
-	titles[0] = _("Status");
-	titles[1] = _("File");
-	titles[2] = _("Size");
-	titles[3] = _("Position");
-	titles[4] = _("Ack");
-	titles[5] = "%";
-	titles[6] = _("CPS");
-	titles[7] = _("ETA");
-	titles[8] = _("To");
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), _("Ack"), "%", "KB/s", _("ETA"), _("To") };
 
 	if (dccswin.window)
 	{
 		fe_dcc_update_send_win ();
 		if (!passive)
-			wins_bring_tofront (dccswin.window);
+			mg_bring_tofront (dccswin.window);
 		return;
 	}
 
-	dccswin.window = maingui_window ("dccsend", _("X-Chat: File Send List"),
-								FALSE, TRUE, close_dcc_send_window, NULL, 595, 0,
-								&dccswin.window);
-	vbox = wins_get_vbox (dccswin.window);
+	dccswin.window =
+			  mg_create_generic_tab ("dccsend", _("X-Chat: File Send List"),
+							FALSE, TRUE, close_dcc_send_window, NULL, 595, 0, &vbox);
 
 	dccswin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
 												 send_row_selected, 0,
@@ -624,12 +595,13 @@ fe_dcc_open_send_win (int passive)
 	gtk_clist_set_column_justification (GTK_CLIST (dccswin.list), 5,
 													GTK_JUSTIFY_CENTER);
 
-	bbox = gtk_hbox_new (FALSE, 2);
+	bbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dccswin.window, 0, _("Abort"), abort_send_clicked, 0, bbox);
-	gtkutil_button (dccswin.window, 0, _("Info"), info_send_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_send_clicked, 0, _("Abort"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_send_clicked, 0, _("Info"), 0, 0);
 
 	gtk_widget_show (dccswin.window);
 	fe_dcc_update_send_win ();
@@ -663,7 +635,7 @@ abort_chat_clicked (GtkWidget * wid, gpointer none)
 	if (row != -1)
 	{
 		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), row);
-		dcc_abort (dcc);
+		dcc_abort (dcc->serv->front_session, dcc);
 	}
 }
 
@@ -701,9 +673,10 @@ fe_dcc_update_chat_win (void)
 		{
 			nnew[0] = _(dccstat[(int) dcc->dccstat].name);
 			nnew[1] = dcc->nick;
-			sprintf (pos, "%u", dcc->pos);
-			sprintf (siz, "%u", dcc->size);
+			sprintf (pos, "%d", dcc->pos);
+			sprintf (siz, "%d", dcc->size);
 			nnew[4] = ctime (&dcc->starttime);
+			nnew[4][strlen (nnew[4]) - 1] = 0;	/* remove the \n */
 			row = gtk_clist_append (GTK_CLIST (dcccwin.list), nnew);
 			gtk_clist_set_row_data (GTK_CLIST (dcccwin.list), row,
 											(gpointer) dcc);
@@ -724,26 +697,20 @@ void
 fe_dcc_open_chat_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	gchar *titles[5];
-
-	titles[0] = _("Status");
-	titles[1] = _("To/From");
-	titles[2] = _("Recv");
-	titles[3] = _("Sent");
-	titles[4] = _("StartTime");
+	gchar *titles[] =
+		{ _("Status"), _("To/From"), _("Recv"), _("Sent"), _("StartTime") };
 
 	if (dcccwin.window)
 	{
 		fe_dcc_update_chat_win ();
 		if (!passive)
-			wins_bring_tofront (dcccwin.window);
+			mg_bring_tofront (dcccwin.window);
 		return;
 	}
 
-	dcccwin.window = maingui_window ("dccchat", _("X-Chat: DCC Chat List"),
-								FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0,
-								&dcccwin.window);
-	vbox = wins_get_vbox (dcccwin.window);
+	dcccwin.window =
+			  mg_create_generic_tab ("dccchat", _("X-Chat: DCC Chat List"),
+							FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0, &vbox);
 
 	dcccwin.list = gtkutil_clist_new (5, titles, vbox, GTK_POLICY_ALWAYS,
 												 chat_row_selected, 0,
@@ -753,12 +720,13 @@ fe_dcc_open_chat_win (int passive)
 	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 2, 65);
 	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 3, 65);
 
-	bbox = gtk_hbox_new (FALSE, 2);
+	bbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dcccwin.window, 0, _("Accept"), accept_chat_clicked, 0, bbox);
-	gtkutil_button (dcccwin.window, 0, _("Abort"), abort_chat_clicked, 0, bbox);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_chat_clicked, 0, _("Abort"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_chat_clicked, 0, _("Accept"), 0, 0);
 
 	gtk_widget_show (dcccwin.window);
 	fe_dcc_update_chat_win ();
