@@ -24,14 +24,20 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "fe-gtk.h"
+
+#include <gtk/gtkstock.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkvseparator.h>
+
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
 #include "../common/xchatc.h"
 #include "../common/fe.h"
 #include "menu.h"
 #include "gtkutil.h"
-#include "usermenu.h"
-#include "wins.h"
+#include "maingui.h"
 #include "editlist.h"
 
 
@@ -82,8 +88,14 @@ editlist_gui_row_selected (GtkWidget * clist, gint row, gint column,
 		gtk_clist_get_text (GTK_CLIST (clist), row, 0, &name);
 		gtk_clist_get_text (GTK_CLIST (clist), row, 1, &cmd);
 
+		name = strdup (name);
+		cmd = strdup (cmd);
+
 		gtk_entry_set_text (GTK_ENTRY (editlist_gui_entry_name), name);
 		gtk_entry_set_text (GTK_ENTRY (editlist_gui_entry_cmd), cmd);
+
+		free (name);
+		free (cmd);
 	} else
 	{
 		editlist_gui_row_unselected (0, 0, 0, 0, 0);
@@ -94,7 +106,7 @@ static void
 editlist_gui_handle_cmd (GtkWidget * igad)
 {
 	int row;
-	char *reply;
+	const char *reply;
 
 	row = gtkutil_clist_selection (editlist_gui_list);
 	if (row != -1)
@@ -108,7 +120,7 @@ static void
 editlist_gui_handle_name (GtkWidget * igad)
 {
 	int row;
-	char *ctcp;
+	const char *ctcp;
 
 	row = gtkutil_clist_selection (editlist_gui_list);
 	if (row != -1)
@@ -122,10 +134,7 @@ static void
 editlist_gui_addnew (GtkWidget * igad)
 {
 	int i;
-	gchar *nnew[2];
-
-	nnew[0] = _("*NEW*");
-	nnew[1] = _("EDIT ME");
+	gchar *nnew[2] = { _("*NEW*"), _("EDIT ME") };
 
 	i = gtk_clist_append (GTK_CLIST (editlist_gui_list), nnew);
 	gtk_clist_select_row (GTK_CLIST (editlist_gui_list), i, 0);
@@ -287,15 +296,12 @@ void
 editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 						 char *help)
 {
-	gchar *titles[2];
+	gchar *titles[] = { _("Name"), _("Command") };
 	GtkWidget *vbox, *hbox, *button;
 
-	titles[0] = _("Name");
-	titles[1] = _("Command");
-
 	if (editlist_gui_window)
 	{
-		wins_bring_tofront (editlist_gui_window);
+		mg_bring_tofront (editlist_gui_window);
 		return;
 	}
 
@@ -303,10 +309,9 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	editlist_file = file;
 	editlist_help = help;
 
-	editlist_gui_window = maingui_window (wmclass, title, TRUE, FALSE,
-														editlist_gui_close, NULL,
-														450, 250, NULL);
-	vbox = wins_get_vbox (editlist_gui_window);
+	editlist_gui_window =
+			  mg_create_generic_tab (wmclass, title, TRUE, FALSE,
+											 editlist_gui_close, NULL, 450, 250, &vbox);
 
 	editlist_gui_list = gtkutil_clist_new (2, titles, vbox, GTK_POLICY_ALWAYS,
 														editlist_gui_row_selected, 0,
@@ -318,56 +323,48 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_UP, _("Move Up"),
-							 editlist_gui_moveup, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_GO_UP, 0, editlist_gui_moveup,
+									 0, _("Move Up"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_DOWN, _("Move Dn"),
-							 editlist_gui_movedown, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_GO_DOWN, 0, editlist_gui_movedown,
+									 0, _("Move Dn"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button = gtk_vseparator_new ();
 	gtk_container_add (GTK_CONTAINER (hbox), button);
 	gtk_widget_show (button);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_BUTTON_CANCEL,
-							 _("Cancel"), gtkutil_destroy, editlist_gui_window, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_CANCEL, 0, gtkutil_destroy,
+									 editlist_gui_window, _("Cancel"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SAVE, _("Save"),
-							 editlist_gui_save, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_SAVE, 0, editlist_gui_save,
+									 0, _("Save"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
 	hbox = gtk_hbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_NEW, _("Add New"),
-							 editlist_gui_addnew, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_NEW, 0, editlist_gui_addnew,
+									 0, _("Add New"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_CUT, _("Delete"),
-							 editlist_gui_delete, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_DELETE, 0, editlist_gui_delete,
+									 0, _("Delete"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button = gtk_vseparator_new ();
 	gtk_container_add (GTK_CONTAINER (hbox), button);
 	gtk_widget_show (button);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SPELLCHECK,
-							 _("Sort"), editlist_gui_sort, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_SORT_ASCENDING, 0, editlist_gui_sort,
+									 0, _("Sort"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
-	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_HELP, _("Help"),
-							 editlist_gui_help, 0, hbox);
+	button = gtkutil_button (hbox, GTK_STOCK_HELP, 0, editlist_gui_help,
+									 0, _("Help"), 0, 0);
 	gtk_widget_set_usize (button, 100, 0);
 
 	if (!help)
