@@ -26,13 +26,20 @@
 #include <unistd.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkmain.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkprogressbar.h>
+#include <gtk/gtkbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktogglebutton.h>
+
 #include "../common/xchat.h"
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/text.h"
 #include "../common/cfgfiles.h"
 #include "../common/xchatc.h"
-#include "../common/perlc.h"
 #include "gtkutil.h"
 #include "maingui.h"
 #include "pixmaps.h"
@@ -42,6 +49,7 @@
 #include "notifygui.h"
 #include "textgui.h"
 #include "fkeys.h"
+#include "tabs.h"
 #include "urlgrab.h"
 
 #ifdef USE_PANEL
@@ -54,12 +62,7 @@
 #include <gtk/gtkinvisible.h>
 #endif
 
-#ifdef USE_GDK_PIXBUF
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#endif
 
-GdkFont *font_normal;
-GdkFont *dialog_font_normal;
 GdkPixmap *channelwin_pix;
 GdkPixmap *dialogwin_pix;
 
@@ -84,11 +87,11 @@ redraw_trans_xtexts (void)
 		if (sess->type == SESS_SHELL)
 		{
 			menu_newshell_set_palette (sess);
-			gtk_widget_queue_draw (sess->gui->textgad);
+			gtk_widget_queue_draw (sess->gui->xtext);
 		} else
 #endif
-			if (GTK_XTEXT (sess->gui->textgad)->transparent)
-				gtk_xtext_refresh (GTK_XTEXT (sess->gui->textgad), 1);
+			if (GTK_XTEXT (sess->gui->xtext)->transparent)
+				gtk_xtext_refresh (GTK_XTEXT (sess->gui->xtext), 1);
 		list = list->next;
 	}
 }
@@ -97,10 +100,10 @@ static gboolean
 handle_property_notify (GtkWidget *widget, GdkEventProperty *event,
 								gpointer user_data)
 {
-	static Atom prop = None;
+	static GdkAtom prop = GDK_NONE;
 
-	if (prop == None)
-		prop = XInternAtom (GDK_DISPLAY (), "_XROOTPMAP_ID", True);
+	if (prop == GDK_NONE)
+		prop = gdk_atom_intern ("_XROOTPMAP_ID", TRUE);
 
 	if (event->atom == prop)
 		redraw_trans_xtexts ();
@@ -113,70 +116,68 @@ handle_property_notify (GtkWidget *widget, GdkEventProperty *event,
 int
 fe_args (int argc, char *argv[])
 {
-#ifdef USE_GNOME
-	struct poptOption options[] = {
-		{"cfgdir", 'd', POPT_ARG_STRING, 0, 0, _("Config dir"), 0},
-		{"noauto", 'a', POPT_ARG_NONE, 0, 0, _("Don't Auto connect"), 0},
-#ifdef USE_PANEL
-		{"no-panel", 'n', POPT_ARG_NONE, 0, 0, _("Don't use GNOME Panel"), 0},
-#endif
-		POPT_AUTOHELP {0, '\0', 0, 0}
-	};
-#endif
+	int offset = 0;
 
 #ifdef ENABLE_NLS
-	bindtextdomain (PACKAGE, LOCALEDIR);
+	bindtextdomain (PACKAGE, PREFIX"/share/locale");
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 #endif
 
 	if (argc > 1)
 	{
-		if (!strcasecmp (argv[1], "-a") || !strcasecmp (argv[1], "--noauto"))
-		{
-			auto_connect = 0;
-		}
-#ifdef USE_PANEL
-		if (!strcasecmp (argv[1], "-n") || !strcasecmp (argv[1], "--no-panel"))
-		{
-			nopanel = TRUE;
-		}
-#endif
 		if (!strcasecmp (argv[1], "-v") || !strcasecmp (argv[1], "--version"))
 		{
 			printf (PACKAGE" "VERSION"\n");
 			return 0;
 		}
-#ifndef USE_GNOME
 		if (!strcasecmp (argv[1], "-h") || !strcasecmp (argv[1], "--help"))
 		{
-			printf(_("%s %s Options:\n\n"
-					"   --cfgdir <dir> -d\t : use a different config dir\n"
-					"   --noauto       -a\t : don't auto connect\n"
-					"   --version      -v\t : show version information\n"
-                ), PACKAGE, VERSION);
+			printf (PACKAGE" "VERSION"\n"
+					"Usage: %s [OPTIONS]... [URL]\n\n", argv[0]);
+			printf (_("Options:\n"
+					"  -d,  --cfgdir DIRECTORY   use a different config dir\n"
+					"  -a,  --no-auto            don't auto connect\n"
+					"  -v,  --version            show version information\n\n"
+					"URL:\n"
+					"  irc://server:port/channel\n\n"));
 			return 0;
 		}
+		if (!strcasecmp (argv[1], "-a") || !strcasecmp (argv[1], "--no-auto"))
+		{
+			auto_connect = 0;
+			offset++;
+		}
+#ifdef USE_PANEL
+		if (!strcasecmp (argv[1+offset], "-n") || !strcasecmp (argv[1+offset], "--no-panel"))
+		{
+			nopanel = TRUE;
+			offset++;
+		}
 #endif
 	}
 
-	if (argc > 2)
+	if (argc > 2 + offset)
 	{
-		if (!strcasecmp (argv[1], "-d") || !strcasecmp (argv[1], "--cfgdir"))
+		if (!strcasecmp (argv[1+offset], "-d") || !strcasecmp (argv[1+offset], "--cfgdir"))
 		{
-			xdir = strdup (argv[2]);
+			xdir = strdup (argv[2+offset]);
 			if (xdir[strlen (xdir) - 1] == '/')
 				xdir[strlen (xdir) - 1] = 0;
+			offset += 2;
 		}
 	}
 
-#ifndef USE_GNOME
+	if (argc > (offset + 1))
+		connect_url = strdup (argv[offset + 1]);
+
 	gtk_set_locale ();
-#endif
 
 #ifdef USE_PANEL
 	if (nopanel)
 		gnome_init_with_popt_table (argv[0], VERSION, argc, argv, options, 0, 0);
-	else {
+	else
+	{
 		CORBA_Environment ev;
 		CORBA_exception_init (&ev);
 
@@ -185,19 +186,9 @@ fe_args (int argc, char *argv[])
 						  GNORBA_INIT_SERVER_FUNC, &ev);
 		CORBA_exception_free (&ev);
 	}
-#else
-#ifdef USE_GNOME
-	gnome_init_with_popt_table (argv[0], VERSION, argc, argv, options, 0, 0);
 #else
 	gtk_init (&argc, &argv);
 #endif
-#endif
-
-#ifndef USE_GNOME
-#ifdef USE_GDK_PIXBUF
-	gdk_rgb_init();
-#endif
-#endif
 
 #ifdef USE_XLIB
 	proxy_invisible = gtk_invisible_new ();
@@ -209,53 +200,23 @@ fe_args (int argc, char *argv[])
 	/* Select for PropertyNotify events from the root window */
 	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);
 
-	gtk_signal_connect (GTK_OBJECT (proxy_invisible), "property-notify-event",
-							  GTK_SIGNAL_FUNC (handle_property_notify), NULL);
+	g_signal_connect (G_OBJECT (proxy_invisible), "property-notify-event",
+							G_CALLBACK (handle_property_notify), NULL);
 #endif
 
 	return 1;
 }
 
-GdkFont *
-my_font_load (char *fontname)
-{
-	GdkFont *font;
-	char temp[256];
-
-	if (!*fontname)
-		fontname = "fixed";
-	if (prefs.use_fontset)
-		font = gdk_fontset_load (fontname);
-	else
-		font = gdk_font_load (fontname);
-	if (!font)
-	{
-		snprintf (temp, sizeof (temp), _("Cannot open font:\n\n%s"), fontname);
-		gtkutil_simpledialog (temp);
-		font = gdk_font_load ("fixed");
-		if (!font)
-		{
-			g_error (_("gdk_font_load failed"));
-			gtk_exit (0);
-		}
-	}
-	return font;
-}
-
 GtkStyle *
-create_inputgad_style (void)
+create_input_style (void)
 {
 	GtkStyle *style;
 
 	style = gtk_style_new ();
-
-	gdk_font_unref (style->font);
-	gdk_font_ref (font_normal);
-	style->font = font_normal;
-
-	style->base[GTK_STATE_NORMAL] = colors[19];
-	style->bg[GTK_STATE_NORMAL] = colors[19];
-	style->fg[GTK_STATE_NORMAL] = colors[18];
+	/* FIXME: fontd leaked, bg/fg hide cursor */
+	style->font_desc = pango_font_description_from_string (prefs.font_normal);
+/*	style->text[GTK_STATE_NORMAL] = colors[18];
+	style->base[GTK_STATE_NORMAL] = colors[19];*/
 
 	return style;
 }
@@ -263,16 +224,13 @@ create_inputgad_style (void)
 void
 fe_init (void)
 {
-	font_normal = my_font_load (prefs.font_normal);
-	dialog_font_normal = my_font_load (prefs.dialog_font_normal);
-
 	palette_load ();
 	key_init ();
 	pixmaps_init ();
 
 	channelwin_pix = pixmap_load_from_file (prefs.background);
 	dialogwin_pix = pixmap_load_from_file (prefs.background_dialog);
-	inputgad_style = create_inputgad_style ();
+	input_style = create_input_style ();
 }
 
 void
@@ -289,7 +247,8 @@ fe_main (void)
 
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
-	sleep (3);
+	if (prefs.wait_on_exit)
+		sleep (3);
 }
 
 void
@@ -324,11 +283,35 @@ fe_timeout_remove (int tag)
 }
 
 void
-fe_new_window (struct session *sess)
+fe_new_window (session *sess)
 {
-	sess->gui = malloc (sizeof (struct session_gui));
-	memset (sess->gui, 0, sizeof (struct session_gui));
-	create_window (sess);
+	static const char *locale = NULL;
+
+	mg_changui_new (sess, NULL);
+
+	if (locale == NULL)
+	{
+		fe_print_text (sess,
+			"*** THIS IS A DEVELOPMENT VERSION! *** "__DATE__" "VERSION" ***\n");
+		fe_print_text (sess,
+			"Your current locale is:\n");
+
+		g_get_charset (&locale);
+		fe_print_text (sess, (char *)locale);
+
+		if (strcmp (locale, "ANSI_X3.4-1968") == 0)
+			fe_print_text (sess, "\00304\002This locale restricts you to ASCII only!\002\017\n");
+
+		fe_print_text (sess,
+			"Set environmental variable LC_ALL to change \00312(\017e.g. \002export LC_ALL=en_US\002\00312)\017\n");
+#ifdef USE_PANGO
+		fe_print_text (sess, "Using Pango for text rendering. This will be slow!\n");
+#endif
+#ifdef USE_XFT
+		fe_print_text (sess, "Using Xft for text rendering (good!).\n");
+
+#endif
+	}
 }
 
 void
@@ -352,8 +335,8 @@ fe_message (char *msg, int wait)
 	dialog = gtkutil_simpledialog (msg);
 	if (wait)
 	{
-		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-								  null_this_var, &dialog);
+		g_signal_connect (G_OBJECT (dialog), "destroy",
+								G_CALLBACK (null_this_var), &dialog);
 		while (dialog)
 			gtk_main_iteration ();
 	}
@@ -394,38 +377,58 @@ fe_input_add (int sok, int read, int write, int ex, void *func, void *data)
 }
 
 void
-fe_set_topic (struct session *sess, char *topic)
+fe_set_topic (session *sess, char *topic)
 {
-	char buf[512];
-
-	gtk_entry_set_text (GTK_ENTRY (sess->gui->topicgad), topic);
-
-	if (sess->type == SESS_CHANNEL)
+	if (!sess->gui->is_tab || sess == current_tab)
 	{
-		snprintf (buf, sizeof (buf), _("Topic for %s is: %s"), sess->channel,
-					 topic);
-		add_tip (sess->gui->topicgad, buf);
+		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic), topic);
+		mg_set_topic_tip (sess);
+	} else
+	{
+		if (sess->res->topic_text)
+			free (sess->res->topic_text);
+		sess->res->topic_text = strdup (topic);
 	}
 }
 
 void
 fe_set_hilight (struct session *sess)
 {
+	if (sess->gui->is_tab)
+	{
+		sess->nick_said = TRUE;
 #ifdef USE_PANEL
-	if (sess->gui->panel_button)
-		gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
-									 bluetab_style);
+		if (sess->gui->panel_button)
+			gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
+									 blue_style);
 #endif
-	gtk_widget_set_style (sess->gui->changad, bluetab_style);
-	if (prefs.treeview)
-		tree_blue_style (sess);
+		tab_style (sess->gui->tab, blue_style);
+	}
+}
+
+static void
+fe_update_mode_entry (session *sess, GtkWidget *entry, char **text, char *new_text)
+{
+	if (!sess->gui->is_tab || sess == current_tab)
+	{
+		if (sess->gui->flag_wid[0])	/* channel mode buttons enabled? */
+			gtk_entry_set_text (GTK_ENTRY (entry), new_text);
+	} else
+	{
+		if (sess->gui->is_tab)
+		{
+			if (*text)
+				free (*text);
+			*text = strdup (new_text);
+		}
+	}
 }
 
 void
 fe_update_channel_key (struct session *sess)
 {
-	if (sess->gui->flag_wid[0])	/* channel mode buttons enabled? */
-		gtk_entry_set_text ((GtkEntry*)sess->gui->key_entry, sess->channelkey);
+	fe_update_mode_entry (sess, sess->gui->key_entry,
+								 &sess->res->key_text, sess->channelkey);
 	fe_set_title (sess);
 }
 
@@ -434,11 +437,9 @@ fe_update_channel_limit (struct session *sess)
 {
 	char tmp[16];
 
-	if (sess->gui->flag_wid[0])	/* channel mode buttons enabled? */
-	{
-		sprintf (tmp, "%d", sess->limit);
-		gtk_entry_set_text ((GtkEntry*)sess->gui->limit_entry, tmp);
-	}
+	sprintf (tmp, "%d", sess->limit);
+	fe_update_mode_entry (sess, sess->gui->limit_entry,
+								 &sess->res->limit_text, tmp);
 }
 
 int
@@ -475,17 +476,16 @@ fe_notify_update (char *name)
 void
 fe_text_clear (struct session *sess)
 {
-	gtk_xtext_clear (GTK_XTEXT (sess->gui->textgad));
+	gtk_xtext_clear (sess->res->buffer);
 }
 
 void
 fe_close_window (struct session *sess)
 {
-#ifdef USE_PERL
-	perl_notify_kill (sess);
-#endif
-
-	gtk_widget_destroy (sess->gui->window);
+	if (sess->gui->is_tab)
+		gtk_widget_destroy (sess->gui->tab);
+	else
+		gtk_widget_destroy (sess->gui->window);
 }
 
 static int
@@ -520,10 +520,10 @@ updatedate_bar (struct session *sess)
 void
 fe_progressbar_start (struct session *sess)
 {
-	if (sess->gui->op_box)
+	if (sess->gui->nick_box)
 	{
 		sess->gui->bar = gtk_progress_bar_new ();
-		gtk_box_pack_start (GTK_BOX (sess->gui->op_box), sess->gui->bar, 0, 0,
+		gtk_box_pack_start (GTK_BOX (sess->gui->nick_box), sess->gui->bar, 0, 0,
 								  0);
 		gtk_widget_show (sess->gui->bar);
 		sess->server->bartag = fe_timeout_add (50, updatedate_bar, sess);
@@ -531,27 +531,23 @@ fe_progressbar_start (struct session *sess)
 }
 
 void
-fe_progressbar_end (struct session *sess)
+fe_progressbar_end (struct server *serv)
 {
-	struct server *serv;
 	GSList *list = sess_list;
+	session *sess;
 
-	if (sess)
+	while (list)				  /* check all windows that use this server and  *
+									   * remove the connecting graph, if it has one. */
 	{
-		serv = sess->server;
-		while (list)				  /* check all windows that use this server and  *
-										   * remove the connecting graph, if it has one. */
+		sess = (struct session *) list->data;
+		if (sess->server == serv && sess->gui->bar)
 		{
-			sess = (struct session *) list->data;
-			if (sess->server == serv && sess->gui->bar)
-			{
-				if (GTK_IS_WIDGET (sess->gui->bar))
-					gtk_widget_destroy (sess->gui->bar);
-				sess->gui->bar = 0;
-				fe_timeout_remove (sess->server->bartag);
-			}
-			list = list->next;
+			fe_timeout_remove (sess->server->bartag);
+			if (GTK_IS_WIDGET (sess->gui->bar))
+				gtk_widget_destroy (sess->gui->bar);
+			sess->gui->bar = 0;
 		}
+		list = list->next;
 	}
 }
 
@@ -560,12 +556,12 @@ fe_print_text (struct session *sess, char *text)
 {
 	int indent;
 
-	if (sess->type == SESS_DIALOG)
+	/*if (sess->type == SESS_DIALOG)
 		indent = prefs.dialog_indent_nicks;
-	else
+	else*/
 		indent = prefs.indent_nicks;
 
-	PrintTextRaw (sess->gui->textgad, text, indent);
+	PrintTextRaw (sess->res->buffer, text, indent);
 
 	if (prefs.limitedtabhighlight && !sess->highlight_tab)
 		return;
@@ -575,20 +571,17 @@ fe_print_text (struct session *sess, char *text)
 	if (!sess->new_data && !sess->nick_said && sess != current_tab)
 	{
 #ifdef USE_PANEL
-
 		if (sess->gui->panel_button)
 			gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
-										 redtab_style);
+										 red_style);
 #endif
-		if (prefs.treeview)
-			tree_red_style (sess);
 	}
 
 	if (!sess->new_data && sess != current_tab &&
-		 sess->is_tab && !sess->nick_said)
+		 sess->gui->is_tab && !sess->nick_said)
 	{
 		sess->new_data = TRUE;
-		gtk_widget_set_style (sess->gui->changad, redtab_style);
+		tab_style (sess->gui->tab, red_style);
 	}
 }
 
@@ -598,40 +591,35 @@ fe_beep (void)
 	gdk_beep ();
 }
 
-int
-fe_is_beep (struct session *sess)
-{
-	if (GTK_TOGGLE_BUTTON (sess->gui->beepbutton)->active)
-		return 1;
-	return 0;
-}
+typedef struct {
+	session *sess;
+	char *sstr;
+} fe_lastlog_info;
 
-int
-fe_is_confmode (struct session *sess)
+static void
+fe_lastlog_foreach (GtkXText *xtext, unsigned char *text, void *data)
 {
-	if (GTK_TOGGLE_BUTTON (sess->gui->confbutton)->active)
-		return 1;
-	return 0;
+	fe_lastlog_info *info = data;
+	 
+	if (nocasestrstr (text, info->sstr))
+		PrintText (info->sess, text);
 }
 
 void
 fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
 {
-	textentry *ent;
+	if (gtk_xtext_is_empty (GTK_XTEXT (sess->gui->xtext)))
+	{
+		PrintText (lastlog_sess, _("Search buffer is empty.\n"));
+	}
+	else
+	{
+		fe_lastlog_info info;
 
-	ent = GTK_XTEXT(sess->gui->textgad)->text_first;
-	if (!ent)
-	{
-		PrintText (lastlog_sess, "Search buffer is empty.\n");
-	} else
-	{
-		do
-		{
-			if (nocasestrstr (ent->str, sstr))
-				PrintText (lastlog_sess, ent->str);
-			ent = ent->next;
-		}
-		while (ent);
+		info.sess = lastlog_sess;
+		info.sstr = sstr;
+		
+		gtk_xtext_foreach (GTK_XTEXT (sess->gui->xtext), fe_lastlog_foreach, &info);
 	}
 }
 
@@ -661,15 +649,21 @@ fe_set_lag (server *serv, int lag)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (sess->gui->lagometer)
+			if (!sess->gui->is_tab || current_tab == sess)
 			{
-				gtk_progress_bar_update ((GtkProgressBar *) sess->gui->lagometer, per);
-			}
-			if (sess->gui->laginfo)
+				if (sess->gui->lagometer)
+				{
+					gtk_progress_bar_update ((GtkProgressBar *) sess->gui->lagometer, per);
+				}
+				if (sess->gui->laginfo)
+				{
+					snprintf (tip, sizeof(tip) - 1, "%s%d.%ds",
+						serv->lag_sent ? "+" : "", lag / 10, lag % 10);
+					gtk_label_set_text ((GtkLabel *) sess->gui->laginfo, tip);
+				}
+			} else
 			{
-				snprintf (tip, sizeof(tip) - 1, "%s%d.%ds",
-					serv->lag_sent ? "+" : "", lag / 10, lag % 10);
-				gtk_label_set_text ((GtkLabel *) sess->gui->laginfo, tip);
+				sess->res->lag_value = per;
 			}
 		}
 		list = list->next;
@@ -693,17 +687,22 @@ fe_set_throttle (server *serv)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (sess->gui->throttlemeter)
+			if (!sess->gui->is_tab || current_tab == sess)
 			{
-				gtk_progress_bar_update ((GtkProgressBar *) sess->gui->throttlemeter, per);
-			}
-			if (sess->gui->throttleinfo)
+				if (sess->gui->throttlemeter)
+				{
+					gtk_progress_bar_update ((GtkProgressBar *) sess->gui->throttlemeter, per);
+				}
+				if (sess->gui->throttleinfo)
+				{
+					snprintf (tbuf, sizeof(tbuf) - 1, "%d bytes", serv->sendq_len);
+					gtk_label_set_text ((GtkLabel *) sess->gui->throttleinfo, tbuf);
+				}
+			} else
 			{
-				snprintf (tbuf, sizeof(tbuf) - 1, "%d bytes", serv->sendq_len);
-				gtk_label_set_text ((GtkLabel *) sess->gui->throttleinfo, tbuf);
+				sess->res->queue_value = per;
 			}
 		}
 		list = list->next;
 	}
 }
-
