@@ -30,6 +30,20 @@
 #include <ctype.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtklabel.h>
+#include <gtk/gtkeditable.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkvscrollbar.h>
+
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
@@ -41,15 +55,14 @@
 #include "gtkutil.h"
 #include "menu.h"
 #include "xtext.h"
-#include "wins.h"
 #include "palette.h"
 #include "maingui.h"
 #include "textgui.h"
 #include "fkeys.h"
 
 
-static int tab_nick_comp (GtkWidget * t, int shift);
-static void nick_comp_chng (GtkWidget * t, int updown);
+static int tab_nick_comp (session *sess, GtkWidget * t, int shift);
+static void nick_comp_chng (session *sess, GtkWidget * t, int updown);
 static void replace_handle (GtkWidget * wid);
 
 
@@ -213,13 +226,13 @@ key_get_key_name (int keyval)
 
  */
 
-int
-key_handle_key_press (GtkWidget * wid, GdkEventKey * evt,
-							 struct session *sess)
+gboolean
+key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, gpointer user_data)
 {
 	struct key_binding *kb, *last = NULL;
 	int keyval = evt->keyval;
 	int mod, n;
+	session *sess = current_sess;
 
 	mod = evt->state & (STATE_CTRL | STATE_ALT | STATE_SHIFT);
 
@@ -256,13 +269,12 @@ key_handle_key_press (GtkWidget * wid, GdkEventKey * evt,
 	}
 
 	/* check if it's a return or enter */
-	if ((evt->keyval == GDK_Return)
-		 || (evt->keyval == GDK_KP_Enter /*&& mod == 0*/))
-		handle_inputgad (wid, sess);
+	if ((evt->keyval == GDK_Return) || (evt->keyval == GDK_KP_Enter))
+		mg_inputbox_cb (wid, sess);
 
 	if (evt->keyval == GDK_KP_Enter)
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (wid), "key_press_event");
-	return 1;
+	return 0;
 }
 
 /* Walks keys_root and free()'s everything */
@@ -375,15 +387,9 @@ key_dialog_close ()
 static void
 key_dialog_add_new (GtkWidget * button, GtkCList * list)
 {
-	gchar *strs[5];
+	gchar *strs[] = { "", _("<none>"), _("<none>"), _("<none>"), _("<none>") };
 	struct key_binding *kb;
 
-	strs[0] = "";
-	strs[1] = _("<none>");
-	strs[2] = _("<none>");
-	strs[3] = _("<none>");
-	strs[4] = _("<none>");
-
 	kb = malloc (sizeof (struct key_binding));
 
 	kb->keyval = 0;
@@ -451,8 +457,8 @@ key_dialog_sel_act (GtkWidget * un, int num)
 								  _(key_actions[num].name));
 		if (key_actions[num].help)
 		{
-			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
-			PrintTextRaw (key_dialog_text, _(key_actions[num].help), 0);
+			gtk_xtext_clear (GTK_XTEXT (key_dialog_text)->buffer);
+			PrintTextRaw (GTK_XTEXT (key_dialog_text)->buffer, _(key_actions[num].help), 0);
 		}
 	}
 }
@@ -474,8 +480,8 @@ key_dialog_sel_row (GtkWidget * clist, gint row, gint column,
 											  kb->action);
 		if (key_actions[kb->action].help)
 		{
-			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
-			PrintTextRaw (key_dialog_text, _(key_actions[kb->action].help), 0);
+			gtk_xtext_clear (GTK_XTEXT (key_dialog_text)->buffer);
+			PrintTextRaw (GTK_XTEXT (key_dialog_text)->buffer, _(key_actions[kb->action].help), 0);
 		}
 	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (key_dialog_tog_c),
@@ -588,7 +594,7 @@ key_dialog_set_key (GtkWidget * entry, GdkEventKey * evt, void *none)
 static void
 key_dialog_set_data (GtkWidget * entry, int d)
 {
-	char *text = gtk_entry_get_text (GTK_ENTRY (entry));
+	const char *text = gtk_entry_get_text (GTK_ENTRY (entry));
 	int row = gtkutil_clist_selection (key_dialog_kb_clist);
 	struct key_binding *kb;
 	char *buf;
@@ -624,29 +630,25 @@ key_dialog_show ()
 {
 	GtkWidget *vbox, *hbox, *list, *vbox2, *wid, *wid2, *wid3, *hbox2;
 	struct key_binding *kb;
-	gchar *titles[] = { NULL, NULL, NULL, "1", "2" };
+	gchar *titles[] = { _("Mod"), _("Key"), _("Action"), "1", "2" };
 	char temp[32];
 	int i;
 
-	titles[0] = _("Mod");
-	titles[1] = _("Key");
-	titles[2] = _("Action");
-
 	if (key_dialog)
 	{
-		wins_bring_tofront (key_dialog);
+		mg_bring_tofront (key_dialog);
 		return;
 	}
 
-	key_dialog = maingui_window ("editkeys", _("X-Chat: Edit Key Bindings"),
-								TRUE, FALSE, key_dialog_close, NULL, 560, 330, NULL);
-	vbox = wins_get_vbox (key_dialog);
+	key_dialog =
+			  mg_create_generic_tab ("editkeys", _("X-Chat: Edit Key Bindings"),
+								TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox);
 
 	hbox = gtk_hbox_new (0, 2);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, 1, 1, 0);
 
 	list = gtkutil_clist_new (5, titles, hbox, 0, key_dialog_sel_row, 0, NULL,
-									  0, 0);
+									  0, GTK_SELECTION_SINGLE);
 	gtk_widget_set_usize (list, 400, 0);
 	key_dialog_kb_clist = list;
 
@@ -705,9 +707,10 @@ key_dialog_show ()
 	{
 		wid3 = gtk_menu_item_new_with_label (_(key_actions[i].name));
 		gtk_widget_show (wid3);
-		gtk_menu_append (GTK_MENU (wid2), wid3);
+		gtk_menu_shell_append (GTK_MENU_SHELL (wid2), wid3);
 		gtk_signal_connect (GTK_OBJECT (wid3), "activate",
-								  GTK_SIGNAL_FUNC (key_dialog_sel_act), GINT_TO_POINTER (i));
+								  GTK_SIGNAL_FUNC (key_dialog_sel_act),
+								  GINT_TO_POINTER (i));
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (wid), wid2);
@@ -737,9 +740,8 @@ key_dialog_show ()
 	hbox2 = gtk_hbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox2, 0, 0, 1);
 
-	wid = gtk_xtext_new (0, 0);
-	gtk_xtext_set_palette (GTK_XTEXT (wid), colors);
-	gtk_xtext_set_font (GTK_XTEXT (wid), font_normal);
+	wid = gtk_xtext_new (colors, 0);
+	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
 	gtk_xtext_set_background (GTK_XTEXT (wid),
 									  channelwin_pix,
 									  prefs.transparent, prefs.tint);
@@ -767,7 +769,9 @@ key_save_kbs (char *fn)
 	if (!fn)
 		snprintf (buf, 510, "%s/keybindings.conf", get_xdir ());
 	else
-		strncpy (buf, fn, 510);
+	{
+		safe_strcpy (buf, fn, sizeof (buf));
+	}
 	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
 	if (fd < 0)
 	{
@@ -1127,8 +1131,6 @@ key_action_page_switch (GtkWidget * wid, GdkEventKey * evt, char *d1,
 
 	if (!d1)
 		return 1;
-	if (!main_window)
-		return 1;
 
 	len = strlen (d1);
 	if (!len)
@@ -1149,17 +1151,9 @@ key_action_page_switch (GtkWidget * wid, GdkEventKey * evt, char *d1,
 	if (!d2)
 		num--;
 	if (!d2 || d2[0] == 0)
-		gtk_notebook_set_page (GTK_NOTEBOOK (main_book), num);
+		mg_switch_page (FALSE, num);
 	else
-	{
-		len = g_list_length (((GtkNotebook *) main_book)->children);
-		i = gtk_notebook_get_current_page (GTK_NOTEBOOK (main_book)) + num;
-		if (i >= len)
-			i = 0;
-		if (i < 0)
-			i = len - 1;
-		gtk_notebook_set_page (GTK_NOTEBOOK (main_book), i);
-	}
+		mg_switch_page (TRUE, num);
 	return 0;
 }
 
@@ -1172,9 +1166,9 @@ key_action_insert (GtkWidget * wid, GdkEventKey * evt, char *d1, char *d2,
 	if (!d1)
 		return 1;
 
-	tmp_pos = GTK_EDITABLE (wid)->current_pos;
+	tmp_pos = gtk_editable_get_position (GTK_EDITABLE (wid));
 	gtk_editable_insert_text (GTK_EDITABLE (wid), d1, strlen (d1), &tmp_pos);
-	GTK_EDITABLE (wid)->current_pos = tmp_pos;
+	gtk_editable_set_position (GTK_EDITABLE (wid), tmp_pos);
 	return 2;
 }
 
@@ -1195,13 +1189,13 @@ key_action_scroll_page (GtkWidget * wid, GdkEventKey * evt, char *d1,
 		adj = GTK_RANGE (sess->gui->vscrollbar)->adjustment;
 		if (up)						  /* PageUp */
 		{
-			value = adj->value - adj->page_size;
+			value = adj->value - (adj->page_size - 1);
 			if (value < 0)
 				value = 0;
 		} else
 		{								  /* PageDown */
 			end = adj->upper - adj->lower - adj->page_size;
-			value = adj->value + adj->page_size;
+			value = adj->value + (adj->page_size - 1);
 			if (value > end)
 				value = end;
 		}
@@ -1220,6 +1214,8 @@ key_action_set_buffer (GtkWidget * wid, GdkEventKey * evt, char *d1, char *d2,
 		return 1;
 
 	gtk_entry_set_text (GTK_ENTRY (wid), d1);
+	gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+
 	return 2;
 }
 
@@ -1231,7 +1227,10 @@ key_action_history_up (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 
 	new_line = history_up (&sess->history);
 	if (new_line)
+	{
 		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
+		gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+	}
 
 	return 2;
 }
@@ -1244,7 +1243,10 @@ key_action_history_down (GtkWidget * wid, GdkEventKey * ent, char *d1,
 
 	new_line = history_down (&sess->history);
 	if (new_line)
+	{
 		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
+		gtk_editable_set_position (GTK_EDITABLE (wid), -1);
+	}
 
 	return 2;
 }
@@ -1255,11 +1257,11 @@ key_action_tab_comp (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 {
 	if (d1 && d1[0])
 	{
-		if (tab_nick_comp (wid, 1) == -1)
+		if (tab_nick_comp (sess, wid, 1) == -1)
 			return 1;
 	} else
 	{
-		if (tab_nick_comp (wid, 0) == -1)
+		if (tab_nick_comp (sess, wid, 0) == -1)
 			return 1;
 	}
 	return 2;
@@ -1270,9 +1272,9 @@ key_action_comp_chng (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 							 struct session *sess)
 {
 	if (d1 && d1[0] != 0)
-		nick_comp_chng (wid, 1);
+		nick_comp_chng (sess, wid, 1);
 	else
-		nick_comp_chng (wid, 0);
+		nick_comp_chng (sess, wid, 0);
 
 	return 2;
 }
@@ -1289,7 +1291,7 @@ static int
 key_action_move_tab_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	wins_move_leftorright (sess->gui->window, TRUE);
+	mg_move_tab (sess->gui->window, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1297,7 +1299,7 @@ static int
 key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	wins_move_leftorright (sess->gui->window, FALSE);
+	mg_move_tab (sess->gui->window, -1);
 	return 2;						  /* -''- */
 }
 
@@ -1305,7 +1307,7 @@ static int
 key_action_put_history (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	history_add (&sess->history, gtk_entry_get_text (GTK_ENTRY (wid)));
+	history_add (&sess->history, (char *)gtk_entry_get_text (GTK_ENTRY (wid)));
 	gtk_entry_set_text (GTK_ENTRY (wid), "");
 	return 2;						  /* -''- */
 }
@@ -1321,7 +1323,7 @@ key_action_put_history (GtkWidget * wid, GdkEventKey * ent, char *d1,
 static void
 replace_handle (GtkWidget *t)
 {
-	char *text, *postfix_pnt;
+	const char *text, *postfix_pnt;
 	struct popup *pop;
 	GSList *list = replace_list;
 	char word[256];
@@ -1382,31 +1384,13 @@ replace_handle (GtkWidget *t)
 				snprintf (word, sizeof (word), "%s%s", pop->cmd, postfix);
 			strcat (outbuf, word);
 			gtk_entry_set_text (GTK_ENTRY (t), outbuf);
+			gtk_editable_set_position (GTK_EDITABLE (t), -1);
 			return;
 		}
 		list = list->next;
 	}
 }
 
-static struct session *
-find_session_from_inputgad (GtkWidget * w)
-{
-	GSList *list = sess_list;
-	struct session *sess;
-
-	/* First find the session from the widget */
-	while (list)
-	{
-		sess = (struct session *) list->data;
-		if (sess->gui->inputgad == w)
-			return sess;
-		list = list->next;
-	}
-
-	/*Erm, we didn't find ANY valid session, HELP! */
-	return NULL;
-}
-
 static int
 nick_comp_get_nick (char *tx, char *n)
 {
@@ -1427,20 +1411,16 @@ nick_comp_get_nick (char *tx, char *n)
 }
 
 static void
-nick_comp_chng (GtkWidget * t, int updown)
+nick_comp_chng (session *sess, GtkWidget * t, int updown)
 {
-	struct session *sess;
 	struct User *user, *last = NULL;
-	char *text, nick[NICKLEN], *lastnick, *newtext;
+	const char *text;
+	char nick[NICKLEN], *lastnick, *newtext;
 	int len, slen;
 	GSList *list;
 
-	sess = find_session_from_inputgad (t);
-	if (sess == NULL)
-		return;
-
 	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (nick_comp_get_nick (text, nick) == -1)
+	if (nick_comp_get_nick ((char *)text, nick) == -1)
 		return;
 
 	len = strlen (nick);
@@ -1456,7 +1436,7 @@ nick_comp_chng (GtkWidget * t, int updown)
 			list = list->next;
 			continue;
 		}
-		if (strncasecmp (user->nick, nick, len) == 0)
+		if (rfc_ncasecmp (user->nick, nick, len) == 0)
 		{
 			if (updown == 0)
 			{
@@ -1478,6 +1458,7 @@ nick_comp_chng (GtkWidget * t, int updown)
 			sprintf (newtext, "%s%s", lastnick, text + strlen (nick));
 			gtk_entry_set_text (GTK_ENTRY (t), newtext);
 			free (newtext);
+			gtk_editable_set_position (GTK_EDITABLE (t), -1);
 			return;
 
 		}
@@ -1508,31 +1489,28 @@ tab_comp_find_common (char *a, char *b)
 }
 
 static void
-tab_comp_cmd (GtkWidget * t)
+tab_comp_cmd (session *sess, GtkWidget * t)
 {
-	char *text, *last = NULL, *cmd, *postfix = NULL;
+	char *last = NULL, *cmd, *postfix = NULL;
+	const char *text;
 	GSList *list = command_list;
 	struct popup *pop;
 	int len, i, slen;
-	struct session *sess;
 	char buf[2048];
 	char lcmd[2048];
 
 	text = gtk_entry_get_text (GTK_ENTRY (t));
-
-	sess = find_session_from_inputgad (t);
-
+	if (text[0] == 0 || text[1] == 0)
+		return;
 	text++;
 	len = strlen (text);
-	if (len == 0)
-		return;
+
 	for (i = 0; i < len; i++)
 	{
 		if (text[i] == ' ')
 		{
-			postfix = &text[i + 1];
-			text[i] = 0;
-			len = strlen (text);
+			postfix = (char *)&text[i + 1];
+			len = i - 1;
 			break;
 		}
 	}
@@ -1546,7 +1524,7 @@ tab_comp_cmd (GtkWidget * t)
 			list = list->next;
 			continue;
 		}
-		if (strncasecmp (pop->name, text, len) == 0)
+		if (strncasecmp (pop->name, (char *)text, len) == 0)
 		{
 			if (last == NULL)
 			{
@@ -1577,7 +1555,7 @@ tab_comp_cmd (GtkWidget * t)
 			i++;
 			continue;
 		}
-		if (strncasecmp (cmd, text, len) == 0)
+		if (strncasecmp (cmd, (char *)text, len) == 0)
 		{
 			if (last == NULL)
 			{
@@ -1612,6 +1590,7 @@ tab_comp_cmd (GtkWidget * t)
 		else
 			snprintf (buf, sizeof (buf), "/%s %s", last, postfix);
 		gtk_entry_set_text (GTK_ENTRY (t), buf);
+		gtk_editable_set_position (GTK_EDITABLE (t), -1);
 		return;
 	} else if (strlen (lcmd) > (sizeof (buf) - 1))
 		return;
@@ -1620,6 +1599,7 @@ tab_comp_cmd (GtkWidget * t)
 	else
 		snprintf (buf, sizeof (buf), "/%s %s", lcmd, postfix);
 	gtk_entry_set_text (GTK_ENTRY (t), buf);
+	gtk_editable_set_position (GTK_EDITABLE (t), -1);
 }
 
 /* In the following 'b4' is *before* the text (just say b4 and before out loud)
@@ -1678,16 +1658,16 @@ tab_nick_comp_next (struct session *sess, GtkWidget * wid, char *b4,
 
 	pos = strlen (buf) - strlen (c5);
 	gtk_entry_set_text (GTK_ENTRY (wid), buf);
-	GTK_EDITABLE (wid)->current_pos = pos;
+	gtk_editable_set_position (GTK_EDITABLE (wid), pos);
 
 	return 1;
 }
 
 static int
-tab_nick_comp (GtkWidget *t, int shift)
+tab_nick_comp (session *sess, GtkWidget *t, int shift)
 {
-	struct session *sess;
 	struct User *user, *match_user = NULL;
+	const char *origtext;
 	char *text, not_nick_chars[5] = { ' ', '.', '?', 0, 0};
 	int len, slen, first, i, cursor_pos, match_count = 0, match_pos = 0;
 	char buf[2048], nick_buf[2048], *b4 = NULL, *c5 = NULL;
@@ -1696,24 +1676,21 @@ tab_nick_comp (GtkWidget *t, int shift)
 	GSList *list, *match_list = NULL, *first_match = NULL;
 	int ret = 0;	/* return value */
 
-	sess = find_session_from_inputgad (t);
-	if (sess == NULL)
-		return 0;
 	if (sess->type == SESS_DIALOG)
 		return 0;
 
-	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (text[0] == '/')
+	origtext = gtk_entry_get_text (GTK_ENTRY (t));
+	if (origtext[0] == '/')
 	{
-		if (!strchr (text, ' '))
+		if (!strchr (origtext, ' '))
 		{
-			tab_comp_cmd (t);
+			tab_comp_cmd (sess, t);
 			return 0;
 		}
 	}
 
 	/* make a copy, since we use this buffer */
-	text = strdup (text);
+	text = strdup (origtext);
 	len = strlen (text);
 
 	/* Is the text more than just a nick? */
@@ -1734,12 +1711,12 @@ tab_nick_comp (GtkWidget *t, int shift)
 				 text[len - (space ? 2 : 1)] == prefs.nick_suffix[0])
 			{
 				/* This causes the nickname to cycle. */
-				nick_comp_chng (t, shift);
+				nick_comp_chng (sess, t, shift);
 				goto compdone1; /* free(text) and return 0; */
 			}
 		}
 
-		cursor_pos = GTK_EDITABLE (t)->current_pos;
+		cursor_pos = gtk_editable_get_position (GTK_EDITABLE (t));
 
 		/* !! FIXME !! */
 		if (len - cursor_pos < 0)
@@ -1806,7 +1783,7 @@ tab_nick_comp (GtkWidget *t, int shift)
 			list = list->next;
 			continue;
 		}
-		if (strncasecmp (user->nick, text, len) == 0)
+		if (rfc_ncasecmp (user->nick, text, len) == 0)
 			match_list = g_slist_prepend (match_list, user->nick);
 		list = list->next;
 	}
@@ -1860,9 +1837,7 @@ tab_nick_comp (GtkWidget *t, int shift)
 		nick_buf[0] = 0;
 		while (match_list)
 		{
-			snprintf (nick_buf, sizeof (nick_buf), "%s%s ", nick_buf,
-						 (char *)match_list->data);
-			nick_buf[sizeof (nick_buf) - 1] = 0;
+			sprintf (nick_buf, "%s%s ", nick_buf, (char *)match_list->data);
 			match_list = match_list->next;
 		}
 		PrintText (sess, nick_buf);
@@ -1899,7 +1874,7 @@ tab_nick_comp (GtkWidget *t, int shift)
 
 	gtk_entry_set_text (GTK_ENTRY (t), buf);
 	/* GeEkMaN: Restore the last cursor position */
-	GTK_EDITABLE (t)->current_pos = cursor_pos;
+	gtk_editable_set_position (GTK_EDITABLE (t), cursor_pos);
 
 compdone:
 	if (b4)
