@@ -14,7 +14,7 @@ int nopanel = FALSE;
 
 
 static void
-panel_invalidate (GtkWidget * wid, gpointer * arg2)
+panel_invalidate (GtkWidget * wid, gpointer arg2)
 {
 	panel_applet = NULL;
 }
@@ -115,7 +115,7 @@ gui_panel_here_clicked (GtkWidget * button, struct session *sess)
 }
 
 static void
-gui_panel_destroy_view_popup (GtkWidget * popup, /* Event */ gpointer * arg2,
+gui_panel_destroy_view_popup (GtkWidget * popup, /* Event */ gpointer arg2,
 										struct session *sess)
 {
 	/* BODGE ALERT !! BODGE ALERT !! --AGL */
