@@ -16,20 +16,27 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define GTK_DISABLE_DEPRECATED
+
 #include <stdio.h>
 #include <string.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkentry.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkstock.h>
+
 #include "../common/xchat.h"
 #include "../common/util.h"
 #include "../common/xchatc.h"
 #include "gtkutil.h"
 #include "xtext.h"
-#include "wins.h"
+#include "maingui.h"
 
 
 static void
-search_search (session * sess, char *text)
+search_search (session * sess, const char *text)
 {
 	static void *last = NULL;
 
@@ -40,16 +47,16 @@ search_search (session * sess, char *text)
 		return;
 	}
 
-	last = gtk_xtext_search (GTK_XTEXT (sess->gui->textgad), text, last);
+	last = gtk_xtext_search (GTK_XTEXT (sess->gui->xtext), text, last);
 }
 
 static void
 search_find_cb (GtkWidget * button, session * sess)
 {
 	GtkEntry *entry;
-	char *text;
+	const char *text;
 
-	entry = gtk_object_get_user_data (GTK_OBJECT (button));
+	entry = g_object_get_data (G_OBJECT (button), "e");
 	text = gtk_entry_get_text (entry);
 	search_search (sess, text);
 }
@@ -78,16 +85,11 @@ search_open (session * sess)
 {
 	GtkWidget *win, *hbox, *vbox, *entry, *wid;
 
-	if (sess->type == SESS_SHELL)
-		return;
-
-	win = maingui_window ("search", _("X-Chat: Search"), TRUE, FALSE,
-								 NULL, NULL, 0, 0, NULL);
+	win = mg_create_generic_tab ("search", _("X-Chat: Search"), TRUE, FALSE,
+								 NULL, NULL, 0, 0, &vbox);
 
 	gtk_container_set_border_width (GTK_CONTAINER (win), 20);
 
-	vbox = wins_get_vbox (win);
-
 	hbox = gtk_hbox_new (0, 10);
 	gtk_container_add (GTK_CONTAINER (vbox), hbox);
 	gtk_widget_show (hbox);
@@ -95,20 +97,23 @@ search_open (session * sess)
 	gtkutil_label_new (_("Find:"), hbox);
 
 	entry = gtk_entry_new ();
-	gtk_signal_connect (GTK_OBJECT (entry), "activate",
-							  GTK_SIGNAL_FUNC (search_entry_cb), sess);
+	g_signal_connect (G_OBJECT (entry), "activate",
+							G_CALLBACK (search_entry_cb), sess);
 	gtk_container_add (GTK_CONTAINER (hbox), entry);
 	gtk_widget_show (entry);
 	gtk_widget_grab_focus (entry);
 
 	hbox = gtk_hbox_new (0, 10);
-	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
+	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 4);
 	gtk_widget_show (hbox);
 
-	wid = gtkutil_button (win, 0, _("Find"), search_find_cb, sess, hbox);
-	gtk_object_set_user_data (GTK_OBJECT (wid), entry);
-	gtkutil_button (win, 0, _("Clear"), search_clear_cb, entry, hbox);
-	gtkutil_button (win, 0, _("Close"), search_close_cb, win, hbox);
+	wid = gtkutil_button (hbox, GTK_STOCK_FIND, 0, search_find_cb, sess,
+								_("Find"), 0, 0);
+	g_object_set_data (G_OBJECT (wid), "e", entry);
+	gtkutil_button (hbox, GTK_STOCK_CLEAR, 0, search_clear_cb, entry,
+						_("Clear"), 0, 0);
+	gtkutil_button (hbox, GTK_STOCK_CLOSE, 0, search_close_cb, win,
+						_("Close"), 0, 0);
 
 	gtk_widget_show (win);
 }
