@@ -25,15 +25,35 @@
 #include <unistd.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkctree.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkhseparator.h>
+#include <gtk/gtkvseparator.h>
+#include <gtk/gtkpixmap.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkscrolledwindow.h>
+
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
 #include "../common/fe.h"
 #include "../common/server.h"
+#include "../common/util.h"
 #include "../common/xchatc.h"
 #include "serverlist.h"
 #include "gtkutil.h"
+#include "maingui.h"
 #include "pixmaps.h"
-#include "wins.h"
 
 
 static GtkWidget *slwin = 0;
@@ -185,8 +205,6 @@ add_defaults (void)
 		strcpy (serv.comment, dserv[i].comment);
 		serv.port = dserv[i].port;
 		serv.flags = 0;
-		if (serv.port == 9999)
-			serv.flags |= USE_SSL;
 		add_server_entry (&serv);
 		i++;
 	}
@@ -452,7 +470,6 @@ close_edit_entry (void)
 	editentry = 0;
 	editnode = 0;
 	new_entry = 0;
-
 	return FALSE;
 }
 
@@ -571,7 +588,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 		return;
 
 	sleditwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	wins_set_icon (sleditwin);
+	gtkutil_set_icon (sleditwin);
 	gtk_window_set_title (GTK_WINDOW (sleditwin), _("X-Chat: Edit entry"));
 	gtk_signal_connect (GTK_OBJECT (sleditwin), "delete_event",
 							  GTK_SIGNAL_FUNC (close_edit_entry), 0);
@@ -580,7 +597,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 																		editnode);
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (sleditwin), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (sleditwin), 5);
+	gtk_container_border_width (GTK_CONTAINER (sleditwin), 5);
 	gtk_widget_show (vbox);
 
 	hbox = gtk_hbox_new (TRUE, 0);
@@ -602,7 +619,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 	gtk_widget_show (frame);
 
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+	gtk_container_border_width (GTK_CONTAINER (hbox), 5);
 	gtk_container_add (GTK_CONTAINER (frame), hbox);
 	gtk_widget_show (hbox);
 
@@ -697,10 +714,8 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 	}
 
 	box1 = gtk_hbox_new (TRUE, 2);
-	gtkutil_button (slwin, GNOME_STOCK_BUTTON_OK, _("Ok"),
-						 done_edit_entry, 0, box1);
-	gtkutil_button (slwin, GNOME_STOCK_BUTTON_CANCEL, _("Cancel"),
-						 close_edit_entry, 0, box1);
+	gtkutil_button (box1, GTK_STOCK_OK, 0, done_edit_entry, 0, _("OK"), 0, 0);
+	gtkutil_button (box1, GTK_STOCK_CANCEL, 0, close_edit_entry, 0, _("Cancel"), 0, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, TRUE, 0);
 	gtk_widget_show (box1);
 
@@ -899,6 +914,8 @@ connect_auto (struct slentry *conn, struct session *sess)
 
 	strcpy (sess->willjoinchannel, conn->channel);
 	strcpy (serv->password, conn->password);
+	strip_spaces (sess->willjoinchannel);
+
 	if (conn->nick[0] == 0)
 		strcpy (serv->nick, prefs.nick1);
 	else
@@ -910,14 +927,14 @@ connect_auto (struct slentry *conn, struct session *sess)
 		(conn->flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
 #endif
 
+	server_connect (serv, conn->server, conn->port, FALSE);
+
 	if (conn->eom_cmd[0])
 	{
 		if (serv->eom_cmd)
 			free (serv->eom_cmd);
 		serv->eom_cmd = strdup (conn->eom_cmd);
 	}
-
-	connect_server (sess, conn->server, conn->port, FALSE);
 }
 
 /*
@@ -986,6 +1003,8 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 	close_server_list (FALSE);
 	strcpy (sess->server->password, myconn.password);
 	strcpy (sess->willjoinchannel, myconn.channel);
+	strip_spaces (sess->willjoinchannel);
+
 	if (myconn.nick[0] == 0)
 		strcpy (sess->server->nick, prefs.nick1);
 	else
@@ -1004,7 +1023,7 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 		serv->eom_cmd = strdup (myconn.eom_cmd);
 	}
 
-	connect_server (sess, myconn.server, myconn.port, FALSE);
+	server_connect (serv, myconn.server, myconn.port, FALSE);
 }
 
 /*
@@ -1045,6 +1064,7 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 	memcpy (&myconn, conn, sizeof (myconn));
 	strcpy (sess->server->password, conn->password);
 	strcpy (sess->willjoinchannel, conn->channel);
+	strip_spaces (sess->willjoinchannel);
 
 	close_server_list (FALSE);
 	serv = sess->server;
@@ -1072,7 +1092,7 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 		(myconn.flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
 #endif
 
-	connect_server (sess, myconn.server, myconn.port, FALSE);
+	server_connect (serv, myconn.server, myconn.port, FALSE);
 
 	if (myconn.eom_cmd[0])
 	{
@@ -1162,7 +1182,7 @@ serverlist_close_cb (GtkWidget *wid, GdkEvent *event, gpointer unused)
 	if (sess_list == NULL)
 		xchat_exit ();
 
-	return FALSE;
+	return 0;
 }
 
 /*
@@ -1191,10 +1211,9 @@ fe_open_serverlist (session *sess, int auto_connect, int dont_show)
 	gtk_signal_connect (GTK_OBJECT (slwin), "delete_event",
 							  GTK_SIGNAL_FUNC (serverlist_close_cb), 0);
 	gtk_window_set_policy (GTK_WINDOW (slwin), TRUE, TRUE, FALSE);
-	gtk_window_set_wmclass (GTK_WINDOW (slwin), "servlist", "X-Chat");
-	wins_set_icon (slwin);
+	gtkutil_set_icon (slwin);
 
-	gtk_container_set_border_width (GTK_CONTAINER (slwin), 5);
+	gtk_container_border_width (GTK_CONTAINER (slwin), 5);
 	vbox = gtk_vbox_new (FALSE, 1);
 	gtk_container_add (GTK_CONTAINER (slwin), vbox);
 
@@ -1237,7 +1256,7 @@ fe_open_serverlist (session *sess, int auto_connect, int dont_show)
 	gtk_entry_set_text (GTK_ENTRY (usernamegad), prefs.username);
 
 	/* X-CHAT ICON */
-	wid4 = gtk_pixmap_new (pix_xchat, mask_xchat);
+	wid4 = gtk_image_new_from_pixbuf (pix_xchat);
 	gtk_box_pack_start (GTK_BOX (wid3), wid4, FALSE, TRUE, 4);
 	gtk_widget_show (wid4);
 
@@ -1283,13 +1302,12 @@ fe_open_serverlist (session *sess, int auto_connect, int dont_show)
 	gtk_box_pack_end (GTK_BOX (wid2), wid1, FALSE, FALSE, 0);
 
 	/* FIRST "NORMAL" BUTTON */
-	button_conn =
-		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, _("Connect"),
-							 connect_clicked, sess, wid1);
+	button_conn = gtkutil_button (wid1, GTK_STOCK_REFRESH, 0, connect_clicked,
+											sess, _("Connect"), 0, 0);
 
-	button_connnew =
-		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, _("Connect New"),
-							 connectnew_clicked, sess, wid1);
+	button_connnew = gtkutil_button (wid1, GTK_STOCK_REFRESH,
+								_("Connect in a new window/tab"),
+							 connectnew_clicked, sess, _("Connect New"), 0, 0);
 
 	wid4 = gtk_vseparator_new ();
 	gtk_box_pack_start (GTK_BOX (wid1), wid4, 0, 0, 2);
@@ -1298,51 +1316,16 @@ fe_open_serverlist (session *sess, int auto_connect, int dont_show)
 	if (sess == NULL)
 		gtk_widget_hide (button_connnew);
 
-#ifdef USE_GNOME
-	/* VERY NICE GRAPHICAL BUTTON 1 */
-	wid4 = gtk_hbox_new (0, 0);
-	gtk_widget_show (wid4);
-	wid3 = gtk_button_new ();
-	gtk_signal_connect (GTK_OBJECT (wid3), "clicked",
-							  GTK_SIGNAL_FUNC (new_server_clicked), sess);
-	gtk_widget_show (wid3);
-	scroll = gtk_pixmap_new (pix_server, mask_server);
-	gtk_container_add (GTK_CONTAINER (wid4), scroll);
-	gtk_widget_show (scroll);
-	scroll = gtk_label_new (_("New Server"));
-	gtk_widget_show (scroll);
-	gtk_container_add (GTK_CONTAINER (wid4), scroll);
-	gtk_container_add (GTK_CONTAINER (wid3), wid4);
-	gtk_container_add (GTK_CONTAINER (wid1), wid3);
-
-	/* VERY NICE GRAPHICAL BUTTON 2 */
-	wid4 = gtk_hbox_new (0, 0);
-	gtk_widget_show (wid4);
-	wid3 = gtk_button_new ();
-	gtk_signal_connect (GTK_OBJECT (wid3), "clicked",
-							  GTK_SIGNAL_FUNC (new_group_clicked), sess);
-	gtk_widget_show (wid3);
-	scroll = gtk_pixmap_new (pix_globe, mask_globe);
-	gtk_container_add (GTK_CONTAINER (wid4), scroll);
-	gtk_widget_show (scroll);
-	scroll = gtk_label_new (_("New Group"));
-	gtk_widget_show (scroll);
-	gtk_container_add (GTK_CONTAINER (wid4), scroll);
-	gtk_container_add (GTK_CONTAINER (wid3), wid4);
-	gtk_container_add (GTK_CONTAINER (wid1), wid3);
-
-#else
-
-	gtkutil_button (slwin, 0, _("New Server"), new_server_clicked, 0, wid1);
-	gtkutil_button (slwin, 0, _("New Group"), new_group_clicked, 0, wid1);
-
-#endif
+	gtkutil_button (wid1, 0, 0, new_server_clicked, 0, _("New Server"),
+						 pix_server, mask_server);
+	gtkutil_button (wid1, 0, 0, new_group_clicked, 0, ("New Group"),
+						 pix_globe, mask_globe);
 
 	/* 2 MORE "NORMAL" BUTTONS */
-	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_CUT, _("Delete"),
-						 delete_server_clicked, 0, wid1);
-	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_PREFERENCES, _("Edit"),
-						 edit_entry_clicked, 0, wid1);
+	gtkutil_button (wid1, GTK_STOCK_DELETE, 0,
+						 delete_server_clicked, 0, _("Delete"), 0, 0);
+	gtkutil_button (wid1, GTK_STOCK_FIND_AND_REPLACE, 0,
+						 edit_entry_clicked, 0, _("Edit"), 0, 0);
 
 	gtk_widget_set_sensitive (button_conn, FALSE);
 	gtk_widget_set_sensitive (button_connnew, FALSE);
