@@ -34,7 +34,7 @@ static const struct defaultserv dserv[] =
 
 	{"", "SUB", "IRCNet", 0},
 	{"", "us.ircnet.org", "IRCNet USA", 6667},
-	{"", "random.ircd.de", "IRCNet Europe", 6667},
+	{"", "eu.ircnet.org", "IRCNet Europe", 6667},
 	{"", "irc.ircd.it", "IRCNet Italy", 6667},
 	{"", "au.ircnet.org", "IRCNet Australia", 6667},
 	{"", "irc.stealth.net", "irc.stealth.net", 6660},
@@ -46,17 +46,14 @@ static const struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "EFNet", 0},
-	{"", "irc.Prison.NET", "irc.Prison.NET", 6667},
-	{"", "irc.Qeast.net", "irc.Qeast.net", 6667},
-	{"", "irc.efnet.pl", "irc.efnet.pl", 6667},
-	{"", "irc.flamed.net", "irc.flamed.net", 6667},
-	{"", "efnet.demon.co.uk", "efnet.demon.co.uk", 6667},
-	{"", "irc.lagged.org", "irc.lagged.org", 6667},
-	{"", "irc.lightning.net", "irc.lightning.net", 6667},
-	{"", "irc.mindspring.com", "irc.mindspring.com", 6667},
-	{"", "irc.rt.ru", "irc.rt.ru", 6667},
-	{"", "irc.easynews.com", "irc.easynews.com", 6667},
-	{"", "irc.servercentral.net", "irc.servercentral.net", 6667},
+	{"", "us.rr.efnet.net", "EFNet USA", 6667},
+	{"", "ca.rr.efnet.net", "EFNet Canada", 6667},
+	{"", "eu.rr.efnet.net", "EFNet Europe", 6667},
+	{"", "au.rr.efnet.net", "EFNet Australia", 6667},
+	{"", "irc.efnet.org", "irc.efnet.org", 6667},
+	{"", "irc.light.se", "irc.light.se", 6667},
+	{"", "irc.stanford.edu", "irc.stanford.edu", 6667},
+	{"", "irc.solidstreaming.net", "irc.solidstreaming.net", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "GalaxyNet", 0},
@@ -130,7 +127,7 @@ static const struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "KewlNet", 0},
-	{"", "random.irc.kewl.org", "random.irc.kewl.org", 6667},
+	{"", "irc.kewl.org", "irc.kewl.org", 6667},
 	{"", "la.defense.fr.eu.kewl.org", "la.defense.fr.eu.kewl.org", 6667},
 	{"", "nanterre.fr.eu.kewl.org", "nanterre.fr.eu.kewl.org", 6667},
 	{"", "ENDSUB", "", 0},
@@ -328,8 +325,16 @@ static const struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "AzzurraNet", 0},
-	{"", "irc.azzurra.org", "irc.azzurra.org", 6667},
-	{"", "crypto.azzurra.org", "crypto.azzurra.org", 9999},
+	{"", "irc.bitchx.it", "irc.bitchx.it", 6667},
+	{"", "irc.jnet.it", "irc.jnet.it", 6667},
+	{"", "irc.net36.com", "irc.net36.com", 6667},
+	{"", "irc.noflyzone.net", "irc.noflyzone.net", 6667},
+	{"", "irc.swappoint.com", "irc.swappoint.com", 6667},
+	{"", "irc.azzurranet.org", "irc.azzurranet.org", 6667},
+	{"", "irc.leonet.it", "irc.leonet.it", 6667},
+	{"", "irc.libero.it", "irc.libero.it", 6667},
+	{"", "irc.estranet.it", "irc.estranet.it", 6667},  
+	{"", "irc.filmaker.it", "irc.filmaker.st", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "NixHelpNet", 0},
@@ -458,56 +463,11 @@ static const struct defaultserv dserv[] =
 	{"#cyberarmy", "irc.cyberarmy.com", "irc.cyberarmy.com", 6667},
 	{"", "ENDSUB", "", 0},
 
-	{"", "SUB", "2600net", 0},
-	{"", "irc.2600.net", "irc.2600.net - main server", 6667},
-	{"", "irc.dc2600.com", "irc.dc2600.com", 6667},
-	{"", "irc.nyc2600.com", "irc.nyc2600.com", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "RealmNET", 0},
-	{"", "irc.realmnet.com", "RealmNET: Random server", 6667},
-	{"", "irc.eu.realmnet.com", "RealmNET: Random server", 6667},
-	{"", "irc.us.realmnet.com", "RealmNET: Random US server", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "Freeworld IRC Network", 0},
-	{"", "kabel.freeworld.nu", "Pottstown, PA", 6667},
-	{"", "irc.freeworld.nu", "Reston, VA", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "hashmark", 0},
-	{"#hashmark", "irc.hashmark.net", "irc.hashmark.net", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "German-Elite-Network", 0},
-	{"", "dominion.german-elite.net", "dOminiOn.german-elite.net", 6667},
-	{"", "komatu.german-elite.net", "komatu.german-elite.net", 6667},
-	{"", "liberty.german-elite.net", "liberty.german-elite.net", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "Evolnet", 0},
-	{"", "irc.evolnet.org", "irc.evolnet.org", 6667},
-	{"", "bean.evolnet.org", "bean.evolnet.org", 6667},
-	{"", "blender.evolnet.org", "blender.evolnet.org", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "irc.oftc.net", 0},
-	{"", "irc.oftc.net", "irc.oftc.net", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "Xentonix.net", 0},
-	{"", "irc.ffm.de.eu.xentonix.net", "irc.ffm.de.eu.xentonix.net", 6667},
-	{"", "irc.brs.de.eu.xentonix.net", "irc.brs.de.eu.xentonix.net", 6667},
-	{"", "irc.stg.ch.eu.xentonix.net", "irc.stg.ch.eu.xentonix.net", 6667},
-	{"", "irc.hou.tx.us.xentonix.net", "irc.hou.tx.us.xentonix.net", 6667},
-	{"", "irc.kar.de.eu.xentonix.net", "irc.kar.de.eu.xentonix.net", 6667},
-	{"", "irc.vie.at.eu.xentonix.net", "irc.vie.at.eu.xentonix.net", 6667},
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "Spidernet.org", 0},
-	{"", "us.spidernet.org", "Random US server", 6667},
-	{"", "eu.spidernet.org", "Random European server", 6667},
-	{"", "irc.spidernet.org", "Random server", 6667},
+	{"", "SUB", "Bappy", 0},
+	{"", "irc.bappy.eu.org", "Random Bappy Server", 6667},
+	{"", "hub.uk.bappy.eu.org", "Bappy UK Hub", 6667},
+	{"", "hub.nl.bappy.eu.org", "Bappy NL Hub", 6667},
+	{"", "hub.se.bappy.eu.org", "Bappy SE Hub", 6667},
 	{"", "ENDSUB", "", 0},
 
 /*	{"", "SUB", "", 0},
