@@ -19,28 +19,34 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkclist.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkvpaned.h>
+
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
-#include "../common/plugin.h"
 #include "../common/outbound.h"
 #include "../common/text.h"
 #include "gtkutil.h"
 #include "xtext.h"
-#include "wins.h"
+#include "maingui.h"
 #include "palette.h"
 #include "textgui.h"
 
 extern struct text_event te[];
-extern char **pntevts_text;
-extern char **pntevts;
+extern char *pntevts_text[];
+extern char *pntevts[];
 
 static GtkWidget *pevent_dialog = NULL, *pevent_dialog_twid,
 	*pevent_dialog_entry, *pevent_dialog_sound_entry,
@@ -53,7 +59,7 @@ get_stamp_str (time_t tim, char *dest, int size)
 }
 
 static void
-PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
+PrintTextLine (xtext_buffer *xtbuf, unsigned char *text, int len, int indent)
 {
 	char *tab, *new_text;
 	int leftlen;
@@ -72,10 +78,10 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 			new_text = malloc (len + stamp_size + 1);
 			memcpy (new_text, buf, stamp_size);
 			memcpy (new_text + stamp_size, text, len);
-			gtk_xtext_append (GTK_XTEXT (textwidget), new_text, len + stamp_size);
+			gtk_xtext_append (xtbuf, new_text, len + stamp_size);
 			free (new_text);
 		} else
-			gtk_xtext_append (GTK_XTEXT (textwidget), text, len);
+			gtk_xtext_append (xtbuf, text, len);
 		return;
 	}
 
@@ -83,14 +89,14 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 	if (tab && (unsigned long) tab < (unsigned long) (text + len))
 	{
 		leftlen = (unsigned long) tab - (unsigned long) text;
-		gtk_xtext_append_indent (GTK_XTEXT (textwidget),
+		gtk_xtext_append_indent (xtbuf,
 										 text, leftlen, tab + 1, len - (leftlen + 1));
 	} else
-		gtk_xtext_append_indent (GTK_XTEXT (textwidget), 0, 0, text, len);
+		gtk_xtext_append_indent (xtbuf, 0, 0, text, len);
 }
 
 void
-PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
+PrintTextRaw (void *xtbuf, unsigned char *text, int indent)
 {
 	char *last_text = text;
 	int len = 0;
@@ -101,10 +107,10 @@ PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
 		switch (*text)
 		{
 		case 0:
-			PrintTextLine (textwidget, last_text, len, indent);
+			PrintTextLine (xtbuf, last_text, len, indent);
 			return;
 		case '\n':
-			PrintTextLine (textwidget, last_text, len, indent);
+			PrintTextLine (xtbuf, last_text, len, indent);
 			text++;
 			if (*text == 0)
 				return;
@@ -123,7 +129,7 @@ PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
 }
 
 static void
-pevent_dialog_close (gpointer * arg)
+pevent_dialog_close (GtkWidget *wid, gpointer arg)
 {
 	pevent_dialog = NULL;
 	pevent_save (NULL);
@@ -141,7 +147,7 @@ pevent_dialog_update_sound (GtkWidget * wid, GtkWidget * clist)
 	gtk_clist_set_text (GTK_CLIST (clist), row, 2,
 							  gtk_entry_get_text (GTK_ENTRY (wid)));
 
-	sig = GPOINTER_TO_INT (gtk_clist_get_row_data (GTK_CLIST (clist), row));
+	sig = GPOINTER_TO_INT(gtk_clist_get_row_data (GTK_CLIST (clist), row));
 
 	if (te[sig].sound)
 		free (te[sig].sound);
@@ -153,14 +159,15 @@ static void
 pevent_dialog_update (GtkWidget * wid, GtkWidget * twid)
 {
 	int row, len, m;
-	char *text, *out;
+	const char *text;
+	char *out;
 	int sig;
 
 	row = gtkutil_clist_selection (pevent_dialog_list);
 	if (row == -1)
 		return;
 
-	sig = (int) gtk_clist_get_row_data (GTK_CLIST (pevent_dialog_list), row);
+	sig = GPOINTER_TO_INT(gtk_clist_get_row_data (GTK_CLIST (pevent_dialog_list), row));
 
 	text = gtk_entry_get_text (GTK_ENTRY (wid));
 	len = strlen (text);
@@ -198,7 +205,7 @@ pevent_dialog_update (GtkWidget * wid, GtkWidget * twid)
 	out[len + 1] = 0;
 	check_special_chars (out, TRUE);
 
-	PrintTextRaw (twid, out, 0);
+	PrintTextRaw (GTK_XTEXT (twid)->buffer, out, 0);
 	free (out);
 }
 
@@ -218,9 +225,6 @@ pevent_dialog_hfill (GtkWidget * list, int e)
 	int i = 0;
 	char *text, buf[64];
 
-	if (!text_event (e))
-		return;
-
 	gtk_clist_clear (GTK_CLIST (list));
 	while (i < te[e].num_args)
 	{
@@ -249,7 +253,7 @@ pevent_dialog_select (GtkWidget * clist, gint row, gint column,
 		gtk_entry_set_text (GTK_ENTRY (pevent_dialog_entry), cmd);
 		gtk_clist_get_text (GTK_CLIST (clist), row, 2, &snd);
 		gtk_entry_set_text (GTK_ENTRY (pevent_dialog_sound_entry), snd);
-		sig = GPOINTER_TO_INT (gtk_clist_get_row_data (GTK_CLIST (clist), row));
+		sig = GPOINTER_TO_INT(gtk_clist_get_row_data (GTK_CLIST (clist), row));
 		pevent_dialog_hfill (pevent_dialog_hlist, sig);
 	} else
 	{
@@ -267,8 +271,6 @@ pevent_dialog_fill (GtkWidget * list)
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		if (!text_event (i))
-			continue;
 		nnew[0] = te[i].name;
 		nnew[1] = pntevts_text[i];
 		if (te[i].sound)
@@ -276,7 +278,7 @@ pevent_dialog_fill (GtkWidget * list)
 		else
 			nnew[2] = "";
 		row = gtk_clist_append (GTK_CLIST (list), nnew);
-		gtk_clist_set_row_data (GTK_CLIST (list), row, GINT_TO_POINTER (i));
+		gtk_clist_set_row_data (GTK_CLIST (list), row, GINT_TO_POINTER(i));
 	}
 }
 
@@ -336,8 +338,6 @@ pevent_test_cb (GtkWidget * wid, GtkWidget * twid)
 
 	for (n = 0; n < NUM_XP; n++)
 	{
-		if (!text_event (n))
-			continue;
 		text = _(pntevts_text[n]);
 		len = strlen (text);
 
@@ -347,7 +347,7 @@ pevent_test_cb (GtkWidget * wid, GtkWidget * twid)
 		out[len + 1] = 0;
 		check_special_chars (out, TRUE);
 
-		PrintTextRaw (twid, out, 0);
+		PrintTextRaw (GTK_XTEXT (twid)->buffer, out, 0);
 		free (out);
 	}
 }
@@ -374,28 +374,21 @@ void
 pevent_dialog_show ()
 {
 	GtkWidget *vbox, *vbox2, *hbox, *tbox, *wid, *bh, *th;
-	gchar *titles[3];
-	gchar *help_titles[2];
-
-	titles[0] = _("Event");
-	titles[1] = _("Text");
-	titles[2] = _("Sound");
-
-	help_titles[0] = _("$ Number");
-	help_titles[1] = _("Description");
+	gchar *titles[] = { _("Event"), _("Text"), _("Sound") };
+	gchar *help_titles[] = { _("$ Number"), _("Description") };
 
 	if (pevent_dialog)
 	{
-		wins_bring_tofront (pevent_dialog);
+		mg_bring_tofront (pevent_dialog);
 		return;
 	}
 
-	pevent_dialog = maingui_window ("edit events", _("Edit Events"),
-											  TRUE, FALSE, pevent_dialog_close, NULL,
-											  600, 455, NULL);
+	pevent_dialog =
+			  mg_create_generic_tab ("edit events", _("Edit Events"),
+											 TRUE, FALSE, pevent_dialog_close, NULL,
+											 600, 455, &vbox);
 
 	vbox2 = gtk_vbox_new (0, 2);
-	vbox = wins_get_vbox (pevent_dialog);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
 
 	wid = gtk_vpaned_new ();
@@ -414,10 +407,8 @@ pevent_dialog_show ()
 	gtk_clist_set_column_width (GTK_CLIST (pevent_dialog_list), 0, 80);
 	gtk_clist_set_column_width (GTK_CLIST (pevent_dialog_list), 1, 380);
 
-	pevent_dialog_twid = gtk_xtext_new (0, 0);
-
-	gtk_xtext_set_palette (GTK_XTEXT (pevent_dialog_twid), colors);
-	gtk_xtext_set_font (GTK_XTEXT (pevent_dialog_twid), font_normal);
+	pevent_dialog_twid = gtk_xtext_new (colors, 0);
+	gtk_xtext_set_font (GTK_XTEXT (pevent_dialog_twid), prefs.font_normal);
 	gtk_xtext_set_background (GTK_XTEXT (pevent_dialog_twid),
 									  channelwin_pix, prefs.transparent, prefs.tint);
 
@@ -480,11 +471,8 @@ pevent_dialog_show ()
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_test_cb), pevent_dialog_twid);
 	gtk_widget_show (wid);
-#ifdef USE_GNOME
-	wid = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
-#else
-	wid = gtk_button_new_with_label (_("Ok"));
-#endif
+
+	wid = gtk_button_new_from_stock (GTK_STOCK_OK);
 	gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_ok_cb), NULL);
