@@ -26,13 +26,19 @@
 #include <stdlib.h>
 
 #include "fe-gtk.h"
+
+#include <gtk/gtkclist.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkhbbox.h>
+
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
 #include "../common/util.h"
 #include "../common/xchatc.h"
 #include "gtkutil.h"
 #include "menu.h"
-#include "wins.h"
+#include "maingui.h"
 #include "urlgrab.h"
 
 
@@ -43,7 +49,7 @@ static void url_addurlgui (char *urltext);
 
 
 static void
-url_closegui (gpointer userdata)
+url_closegui (GtkWidget *wid, gpointer userdata)
 {
 	urlgrabberwindow = 0;
 }
@@ -135,7 +141,6 @@ url_clicklist (GtkWidget * widget, GdkEventButton * event, gpointer userdata)
 			}
 		}
 	}
-
 	return FALSE;
 }
 
@@ -147,14 +152,13 @@ url_opengui ()
 
 	if (urlgrabberwindow)
 	{
-		wins_bring_tofront (urlgrabberwindow);
+		mg_bring_tofront (urlgrabberwindow);
 		return;
 	}
 
 	urlgrabberwindow =
-		maingui_window ("urlgrabber", _("X-Chat: URL Grabber"), FALSE, TRUE,
-							 url_closegui, NULL, 350, 100, &urlgrabberwindow);
-	vbox = wins_get_vbox (urlgrabberwindow);
+		mg_create_generic_tab ("urlgrabber", _("X-Chat: URL Grabber"), FALSE,
+							 TRUE, url_closegui, NULL, 350, 100, &vbox);
 
 	urlgrabberlist = gtkutil_clist_new (1, 0, vbox, GTK_POLICY_AUTOMATIC,
 													0, 0, 0, 0, GTK_SELECTION_BROWSE);
@@ -163,14 +167,15 @@ url_opengui ()
 	gtk_widget_set_usize (urlgrabberlist, 350, 0);
 	gtk_clist_set_column_width (GTK_CLIST (urlgrabberlist), 0, 100);
 
-	hbox = gtk_hbox_new (FALSE, 1);
-	gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
+	hbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_SPREAD);
+	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
-	gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_CLEAR,
-						 _("Clear"), url_button_clear, 0, hbox);
-	gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_SAVE,
-						 _("Save"), url_button_save, 0, hbox);
+	gtkutil_button (hbox, GTK_STOCK_CLEAR,
+						 0, url_button_clear, 0, _("Clear"), 0, 0);
+	gtkutil_button (hbox, GTK_STOCK_SAVE,
+						 0, url_button_save, 0, _("Save"), 0, 0);
 
 	gtk_widget_show (urlgrabberwindow);
 
