@@ -44,7 +44,7 @@ static int done = FALSE;		  /* finished ? */
 static void
 send_command (char *cmd)
 {
-	handle_command (cmd, sess_list->data, TRUE, FALSE);
+	handle_multiline (sess_list->data, cmd, TRUE, FALSE);
 }
 
 static void
@@ -131,7 +131,7 @@ fe_new_window (struct session *sess)
 #ifdef SOCKS
 	"Socks5 "
 #endif
-#ifdef HAVE_ICONV
+#ifdef USE_JCODE
 	"JCode "
 #endif
 #ifdef USE_IPV6
@@ -442,7 +442,7 @@ fe_main (void)
 	guint64 shortest, delay;
 
 #ifdef ENABLE_NLS
-	bindtextdomain (PACKAGE, LOCALEDIR);
+	bindtextdomain (PACKAGE, PREFIX"/share/locale");
 	textdomain (PACKAGE);
 #endif
 
@@ -648,16 +648,17 @@ fe_progressbar_start (struct session *sess)
 {
 }
 void
-fe_progressbar_end (struct session *sess)
+fe_progressbar_end (struct server *serv)
 {
 }
 void
-fe_userlist_insert (struct session *sess, struct User *newuser, int row)
+fe_userlist_insert (struct session *sess, struct User *newuser, int row, int sel)
 {
 }
-void
+int
 fe_userlist_remove (struct session *sess, struct User *user)
 {
+	return 0;
 }
 void
 fe_userlist_move (struct session *sess, struct User *user, int new_row)
@@ -759,17 +760,6 @@ void
 fe_dcc_open_chat_win (int passive)
 {
 }
-int
-fe_is_confmode (struct session *sess)
-{
-	return 0;
-}
-int
-fe_is_beep (struct session *sess)
-{
-	return 0;
-}
-
 void
 fe_userlist_hide (session * sess)
 {
